function [mu, sigma, err_mu, err_sig, V] = EstBS1dExp( data, h, significanceLevel )
% This function estimates explicitely the parameters of a
% univariate geometric Brownian motion.
% 
% Input    
%           data: prices;
%           h: time scale; for daily data, h=1/252 (default);
%           significanceLevel (95% default).
%
% Output    mu: estimation of the average return per annum; 
%           sigma: estimation of the volatility  per annum;
%           err_mu: significanceLevel% CI error for mu;
%           err_sig: significanceLevel% CI error for sigma;
%           V: estimation of covariance matrix.
 
if nargin < 2
    significanceLevel = 0.95;
    h = 1/252;
elseif nargin < 3
    significanceLevel = 0.95;
end

%% plot prices
figure
subplot(2,1,1)
plot(data);
title('Adjusted prices');

%% compute and plot log-returns
X = diff( log( data ), 1 );
 
subplot(2,1,2)
plot(X);
title('Adjusted log-returns');

%% Estimation of parameters

mu0      = mean( X );

sigma   = std( X )/sqrt(h);

var     = sigma^2; 

mu      = mu0/h + 0.5 * var;

n       = length( X );

V       = zeros( 2, 2) ;

V(2,2)  = 0.5 * var;

V(1,1)  = V(2,2) * ( 2/h + var );
V(1,2)  = V(2,2) * sigma;
V(2,1)  = V(1,2);

criticalValue = norminv(0.5 * (1+significanceLevel) );

err_sig = criticalValue * sqrt( V(2,2) / n );

err_mu  = criticalValue * sqrt( V(1,1) / n );

%% Print output
fprintf('\n\n');
fprintf('Estimation of mu (annual)   : %2.4f, estimation error(%d%%)  = %2.4f\n', mu   , 100*significanceLevel, err_mu )
fprintf('Estimation of sigma (annual): %2.4f, estimation error(%d%%)  = %2.4f\n', sigma, 100*significanceLevel, err_sig )
 

