function [mu, sigma, err_mu, err_sig, V] = EstBS1dNum( data, method, h, significanceLevel )
% This function estimates the parameters of the one-dimensional Black-Scholes model 
% using a numerical method to maximize the likelihood. Fisher information
% can be estimated with the Hessian matrix (fminunc output), with the exact
% gradients, or with the numerical gradients. See Appendix B.5.1 for more
% details.
% 
% Input    
%           data: prices (column vector);
%           method: 'Hessian' (default), 'exact', 'num';
%           h: time scale; for daily data, h=1/252 (default);
%           significanceLevel (95% default).
%
% Output:   mu: estimation of the average return per annum;
%           sigma: estimation of the volatility  per annum;
%           err_mu: significanceLevel% CI error for mu;
%           err_sig: significanceLevel% CI error for sigma;
%           V: estimation of covariance matrix.
% Example: 
%  [mu, sigma, err_mu, err_sig, V] = EstBS1dNum(ApplePrices);


if nargin < 2
    method = 'Hessian';
    significanceLevel = 0.95;
    h = 1/252;
elseif nargin < 3
    significanceLevel = 0.95;
    h = 1/252;
elseif nargin < 4
    significanceLevel = 0.95;
end

%% compute and plot log-returns
returns = diff( log( data ), 1 );
n       = length( returns );
 
%% PRE-OPTIMIZATION
mu0=0;
sigma0=1; %do not start with a too small value; we are working on the annual time scale for the likelihood
initialGuess    = [mu0;log(sigma0)]; % Initial values of mu and log(sigma)



options         = optimset('largescale','on','MaxFunEvals',5000,'MaxIter',2000,'Display','iter');

%% OPTIMIZATION
[theta, f, exitflag, output, grad, hessian] = fminunc( @(x) sum(LogLikBS1D( x, returns,h)), initialGuess, options );


% remove persistent variables
clear functions

%% OUTPUT

mu     = theta(1);

sigma  = exp( theta(2));


 addpath('../ChapterB')
%%  Fisher information matrix 
fprintf('\n\n');

lmin = min(eig(hessian));

if(lmin<0)
 fprintf('\n The Hessian is not positive definite. Estimation errors are not reliable. The exact method will be used instead\n\n');
 method = 'exact';
end

switch method
    case 'Hessian'
        FI = hessian/n;
fprintf('\n Fisher information computed with the numerical Hessian from fminunc (Appendix B.5.1)\n\n');


    case 'exact'
        [LL numGrad] = LogLikBS1D(theta,returns,h);
         fprintf('Fisher information computed with the exact gradient (Appendix B.5.1)\n\n');
         FI = cov(numGrad);
    case 'num'
        numGrad = NumJacobian(@(x) LogLikBS1D(x,returns,h),theta,0.001);
        fprintf('Fisher information computed with the numerical gradient (Appendix B.5.1)\n\n');
        FI = cov(numGrad);
end

rmpath('../ChapterB')

%% Corrections
J = diag([1;sigma]); %Jacobian matrix of the transformation for positiveness


V = J*pinv(FI)*J;


%% Precision

criticalValue = norminv( 0.5 *(1+significanceLevel) );

err_sig = criticalValue * sqrt( V(2,2) / n );

err_mu  = criticalValue * sqrt( V(1,1) / n );

%% Print output
fprintf('Estimation of mu    : %.4f, estimation error(%3d%%)  = %.4f\n', mu, 100*significanceLevel, err_mu )
fprintf('Estimation of sigma : %.4f, estimation error(%3d%%)  = %.4f\n', sigma, 100*significanceLevel, err_sig )



end

function [LL,grad] = LogLikBS1D( theta, returns, h)
% This function computes minus the log-likelihoods for the 
% univariate Black-Scholes model under the annual time scale
%
% Input
%        theta = (mu, log(sigma))
%        returns
%        n (length of returns )
%
% Output
%        LL: -log-likelihood
%        grad: gradient of LL
%


    logLikelihoodFactor = 0.5 * log( 2 * pi );


%  standard deviation of returns
sigma = exp( theta(2) )*sqrt( h );         

%  mean of returns
drift = theta(1) * h - 0.5 * sigma^2; 

z   = ( returns - drift ) / sigma;

LL = logLikelihoodFactor + log( sigma ) + 0.5 * z.^2;
grad =  -[z/sigma (z.^2-1-sigma*z)];
end