function out = OptionBS(S,option,param,r,tau,sigma,yield)
% Evaluation of options based on calls and puts using Black-Scholes formula
%
% Input 
%
%  S: Current price of the underlying asset;
%  option: 1=call,2=put,3=bear,4=bull,5=straddle,6=strangle,7=butterfly;
%  param: vector of strike prices for the option; 
%    1 strike price for call, put, and straddle options; 
%    2 strike prices for bear, bull, and strangle options;
%    3 strike prices for butterfly options;
%  r: (continuous) short-time interest rate for the period;
%  tau: time to maturity;
%  sigma: volatility in the B-S model;
%  yield: continuous dividend rate.
%
% N.B. All parameters can be vectors. If there is more than one, then they
% must have the same dimension.
%
% Output
%        val: value of the option;
%        delta: delta of the option;
%        gamma: gamma of the option;
%        vega: vega of the option.
%
% N.B.: The time and rate must be on the same scale as the one used for
%       estimating sigma. For example, if sigma is estimated with daily
%       prices, then tau should be in days, and r should be the interest
%       rate for 1 day.

if nargin < 7
    yield = 0;
end

optionnames = {'Call','Put','Bear','Bull','Straddle','Stangle','Butterfly'};

[Call1, Put1]= FormulaBS(S,param(1),r,tau,sigma,yield);
[Call_D1, Put_D1, Gamma1, Vega1] = FormulaBSGreeks(S,param(1),r,tau,sigma,yield);
     
     
      
      switch option
          case 1
            C_BS = Call1;
            phi_BS = Call_D1;
            gamma_BS = Gamma1;
            Vega_BS  = Vega1;
      

          case 2
            C_BS = Put1;
            phi_BS = Put_D1;
            gamma_BS = Gamma1;
            Vega_BS  = Vega1;

        
          case 3
            [Call2, Put2]     = FormulaBS(S,param(2),r,tau,sigma,yield);
            [Call_D2, Put_D2,Gamma2, Vega2] = FormulaBSGreeks(S,param(2),r,tau,sigma,yield);
            
            C_BS = Put2-Put1;
            phi_BS = Put_D2-Put_D1;
            gamma_BS = -Gamma1+Gamma2;
            Vega_BS  = -Vega1+Vega2;
             
          case 4
            
             [Call2, Put2]     = FormulaBS(S,param(2),r,tau,sigma,yield);
             [Call_D2, Put_D2,Gamma2, Vega2] = FormulaBSGreeks(S,param(2),r,tau,sigma,yield);
             
             C_BS = Call1-Call2;
             phi_BS = Call_D1-Call_D2;
             gamma_BS = Gamma1-Gamma2;
             Vega_BS  = Vega1-Vega2;
             
             
          case 5
            C_BS = Call1+Put1;
            phi_BS = Call_D1+Put_D1;
            gamma_BS = 2*Gamma1;
            Vega_BS  =  2*Vega1;
            
          case 6
            [Call2, Put2]     = FormulaBS(S,param(2),r,tau,sigma,yield);
            [Call_D2, Put_D2,Gamma2, Vega2] = FormulaBSGreeks(S,param(2),r,tau,sigma,yield);
            
            C_BS = Call2+Put1;
            phi_BS =  Call_D2+Put_D1;
            gamma_BS = Gamma1+Gamma2;
            Vega_BS  = Vega1+Vega2;
            
          case 7
             [Call2, Put2]     = FormulaBS(S,param(2),r,tau,sigma,yield);
             [Call_D2, Put_D2,Gamma2, Vega2] = FormulaBSGreeks(S,param(2),r,tau,sigma,yield);
             [Call3, Put3]     = FormulaBS(S,param(3),r,tau,sigma,yield);
             [Call_D3, Put_D3,Gamma3, Vega3] = FormulaBSGreeks(S,param(3),r,tau,sigma,yield);
             
             C_BS = Call1-2*Call2+Call3;
             phi_BS = Call_D1-2*Call_D2+Call_D3;
             gamma_BS = Gamma1-2*Gamma2+Gamma3;
             Vega_BS = Vega1-2*Vega2+Vega3;
            

      end   
      
       
       out.val   = C_BS;
       out.delta = phi_BS;
       out.gamma = gamma_BS;
       out.vega  = Vega_BS;
       out.name  = optionnames{option};



