function [S,t] = SimBS1d(mu, sigma, s0, h, n, m,graph)
% This function generates m paths of a geometric Brownian
% motion observed at n+1 points, with parameters mu and 
% sigma, and initial value s0. The time between observations is h.
%
% Input
%     mu, sigma: Mean and volatility parameters of the geometric 
%                Brownian motion;
%     s0: Starting value;
%     h : Time between observations;
%     n : Number of observations points;
%     m : Number of trajectories (default 1);
%     graph: 1 if one wants the graph of trajectories, 0 otherwise
%            (default).
%
% Output
%     S: simulated series;
%     t: observation times.
%
%
% Example:  
% [S,t] = SimBS1d(0.05,0.1,100,1/252,252,10,1);




if nargin <6
    m = 1;
    graph = 0;
end

if nargin <7
    graph = 0;
end

% time vector
t = [0; h * (1:1:n)'];

% simulated prices
S = s0 * exp( [zeros(1, m); cumsum( ( mu - 0.5 * sigma^2 ) * h +...
    sigma * sqrt( h ) * randn( n, m ) )] );


if(graph)
plot( t, S )
title({[num2str( m ) ,' paths of a Geometric Brownian motion ']; ...
    ['with \mu =', num2str(mu) ,' and \sigma = ', num2str(sigma) ,...
    ' starting at ', num2str(s0)]});
end