function [mu, err_mu, sigma, err_sig, rho, err_rho] = EstBS2dExp(data, names, h, significanceLevel )
% This function estimates explicitly the parameters of bivariate geometric 
% Brownian motions.
% 
% Input:    data: prices ( n x 2);
%           names (list of assets) e.g. names= {'Apple';'Microsoft'};
%           h: time scale; for daily data, h=1/252 (default);
%           significanceLevel (5% default);
%
% Output:   mu: estimation of mu per annum; 
%           sigma: estimation of sigma per annum;
%           rho: estimation of rho;
%           err_mu: significanceLevel% CI error for mu;
%           err_sig: significanceLevel% CI error for sigma;
%           err_rho: significanceLevel% CI error for rho.
%
% Example: 
%  [mu, err_mu, sigma, err_sig, rho, err_rho] = EstBS2dExp([ApplePrices MicrosoftPrices],{'Apple';'Microsoft'});

if nargin < 2
    significanceLevel = 0.95;
    names = {'Asset 1';'Asset 2'};
    h = 1/252;
elseif isempty( names )
    names = {'Asset 1';'Asset 2'};
elseif nargin < 3
    h = 1/252;
    significanceLevel = 0.95;
elseif nargin < 4
     significanceLevel = 0.95;
end

%Normalized prices
[n,d] = size(data);

data1 = 100 * data ./ repmat( data(1,:), n, 1);

%% plot prices
figure
subplot(2,1,1)
plot(data1);
title('Normalized adjusted prices (both starting at 100$)');
legend( names, 'location', 'best' );

%% compute and plot log-returns
X = diff( log( data1 ), 1 );
 
subplot(2,1,2)
plot(X);
title('Adjusted log-returns');

%% Estimation of parameters

mu0 = mean(X);

sigma = std(X)/sqrt(h);

var =  sigma .^ 2;

rho = corr( X(:,1), X(:,2) );

mu =  mu0/h + 0.5 * var;



V22  = 0.5 * var;

V11  = V22 .* ( 2/h + var );

criticalValue = norminv( 0.5 *(1 + significanceLevel ) );


err_rho = criticalValue *(1-rho^2)/sqrt(n);
err_sig = criticalValue * sqrt( V22 / n );

err_mu  = criticalValue * sqrt( V11 / n );

%% Print output

fprintf('\n\n Estimation of mu per annum: \n\n')
fprintf('%20s : %.4f  +/- %.4f \n', names{1}, mu(1), err_mu(1)) 
fprintf('%20s : %.4f  +/- %.4f \n', names{2}, mu(2), err_mu(2))

fprintf('\n\n Estimation of sigma per annum: \n\n')
fprintf('%20s : %.4f  +/- %.4f \n', names{1}, sigma(1), err_sig(1)) 
fprintf('%20s : %.4f  +/- %.4f \n', names{2}, sigma(2), err_sig(2))


fprintf('\n\n Estimation of rho   : %.4f  +/- %.4f \n\n',rho, err_rho)

theta = 0.5*log((1+rho)/(1-rho));
rho1 = tanh( theta-criticalValue/sqrt(n));
rho2 = tanh( theta+criticalValue/sqrt(n));
fprintf('\n\n A confidence interval for rho based on the Fisher transformation is ')
fprintf('\n\n    rho in [%.4f, %.4f] \n\n',rho1,rho2)

