function [mu, sigma, rho, err_mu, err_sig, err_rho, V] = EstBS2dNumCor(data, names, method, h, significanceLevel )
% This function estimates the parameters of a bivariate geometric Brownian
% motion by numerical methods for the parametrization with the correlation 
% coefficient. Fisher information can be estimated with the Hessian matrix 
% (fminunc output) or with the numerical gradients. See Appendix B.5.1 for 
% more details.
% 
% Input    
%           data: bivariate prices;
%           names: list of assets, e.g. names= {'Apple';'Microsoft'};
%           method: 'Hessian' (default), 'num';
%           h: time scale; for daily data, h=1/252 (default);
%           significanceLevel (5% default).
%
% Output:   mu: estimation of mu per annum; 
%           sigma: estimation of sigma per annum;
%           rho : estimation of rho;
%           err_mu: significanceLevel% CI error for mu;
%           err_sig: significanceLevel% CI error for sigma;
%           err_rho: significanceLevel% CI error for rho;
%           V: covariance matrix of estimation errors.
% 
% Example:
% [mu, sigma, rho, err_mu, err_sig, err_rho, V] =
% EstBS2dNumCor([ApplePrices MicrosoftPrices], {'Apple';'Microsoft'});

if nargin < 2
    names = {'Asset 1';'Asset 2'};
    method = 'Hessian';
    h = 1/252;
    significanceLevel = 0.95;
elseif isempty( names )
    names = {'Asset 1';'Asset 2'};
elseif nargin < 3
    method = 'Hessian';
    h = 1/252;
    significanceLevel = 0.95;
    elseif nargin < 4
    h = 1/252;
    significanceLevel = 0.95;

elseif nargin < 5
    significanceLevel = 0.95;
 
end

%Normalized prices
[n,d] = size(data);

data1 = 100 * data ./ repmat( data(1,:), n, 1);

%% plot prices
figure
subplot(2,1,1)
plot(data1);
title('Normalized adjusted prices (both starting at 100$)');
legend( names, 'Location', 'Best')

%% compute and plot log-returns
returns = diff( log( data ), 1 );

subplot(2,1,2)
plot(returns);
title('Log-returns');
legend( names, 'Location', 'Best')

%% Estimation of parameters

initialGuess = [    0; ... % mu1
                    0; ... % mu2
                    log(1); ... % log(vol1)
                    log(1); ... % log(vol2)
                    0];    % arctanh(rho)

% initialGuess = [0;0;log(.001);log(.001);0];
               
                
options = optimset( 'largescale',   'on', ...
                    'MaxFunEvals',  1000, ...
                    'MaxIter',      200, ...
                    'Display',      'iter');
 
[theta, f, exitflag, output, grad, hessian] = fminunc( @(x) sum(LogLikBS2DCor(x, returns, h)), initialGuess, options ); 

%% Reparametrization and computation of estimation errors covariance matrix
mu(1)       = theta(1);
mu(2)       = theta(2);
sigma(1)    = exp(theta(3));
sigma(2)    = exp(theta(4));
rho         = tanh(theta(5));

% number of returns;
n = n-1;

sv = sech( theta(5) ); % derivatives of tanh; needed for Slutzky formula

lmin = min(eig(hessian));
if(lmin<0)
 fprintf('\n The Hessian is not positive definite. Estimation errors are not reliable. The numerical method will be used instead\n\n');
 method = 'num';
end

 addpath('../ChapterB')

%%  Fisher information matrix 
fprintf('\n\n');
switch method
    case 'Hessian'
        FI = hessian/n;
fprintf('\n Fisher information computed with the numerical Hessian from fminunc (Appendix B.5.1)\n\n');
    case 'num'
        numGrad = NumJacobian(@(x) LogLikBS2DCor(x,returns,h),theta,0.001);
        fprintf('Fisher information computed with the numerical gradient (Appendix B.5.1)\n\n');
        FI = cov(numGrad);
end

rmpath('../ChapterB')

J = diag( [ 1, 1, sigma(1), sigma(2), sv^2 ] ); %Jacobian matrix of the transformation 



V = J*pinv(FI)*J;


criticalValue   = norminv( 0.5 * ( 1 + significanceLevel ) );
err             = criticalValue * sqrt(diag(V)/n);
err_mu          = err(1:2);
err_sig         = err(3:4);
err_rho         = err(5);

%% Printed output
fprintf('\n\n %3d%% Confidence Intervals: \n\n',100*significanceLevel )
fprintf('\n\n Estimation of mu: \n\n')
fprintf('%20s : %.4f  +/- %.4f \n', names{1}, mu(1), err_mu(1)) 
fprintf('%20s : %.4f  +/- %.4f \n', names{2}, mu(2), err_mu(2))

fprintf('\n\n Estimation of sigma: \n\n')
fprintf('%20s : %.4f  +/- %.4f \n', names{1}, sigma(1), err_sig(1)) 
fprintf('%20s : %.4f  +/- %.4f \n', names{2}, sigma(2), err_sig(2))


fprintf('\n\n Estimation of rho   : %.4f  +/- %.4f \n\n',rho, err_rho)

end

function LL = LogLikBS2DCor( theta, X , h)
% This function computes minus the log-likelihhods for a bivariate 
% Black-Scholes model for the parametrisation with rho on an annual time
% scale
%
% input: theta = (mus, log(sigmas),atanh(rho))
%        X ( n x 2 returns).
%        
% output: -log-likelihoods (LL)

logLikelihoodFactor = 0.5 * log( 2 * pi );

% standard deviations of the returns
v = exp( theta(3:4) )*sqrt( h );


% mean of the returns
mu = theta(1:2) * h - 0.5 * v .^ 2;

rho = tanh( theta(5) );  %correlation

z = zeros( size(X) ); %standardized values
z(:,1)  = (X(:,1) - mu(1)) / v(1);
z(:,2)  = (X(:,2) - mu(2)) / v(2); 

LL = ( sum(z.^2,2) - 2 * rho * prod( z, 2) ) / (2 * (1-rho^2) ) + log( 2 * pi * prod( v ) ) + 0.5 * log( 1-rho^2 );
end