function [mu, sigma, a, vol, err_mu, err_sig, err_vol, V] = EstBS2dNumVol( data, names, method, h, significanceLevel )
% This function estimates the parameters of a bivariate geometric Brownian
% motion by numerical methods for the parametrization with the volatility 
% vector, under an annual time scale. Fisher information can be estimated 
% with the Hessian matrix fminunc output) or with the numerical gradients. 
% See Appendix B.5.1 for more details.
% 
% Input    
%           data: bivariate prices;
%           names: list of assets, e.g. names= {'Apple';'Microsoft'};
%           method: 'Hessian' (default), 'num';
%           h: time scale; for daily data, h=1/252 (default);
%           significanceLevel (5% default).
%
% Output:   mu: estimation of mu per annum; 
%           sigma: estimation of sigma per annum;
%           vol : estimation of the volatility vector v;
%           err_mu: significanceLevel% CI error for mu;
%           err_sig: significanceLevel% CI error for sigma;
%           err_vol: significanceLevel% CI error for v;
%           V: covariance matrix of the estimation errors.
% 
% Example:
% [mu, sigma, a, vol, err_mu, err_sig, err_vol, V] =
% EstBS2dNumVol([ApplePrices MicrosoftPrices], {'Apple';'Microsoft'});

if nargin < 2
    names = {'Asset 1';'Asset 2'};
    method = 'Hessian';
    h = 1/252;
    significanceLevel = 0.95;
elseif isempty( names )
    names = {'Asset 1';'Asset 2'};
    
elseif nargin < 3
    method = 'Hessian';
    h = 1/252;
    significanceLevel = 0.95;
    
elseif nargin < 4
    h = 1/252;
    significanceLevel = 0.95;

elseif nargin < 5
    significanceLevel = 0.95;
 
end 

 

%Normalized prices
[n,d] = size(data);

data1 = 100 * data ./ repmat( data(1,:), n, 1);

%% plot prices
figure
subplot(2,1,1)
plot(data1);
title('Normalized adjusted prices (both starting at 100$)');
legend( names, 'Location', 'Best')

%% compute and plot log-returns
returns = diff( log( data1 ), 1 );
 
subplot(2,1,2)
plot(returns);
title('Adjusted log-returns');
legend( names, 'Location', 'Best')

%% Estimation of parameters

initialGuess = [    0; ...      % mu1
                    0; ...      % mu2
                    log(1); ... % log( a(1,1) )
                    0; ...      % a(1,2)
                    log(1)];    % log( a(2,2) )

                
options = optimset( 'largescale',   'on', ...
                    'MaxFunEvals',  1000, ...
                    'MaxIter',      200, ...
                    'Display',      'iter');
                
 
[theta, f, exitflag, output, grad, hessian] = fminunc(@(x) sum(LogLikBS2DVol(x, returns, h)), initialGuess, options); 


%% Reparametrization and computation of estimation errors covariance matrix
mu(1) = theta(1);
mu(2) = theta(2);

a(1,1) = exp(theta(3));  %%%% Pour respecter l'ordre de dfinition de theta (voir notes de cours)
a(1,2) = theta(4);
a(2,2) = exp(theta(5));  
a(2,1) = 0 ; 

vol = [ a(1,1); a(1,2); a(2,2)];

n = n-1; % number of returns;

lmin = min(eig(hessian));
if(lmin<0)
 fprintf('\n The Hessian is not positive definite. Estimation errors are not reliable. The numerical method will be used instead\n\n');
 method = 'num';
end

 addpath('../ChapterB')
%  Fisher information matrix 
switch method
    case 'Hessian'
        FI = hessian/n;
fprintf('\n Fisher information computed with the numerical Hessian from fminunc (Appendix B.5.1)\n\n');
    case 'num'
        numGrad = NumJacobian(@(x) LogLikBS2DVol(x,returns,h),theta,0.001);
        fprintf('Fisher information computed with the numerical gradient (Appendix B.5.1)\n\n');
        FI = cov(numGrad);
end


 rmpath('../ChapterB')

J = diag([1,1,a(1,1),1,a(2,2)]);   %Jacobian matrix of the transformation 



V = J*pinv(FI)*J;


criticalValue = norminv(  0.5 * ( 1 + significanceLevel ) );


err   = criticalValue * sqrt(diag(V)/n);
err_mu  = err(1:2);
err_vol = err(3:5);

sigma = sqrt(diag(a'*a));

err_sig = zeros(2,1);

err_sig(1) = err_vol(1);

err_sig(2) = criticalValue * sqrt( a(1,2)^2 * V(4,4)+a(2,2)^2*V(5,5)+2*a(1,2)*a(2,2)*V(4,5))/(sqrt(n)*sigma(2));


%% Printed output
fprintf('\n\n %3d%% Confidence Intervals: \n\n',100*significanceLevel )
fprintf('\n\n Estimation of mu: \n\n')
fprintf('%20s : %.4f  +/- %.4f \n', names{1}, mu(1), err_mu(1)) 
fprintf('%20s : %.4f  +/- %.4f \n', names{2}, mu(2), err_mu(2))

fprintf('\n\n Estimation of sigma: \n\n')
fprintf('%20s : %.4f  +/- %.4f \n', names{1}, sigma(1), err_sig(1)) 
fprintf('%20s : %.4f  +/- %.4f \n', names{2}, sigma(2), err_sig(2))

fprintf('\n\n Estimation of the volatility vector v: \n\n')
fprintf('%20s : %.4f  +/- %.4f \n', 'v(1) = a(1,1)'  ,vol(1), err_vol(1)) 
fprintf('%20s : %.4f  +/- %.4f \n', 'v(2) = a(1,2)'  ,vol(2), err_vol(2))
fprintf('%20s : %.4f  +/- %.4f \n', 'v(3) = a(2,2)'  ,vol(3), err_vol(3))
end

function LL = LogLikBS2DVol( theta, X , h)
% This function computes the minus log-likelihoods for a bivariate 
% Black-Scholes model with the volatility vector  parametrisation under the annual time scale
%
% Input: theta = (mus, log(a11), a12, loag(a13)) 
%        X ( n x 2 returns).
%        
% Output: -log-likelihoods (LL)

logLikelihoodFactor = 0.5 * log( 2 * pi );

%mean of the returns
mu = [theta(1);theta(2)]*h;

% volatility vector of the returns
a = zeros( 2, 2);
a(1,1) = exp(theta(3))* sqrt( h );
a(1,2) = theta(4)* sqrt( h );
a(2,1) = 0;
a(2,2) = exp(theta(5))* sqrt( h );

Sigma = a'*a;

[n,p] = size( X );

mu = repmat( mu' - 0.5 * diag(Sigma)', n, 1);

%  -log(density)
z = X - mu;  %centered returns

aInv = inv(a);
SInv = aInv*aInv';

 % determinant of a
detA = prod( diag(a) );

% - Log(likelihood)
LL = 0.5 * diag( z * SInv * z') + logLikelihoodFactor + log( detA );
end
