function out = EstPdfGK( x, m, comp )
% This function compute the nonparametric estimation of the density with 
% the Gaussian kernel, and makes graphical comparisons with the following 
% densities: Gaussian, Johnson_SU and Student. 
% It also compute goodness-of-fit tests for the Gaussian distribution:
% Jarque-Berra, revisited Jarque-Berra, corrected Kolmogorov-Smirnov and
% Cramer-von Mises tests.
%
% Input:  x: data (assumed i.i.d.) (n x 1 vector);
%         m: number of points for the graphs;
%         comp: 0 (default) or 1, if one wants the comparisons with
%               the Gaussian, Johnson_SU and Student densities.
%
%
% Output: x: points at which the density is estimated;
%        fn: estimation of the density at points x;
%      stat: tests statistics for the following tests of normality: 
%            Jarque-Berra (JB), revisited Jarque-Berra (JBR), corrected 
%            Kolmogorov-Sminorv (K)  and Cramer-von Mises (CVM);
%      pval: pvalue (%) for the following tests of normality: 
%            Jarque-Berra (JB), revisited Jarque-Berra (JBR), corrected 
%            Kolmogorov-Sminorv (K)  and Cramer-von Mises (CVM).
%
% Modified October 1, 2013
%
if nargin < 3
    comp =0;
end

n   = length(x);
mu  = mean(x);   %mean
sx  = std(x);    %standard deviation
h   = sx * n^-0.2;

t = (min(x):(max(x)-min(x))/m:max(x));
nbPoints = length( t );
y = zeros( nbPoints, 1 );

for i = 1:nbPoints
    y(i) = mean( normpdf( (x-t(i))/h)) / h;
end

figure
plot(t,y);
title('Nonparametric density');

out.x = t;
out.fn = y;

%%
if comp
    
    %% Gaussian
    z = normpdf((t-mu)/sx)/sx;

    %% Johnson-SU
    sk = skewness(x) ;
    ku = kurtosis(x) ;
    moments = [mu;sx;sk;ku];
    
    % get parameters
    param_J = EstJohnsonSU(moments);
    a = param_J(1); b = param_J(2); c = param_J(3); d= param_J(4);

    % compute density
    v = ( t - a ) / b;
    vv = sqrt( 1 + v .^ 2 );
    lawJ = (d / b) * normpdf( c + d * log( v + vv ) ) ./ vv;
    
    %% Student t
    if ku > 3 
        
        % estimate degrees of freedom 
        r = 4 + 6 / ( ku - 3 );
        
        % compute density
        vol = sx * sqrt( (r-2) / r );          
        lawT = tpdf( (t-mu) / vol, r ) / vol;  

    end
    
    %% Graphs
    figure
    plot(t,y)
    hold on
    plot(t,z,'r.-')
    if ku > 3 
        plot(t,lawT,'k--');
    end
    plot(t,lawJ,'g:');
    hold off
    title('Graph of estimated densities')
    if ku > 3 
        legend({'Nonparametric','Gaussian', 'Student','Johnson'}, 'location', 'best' );
    else
        legend({'Nonparametric','Gaussian', 'Johnson'}, 'location', 'best' );
    end
    
    %% GOF tests
    pval1   = 2 * ( 1 - normcdf( sqrt( n / 6 ) * abs( sk ) ) );
    pval2   = 2 * ( 1 - normcdf( sqrt( n / 24 ) * abs( ku - 3 ) ) );
    JBR     = - 2 * log( pval1 ) - 2 * log( pval2 ); %Improved Jarque-Berra test statistic
    pvalJBR = 100*( 1-chi2cdf(JBR,4)) ; % Improved Jarque-Berra test P-value
    JB      = n * ( sk^2 / 6 + ( ku - 3 )^2 / 24 ); %Jarque-Berra test statistic
    pvalJB  = 100*( 1 - chi2cdf( JB, 2 )); %Jarque-Berra test P-value
    
    [CVM,pvalcvm]   = GofGauss1dCVM( x, 1000 );
    [KS,pvalks]     = GofGauss1dKS( x, 1000 );
    
    out.stat.JB  = JB;
    out.stat.JBR = JBR;
    out.stat.KS  = KS;
    out.stat.CVM = CVM;
    out.pval.JB  = pvalJB;
    out.pval.JBR = pvalJBR;
    out.pval.KS  = pvalks;
    out.pval.CVM = pvalcvm;
    
    %% Output for parameters
    fprintf('\n')
    fprintf('Estimated mean  %8f \n',mu)
    fprintf('Estimated standard deviation: %8f \n', std(x))
    fprintf('Skewness = %5f , p-value = %3.2f%% \n',sk,100*pval1)
    fprintf('Kurtosis = %5f , p-value = %3.2f%% \n',ku, 100*pval2)
    if ku > 3 
        fprintf('Estimated sigma for Student model: %8f \n', vol)
        fprintf('Estimated degrees of freedom for the Student distribution (method of moments) = %g \n',r)
    end
    
    %% Output for Gof tests
    fprintf('\n\nGoodness-of-fit tests for the Gaussian distribution\n\n')
    fprintf('Jarque-Berra test         : JB  = %5.2f , P-value = %3.2f%% \n',JB, pvalJB)
    fprintf('Improved Jarque-Berra test: JBR = %5.2f , P-value = %3.2f%% \n',JBR, pvalJBR)
    fprintf('Lilliefors test           : KS  = %5.2f , P-value = %3.2f%%\n',KS, pvalks)
    fprintf('Cramer-von Mises test     : CVM = %5.2f , P-value = %3.2f%% \n',CVM, pvalcvm)
    fprintf('\n\n1000 iterations were used to approximate the P-values of the Lilliefors and CVM tests\n')
    fprintf('\n')
    
end
end
function [Parameters, Johnson_moments]= EstJohnsonSU(Estimated_moments)

 Est_skew_kurt = Estimated_moments(3:4);
options=optimset('display', 'off','TolFun',1e-16); 


 
    [Param] = fsolve(@InvJohnsonSkewKurt,[0;1],options,Est_skew_kurt);
        
    c = Param(1);
    d = Param(2);
    
    v1=exp(1/(2*d^2))*sinh(-c/d);
    v2=0.5*(exp(2/(d^2))*cosh(2*c/d)-1);
    sigma0 = sqrt(v2-v1^2);
    
    
    b = Estimated_moments(2)/sigma0;
    
    a = Estimated_moments(1)-b*v1;
    
    Parameters(2) = b;
    Parameters(1) = a;
    
    Parameters(3) = c;
    Parameters(4) = d;
    
    Johnson_moments(1) = a+b*v1;
    Johnson_moments(2) = b*sqrt(v2-v1^2);
    [Moments]=InvJohnsonSkewKurt(Param,Est_skew_kurt);
    Johnson_moments(3) = Moments(1)+Est_skew_kurt(1);
    Johnson_moments(4) = Moments(2)+Est_skew_kurt(2);
    

end

function [Moments]=InvJohnsonSkewKurt(param,Est)

c=param(1);
d=param(2);

v1=exp(1/(2*d^2))*sinh(-c/d);
v2=0.5*(exp(2/(d^2))*cosh(2*c/d)-1);
v3=0.25*(exp(9/(2*d^2))*sinh(-3*c/d)-3*exp(1/(2*d^2))*sinh(-c/d));
v4=(1/16)*(2*exp(8/(d^2))*cosh(4*c/d)-8*exp(2/(d^2))*cosh(2*c/d)+6);

sigma0 = sqrt(v2-v1^2);
skew0 = (v3-3*v1*v2+2*v1^3)/sigma0^3;

kurt0 = (v4-4*v1*v3+6*v2*v1^2-3*v1^4)/sigma0^4;




Moments = [skew0; kurt0]-Est;

end