function [dbstat, pValue, q95] = GofGauss1dCVM( X, N )
% Durbin test for normality using Cramr- von Mises statistic
% with a P-value calculated with N Monte Carlo simulations.
%
% Input : X: series of observations (assumed i.i.d.) (n x 1);
%         N: number of iterations for estimating the P-value.
%
% Output: dbstat: Cramer-von Mises test statistic;
%         pvalue: estimated p-value (%) of dbstat;
%         q95   : 95% quantile under the null hypothesis.
%


n = length(X);

t       = ( (1:n)' - 0.5) / n;
mu0     = mean(X);
std0    = std(X);

Z = ( X - mu0 ) / std0;
U = sort( normcdf( Z ) );

sn0     = (U-t)' * (U-t);
dbstat  = sn0 + 1/(12*n);



if nargin < 2
    pValue = NaN;
    q95    = NaN;
else
    sn = zeros(N,1);
    
parfor j = 1:N
    XX = randn(n,1);

    ZZ = ( XX - mean(XX) ) / std(XX);

    UU = sort(normcdf(ZZ));

    sn(j) = (UU-t)' * (UU-t);

end

pValue = 100*mean( (sn > sn0) );
q95    = quantile(sn,0.95);
end