function out = SerIndGRTest(x,p,N,Sig,graph)
% This function computes the Cramer-von Mises statistics  and the Fisher 
% statistic combining the p-values of the CVM statistics for testing
% serial independence between p consecutive observations, as in 
% Genest & Remillard (2004). All statistics are non-parametric. 
% The p-values are computed by simulation. The Fisher statistic is the 
% optimal one to combine p-values.
%
% Input
%    x: observations ( n x 1 matrix) assumed to be i.i.d.;
%    p: number of consective observations (2 <= p <= 6) ;
%    N: Number of samples to generate for estimating the p-value;
%    Sig: Order of quantile, default is 0.95;
%    graph: if 1, the dependogram is computed; the default is 0.
%
% Output 
%   List containing
%
%   cvmStats: Cramer-von Mises statistics for lags 1 to Lag defined by 
%             equation (3.12);
%   FisherStat: Statistic combining all p-values of Cramer-von Mises
%               statistics, distributed as a chi square;
%   cvmPvalues: Estimated p-values (percentage of rejection of the null 
%               hypothesis of serial independance) computed with N 
%               samples of i.i.d. uniforms variables;
%   FisherPvalue: p-value (%) of the Fisher statistic;
%   Q: quantiles of the simulated statistics;
%   rho: vector of autocorrelations;
%   LJstat: Ljung-Box statistic;
%   LJPval: p-value (%) of the Ljung-Box test;
%   cvm_sets: list of the subsets of indices.
%
% Reference: Genest, C. & Remillard, B. (2004). Tests of independence or 
%            randomness based on the empirical copula process. Test, 13, 
%            335--369.

%% Simulation

if nargin < 4
    Sig = 0.95;
    graph = 0;
elseif nargin <5;
    graph = 0;
end

n = length(x);

if(p<2)
    fprintf('p must be greater than  1 \n');
    return ;
elseif (p>6)
     fprintf('p must be smaller than 7 \n');
    return ;
end

%% Subsets
if p==2
  cvm_sets = {'12'};
elseif p== 3
  cvm_sets = {'12','13','123'} ;
elseif p==4
  cvm_sets = {'12','13','14','123','124','134','1234'} ;
elseif p==5
  cvm_sets = {'12','13','14','15','123',...
              '124','125','134','135','145',...
              '1234','1235','1245','1345','12345'} ;
elseif p==6
   cvm_sets = {'12','13','14','15','16','123',...
               '124','125','126','134','135','136','145',...
               '146','156','1234','1235','1236','1245',...
               '1246','1256','1345','1346',...
               '1356','1456','12345','12346','12356','12456',...
               '13456','123456'} ; 
end
 

[cvmStats,cor_mat,m] = SerIndGRCVM(x,p);
tt = n-(1:(p-1));
rho = cor_mat(1,2:end);

LJStat = n*(n+2)*sum((rho.^2)./tt);

LJPvalue = 100*(1-chi2cdf(LJStat,(p-1)));

S = zeros(N,m);

parfor k=1:N
    U = rand(n,1);
    S(k,:) = SerIndGRCVM(U,p);
end

Q = quantile(S,Sig);
 
   cvmPvalues = mean( ( S>repmat(cvmStats,N,1) ) ); 


FisherStat = -2*sum(log(cvmPvalues)); % Fisher stat

 cvmPvalues = 100* cvmPvalues; % percentage of rejection

FisherPval = 100*(1-chi2cdf(FisherStat,2*m));


%%

out.cvmStats     = cvmStats;
out.FisherStat   = FisherStat;
out.cvmPvalues   = cvmPvalues, 
out.FisherPvalue = FisherPval;
out.cvm_sets     = cvm_sets;
out.quantile     = Q;
out.rho          = rho;
out.LJStat       = LJStat;
out.LJPvalue     = LJPvalue;

%%
if(graph)
ligne = 5*ones(m,1);
figure


lineHandles = stem(cvmPvalues','filled','b-o');
 set(gca,'XLim',[0.95 (m+.05)]);
 set(gca,'XTick',[1:m]);
 set(gca,'XTickLabel',cvm_sets);
 set(lineHandles(1),'MarkerSize',4)
 
 if verLessThan('matlab','8.4')
 % <=R2014a 
   rotateXLabels( gca(), 90 )% From Matlab Central, by Ben Tordoff.
else
 % >R2014a
  set(gca, 'XTickLabelRotation', 90);
end
 
 
   hold on
   
   plot(ligne,'--r')
          
    xlabel('Sets')
    ylabel('P-value (%) of CVM ')
    title('Dependogram of CVM tests')
    
%% printed output
fprintf('\n\n Cramr-von Mises test of independence between %d consecutive values X(i+1),...X(i+p) \n\n',p);


fprintf('  Subset   Statistic   Pvalue\n\n');
for j=1:m
fprintf('%6s       %4.4f        %3.1f%%\n',cvm_sets{j},cvmStats(j),cvmPvalues(j));
end
fprintf('\n\n');
   
fprintf('Global Fisher test combining %d p-values of the CVM statistics\n\n',m);

fprintf('Fisher Stat: %4.2f    P-value: %3.1f%% \n\n',FisherStat,FisherPval);
    
end
end


function [cvm,cor_mat,m] = SerIndGRCVM(x,p)
% Cramer-von Mises statistics for testing serial independence between two 
% X(i), X(i+1), and X(i+p-1), as in Genest & Remillard (2004).
%% Indices

if p==2
  ind ={ [1 2]} ;
elseif p== 3
  ind = {[1 2],[1 3],[1 2 3]} ;
elseif p==4
  ind = {[1 2],[1 3],[1 4],[1 2 3],[1 2 4],[1 3 4],[1 2 3 4]} ;
elseif p==5
  ind = {[1 2],[1 3],[1 4],[1 5],[1 2 3],...
              [1 2 4],[1 2 5],[1 3 4],[1 3 5],[1 4 5],...
              [1 2 3 4],[1 2 3 5],[1 2 4 5],[1 3 4 5],[1 2 3 4 5]} ;
elseif p==6
  ind = {[1 2],[1 3],[1 4],[1 5],[1 6],[1 2 3],...
               [1 2 4],[1 2 5],[1 2 6],[1 3 4],[1 3 5],[1 3 6],[1 4 5],...
               [1 4 6],[1 5 6],[1 2 3 4],[1 2 3 5],[1 2 3 6],[1 2 4 5],...
               [1 2 4 6],[1 2 5 6],[1 3 4 5],[1 3 4 6],...
               [1 3 5 6],[1 4 5 6],[1 2 3 4 5],[1 2 3 4 6],[1 2 3 5 6],...
               [1 2 4 5 6],[1 3 4 5 6],[1 2 3 4 5 6]} ; 
end
n = length(x);
m = 2^(p-1)-1; %number of statistics

xx(:,1)=x;
for j=2:p
    xx(:,j) = [x(j:end);x(1:(j-1))];
end
cor_mat = corr(xx);
for i=1:n
    y= repmat(xx(i,:),n,1);
    v = (xx<=y);
    F = repmat(mean(v),n,1);
    vf = sqrt(6)*(v-F);  %to get a mean 1 approx for the cvm stats
    for j=1:m
        ll = ind{j};
        
      D(i,j) = mean ( prod( vf(:,ll) , 2 ) );
    end
end

cvm = sum(D.^2);
end