function [x,F,f] = DaviesApprox( delta, K, a, b, npts, model)
% Illustration of the Davies approximation of a distribution function
% by inversion of the characteristic function.
%
%  Input:   delta, K : parameters to be chosen affecting the approximation,
%           a, b: range of values for the approximation,
%           npts: number of points for the graph,
%           model: character string 'poiss', 'gauss', 'chi2', 'ncchi2'.
%  Output:  x: pts at which F is evaluated,
%           F: approximation of the distribution function,
%           f: approximation of the density (for continuous models).
%


t = (0:1:K)';

s = (t+1/2)'*delta;

tt = [0:1:(npts-1)]';

x = a + tt*(b-a)/(npts-1);

switch  model
    
    case  'poiss'
        % Poisson distribution
        l = 1;                
        b0 = exp( l * ( exp( 1i * s ) - 1 ) );
        trueF = poisscdf( x, 1 );
        mod = [' Poisson distribution with parameter \lambda =',num2str(l)];
        
    case  'gauss'
        
        % Gaussian distribution
        mu = 3; sigma = 0.25;
        b0 = exp( 1i * mu * s - 0.5 * sigma^2 * s .* s );
        trueF = normcdf( x, mu, sigma );
        mod = [' Gaussian distribution with \mu = ',num2str(mu),' and \sigma = ',num2str(sigma)];
        
    case  'chi2'
        % Chi-square distribution
        r = 5;                   
        b0 = ( 1 - 2 * 1i * s ) .^ (-r/2) ;
        trueF = chi2cdf( x, r );
        mod = [' \chi^2 distribution with  \nu = ',num2str(r),' degrees  of freedom'];
        
    case  'ncchi2'
        % noncentral chi-square distribution
        r = 5;
        lambda = 5;
        b0 = ( exp( 1i * lambda * s ./ (1 - 2 * 1i * s ) ) ) .* ( 1 - 2 * 1i * s ) .^ (-r/2);
        trueF = ncx2cdf( x, r, lambda );
        mod = [' Noncentral \chi^2 distibution with \nu = ',num2str(r),' and  \lambda = ', num2str(lambda)];

end

bb = delta * ( b0 ./ s );


F = zeros( npts, 1);
f = zeros( npts, 1);

for j = 1:npts
    
    z = x(j);
    v = exp( -1i * s * z );
    v1 = bb .* v;
    
    F(j) =  0.5 - sum( imag( v1 ) ) / pi;
    f(j) = delta * sum( real( b0 .* v ) ) / pi;
    
end

figure
plot(x,F,'r-');
hold on
plot(x,trueF,'b-');
title(['Davies''s approximation for ', mod, ', \Delta = ',num2str(delta), ' and K = ',num2str(K) ]);
legend( {'Estimated distribution function', 'Real distribution function'}, 'location', 'best');
hold off