function [VaR, error_dens, error_boot] = EstVaR( x, p )
%  Estimation of the quantile of order p (VaR(p) where x are losses)
%
% Input
%         x: iid sample (n x 1 vector);
%         p: orders (in fraction) for the quantile to be estimated
%            ( d x 1 vector).
%
% Output
%         Var: Estimated quantiles;
%         err_dens: 95% level estimation error using the density method;
%         err_boot: 95% level estimation error using the bootstrap method.
%
% Example of application:
% [VaR, error_dens, error_boot] = EstVaR(x,[0.95;0.99;0.999])

p = p(:);

addpath('../Chapter03')

X = sort(x);

n  = length(X);

VaR =  X( floor( n * p ) );  %VaR

fn = EstPdfGKpt(X,VaR);  %Estimation of the density at the VaR points
q = sqrt( p.* (1 - p) );

%% Estimation error for a 95% CI level using the density method
error_dens = 1.96 * q ./ ( fn * sqrt( n ) );

%% Estimation error for a 95% CI level using 1000 bootstrap samples
nbBootstrap = 1000;
bootst = bootstrp(nbBootstrap, @(y) quantile(y,p), X);
sd = std(bootst)';
error_boot = 1.96 * sd / sqrt( nbBootstrap );

%% Printed output

fprintf('\nLevel             Var            Error(density)        Error(bootstrap)\n')
for j=1:length(p)
    fprintf(' %2.2f       %7f           %7f               %7f \n', 100 * p(j), VaR(j), error_dens(j), error_boot(j) )
end

rmpath('../Chapter03')