function [X,V0,St,Vt] = SimLosses(N, mu, a, S0, Strike, Rate, t, T, ms, mc)
%  Simulation of the losses of a portofolio (long mc shares of a call 
%  option on asset 1, and short of ms shares of asset 2. The Black-Scholes 
%  model is used for the dynamics of the two assets.
%
% Example of application:
%
% N = 100000; %number of simulated values
% mu = [0.7314; 0.2441];
% a = [0.2975  0.1374; 0 0.2661]; % Cholesky decomposition of Sigma
% S0 = [348;28];
% t =  1/12;                     % 1 month
% T = 1/2;                       % Time to maturity: 6 months
% Strike = 350;                     % Strike price of the call option
% Rate = log(1+ 0.0046);       % Annual  rate in continuous time
% Sigma = a'*a;                    % Covariance matrix
% Volatility = sqrt(Sigma(2,2));
% ms = 1000;                        % number of shares of asset 2
% mc = 1000;                       % number of call options on asset 1
% [Y] = SimLosses(N,mu,a,S0, Strike, Rate,t,T,ms,mc);

addpath('../Chapter01')


Z =  randn(N,2) * a;  % N_2(0,Sigma)

Sigma = a'*a;

Volatility = sqrt( Sigma(1,1) );

%% BS prices of a call

tau = T; %maturity of the option

C0 = FormulaBS( S0(1), Strike, Rate, tau, Volatility);
V0 = mc * C0 

% Can use blsprice function if you have the Financial Toolbox
% V0 =  blsprice(S0, Strike, Rate, Time, Volatility, DividendRate);

%% Monte Carlo Method

St = zeros(N,2);

for j=1:2
    % Similated prices at time t
    St(:,j) = S0(j) * exp( (mu(j) - 0.5 * Sigma(j,j) ) * t + sqrt( t  ) * Z(:,j) );
end

tau = T-t;
Vt = mc * FormulaBS( St(:,1), Strike, Rate, tau, Volatility ) - ms * St(:,2) + ms * exp( t * Rate) * S0(2);

% Can use blsprice function if you have the Financial Toolbox
% Vt = blsprice(St, Strike, Rate, tau, Volatility, DividendRate);

% sample of possible losses
X = V0 - exp( -t * Rate ) * Vt;

X = sort( X );  % Sorting of values of X

rmpath('../Chapter01')