function [P, R, tau, r] = BondCIR( alpha, beta, sigma,  q1, q2, r0, n, maturities, days)
% Simulation of the values and yields of zero-coupon bonds
% when the (annualized ) spot rate (in percent) is modeled by a 
% Ornstein-Uhlenbeck process satisfying
%
%                dr = alpha(beta-r)dt + sigma  dW,   
%
%  with market price of risk q(r) = q1+q2 r. The maturities are 1,3,6 ad 12
%  months.
%
%  Input
%    r0: initial value;
%    n: number of values;
%   days: days in a year convention (360 default);
%   maturities: maturities in years (row vector).
%
%
%   Output
%      P: bond values;
%      R: Annual rate for the bond;
%      tau: maturities in years.
%
% Example:
% [P,R,tau,rimp] = BondCIR(0.5,2.55,0.365,0.3,0,3.55,1080,[1/12 3/12 6/12 1],365);

if nargin < 9
    days = 360;
end

nbBonds=length(maturities);
tau = repmat( maturities, n+1, 1); % maturity in years

scalingFact   = 100;

% simulate short rate (%) on an annual time scale 
r = SimCIR( alpha, beta, sigma, r0, n ,1/days); 


% get pricing coefficients
[ A, B ] = getCIRParam( [alpha; beta; sigma; q1; q2], tau, scalingFact );

P = 100 * exp( A - B .* repmat( r, 1 , nbBonds) / scalingFact );
R = ( B .* repmat( r, 1, nbBonds) - scalingFact * A ) ./ tau;


month = 12*maturities;
figure(1)
subplot(2,1,1)
plot(P)
legend( strcat('\tau = ',num2str(month', ' %d'), ' months' ), 'location', 'BestOutside')
title('Daily values of a zero-coupon bond with $100 face value for different maturities');
ylabel('$')
xlabel('Days')
subplot(2,1,2)
plot(R)
title('Annual yield of a zero-coupon bond for different maturities');
ylabel('%')
xlabel('Days')