function [param, error, r] = EstCIR(  data, method, days , significanceLevel)
% Estimation of the parameters of the CIR model
%
%        dr = alpha(beta-r)dt + sigma sqrt(r) dW 
%
% with market price of risk q(r) = q1/sqrt(r) +q2 sqrt(r). The time scale 
% is in years and the units are percentages.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input
%      data: [R,tau] (n x 2), with R: annual bonds yields in percentage,
%             and tau: maturities in years;
%      method: 'Hessian' (default), 'exact', 'num';
%      days: number of days per year (default: 360);
%      significanceLevel: (95% default).
%
% Output
%       param: parameters (alpha, beta, sigma, q1,q2) of the model;
%       error: estimation errors for the given confidence level;
%       rimp: implied spot rate.
%
% Example:
%  [theta, error, rimp] = EstCIR(DataVasicek);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%
if nargin < 2
    method = 'Hessian';
    days = 360;
    significanceLevel = 0.95;

elseif nargin < 3
    days = 360;
    significanceLevel = 0.95;
    
elseif nargin < 4
    significanceLevel = 0.95;
end 


%%
R   = data(:,1);
tau = data(:,2);
h = 1/days;
scalingFact = 100;

%% starting values

% Rough estimation in the case of a Feller process
phi0 = autocorr(R,1) ;
alpha0 = -log(phi0(2))/h;
beta0  = mean(R);
sigma0 = std(R)*sqrt(2*alpha0/beta0);
nu0 = 0.25*sigma0^2/(alpha0*beta0);
param0 =  [alpha0; beta0; sigma0; 0 ; 0];
theta0 = [log(alpha0); log(beta0); log(sigma0); 0 ; 0];
%theta0 = [log(2); log(mean(R)); log(1); 0; 0];
%param0 = [0.5;2.55;0.365;0.3;0]; % true parameters


% pricing coefficients (computed by changing the scale from percentage
% to fraction)

[A, B] = getCIRParam( param0, tau, scalingFact );

% get short rate
r = ( tau .* R + scalingFact * A )./ B;    

figure(2)
plot(r)
title('Annual implied rates (%) for the starting parameters');

button = questdlg(  'Do you want to continue?',...
                    'Estimation','Yes','No','Yes');

if strcmp(button,'No')
    
    fprintf('\n\n');
    disp('It the rates are too big, decrease q1.');
    fprintf('\n');
    disp('If the rates are too small, increase q1.');
    fprintf('\n\n');
    param   = NaN;
    error   = NaN;
    rimp    = NaN;
    close

else %strcmp(button,'Yes')
    
      close;
                                   
    %% Options for fminunc
    options = optimset( 'largescale',   'on', ...
                        'MaxFunEvals',  50000, ...
                        'MaxIter',      5000, ...
                        'Display',      'iter', ...
                        'TolFun'      , 1e-6, ...
                        'TolCon'      , 1e-6, ...
                        'TolX'        , 1e-6, ...                        
                        'diagnostics',  'on');
    
    %% Maximisation of the log-likelihood
     addpath('../ChapterA', '../ChapterB');
    [theta, fval, exitflag, output, grad, hessian] = fminunc(   @(x) sum(LogLikCIR(x, R, tau, days)), ...
                                                                    theta0, options );
    
    clear functions
    
    
    alpha = exp(theta(1));
    beta  = exp(theta(2));
    sigma = exp(theta(3));
    q1    = theta(4);
    q2    = theta(5);
    
   param = [alpha; beta; sigma; q1; q2];
    [A, B] = getCIRParam(param, tau, scalingFact );
    
    % Implied spot rates
    r = ( tau .* R + scalingFact * A ) ./ B;
    
    % real parameters
    paramTrue = [0.5; 2.55; 0.365;0.3;0];
    
    
    [A, B] = getCIRParam( paramTrue, tau, scalingFact );
    
    % Implied spot rates
    r0 = ( tau .* R + scalingFact * A ) ./ B;
    
    figure
    plot(r,'b-')
    hold on
    plot(r0,'r:')
    title('Implied annual spot rates (%)');
    legend({'Implied rates', 'Real rates'}, 'location', 'best' );
    
    fprintf('\n\nThe estimated coefficients correspond to the annualized spot rate (%%)');
    
    n = length( R );
   
    
    lmin = min(eig(hessian));
if(lmin<0)
 fprintf('\n The Hessian is not positive definite. Estimation errors are not reliable.\n')
 fprintf('The numerical method will be used instead.\n\n');
 method = 'num';
end


   %  Fisher information matrix 

%%  Fisher information matrix 
switch method
    case 'Hessian'
        FI = hessian/n;
fprintf('\n Fisher information computed with the numerical Hessian from fminunc (Appendix B.5.1)\n\n');
    case 'num'
        J = NumJacobian(@(x) LogLikCIR(x, R, tau, days),theta,0.01);
        fprintf('\n Fisher information computed with the numerical gradient (Appendix B.5.1)\n\n');
        FI = cov(J);
end

rmpath('../ChapterA','../ChapterB');
  
  D = diag([alpha beta sigma 1 1]); 
 cov_est = D*pinv(FI)*D;
 
  %% precision
    criticalValue = norminv( 0.5 *(1+significanceLevel) );
    error = criticalValue * sqrt( diag( cov_est / n ) );
    
    phi = exp(-paramTrue(1)*h);
    phiest = exp(-param(1)*h);
    errorphi = h*phiest*error(1);
    fprintf( '\n alpha = %.4f /+ %.4f \n', param(1), error(1));
    fprintf( '\n  beta = %.4f /+ %.4f \n', param(2), error(2));
    fprintf( '\n sigma = %.4f /+ %.4f \n', param(3), error(3));
    fprintf( '\n    q1 = %.4f /+ %.4f \n', param(4), error(4));
    fprintf( '\n    q2 = %.4f /+ %.4f \n', param(5), error(5));
    fprintf( '\n   phi = %.4f phiest = %.4f /+ %.4f \n', phi, phiest, errorphi);   
    
    
end
end
function LL = LogLikCIR( theta, R, tau, days )
%  This function computes the -Log-likelihoods for the CIR model
%
%            dr = alpha(beta-r)dt + sigma sqrt(r) dW   
% with market price of risk q(r) = q1/sqrt(r) +q2 sqrt(r)
%
% Input
%       theta: parameters of the annualized spot rate ;
%           R: annual returns in percent of the bonds (n x 1);
%         tau: maturities (n x 1) in days;
%        days: number of days in a year.
%
% Output
%        LL: -log-likelihoods.

persistent n
if isempty( n )
    n = length( R );
end

scalingFact = 100;
h = 1/days;

LL = ones(n-1,1);

    alpha = exp(theta(1));
    beta  = exp(theta(2));
    sigma = exp(theta(3));
    q1    = theta(4);
    q2    = theta(5);
    
   param = [alpha; beta; sigma; q1; q2];


[ A, B, a, b ] = getCIRParam(param, tau, scalingFact );
if (a<=0) || (b<=0)
    LL = 1.0e+20*LL;
    return
end

r = ( tau .* R + scalingFact * A ) ./ B;
if any( r <= 0 )
    LL = 1.0e+20*LL;
    return
end

% parameters for the returns
phi     = exp( - alpha *h);
nu      = 4 * alpha * beta / sigma^2;
omega = beta * ( 1 - phi ) / nu ; 

r       = r / omega;
D       = r(1:end-1) * phi;

% log-likelihood function
%LL = log( omega ) +  log( abs( B(2:end) ./ tau(2:end) ) ) -  log( max(  PDFNChi2( r(2:end), nu, D ), eps ) ) ;
% Better take the Matlab function ncx2pdf which is more stable numerically.
LL = log( omega ) +  log( abs( B(2:end) ./ tau(2:end) ) ) -  log( max(  ncx2pdf( r(2:end), nu, D ), eps ) ) ;

if any(isinf( LL )) | any(isnan( LL )) | any(imag( LL ))
    LL = 1.0e+20*ones(n-1,1);
    return
end
end