function [param, error, r] = EstFeller(  data, method, days , significanceLevel)
% Estimation of the parameters of the Feller model
%
%        dr = alpha(beta-r)dt + sigma sqrt(r) dW 
%
% The time scale is in years and the units are percentages.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input
%      data: annual bonds yields in percentage;
%      method: 'Hessian' (default), 'exact', 'num';
%      days: number of days per year (default: 360);
%      significanceLevel: (95% default).
%
% Output
%       param: parameters (alpha, beta, sigma) of the model;
%       error: estimation errors for the given confidence level.
%
% Example:
%  [theta, error] = EstCIR(data);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%
if nargin < 2
    method = 'Hessian';
    days = 360;
    significanceLevel = 0.95;

elseif nargin < 3
    days = 360;
    significanceLevel = 0.95;
    
elseif nargin < 4
    significanceLevel = 0.95;
end 


%%
R   = data;
h = 1/days;

% starting values

phi0 = autocorr(R,1);

alpha0 = -log(phi0(2))/h;
beta0  = mean(R);
sigma0 = std(R)*sqrt(2*alpha0/beta0);

theta0 = [log(alpha0); log(beta0); log(sigma0)];
% true parameters
% alpha0 = 0.5; beta0 = 2.55; sigma0 = 0.365;

 %% Options for fminunc
    options = optimset( 'largescale',   'off', ...
                        'MaxFunEvals',  50000, ...
                        'MaxIter',      5000, ...
                        'Display',      'iter', ...
                        'TolFun'      , 1e-6, ...
                        'TolCon'      , 1e-6, ...
                        'TolX'        , 1e-6, ...                        
                        'diagnostics',  'on');
    

    
    %% Maximisation of the log-likelihood
     addpath('../ChapterA', '../ChapterB');
    [theta, fval, exitflag, output, grad, hessian] = fminunc(   @(x) sum(LogLikFeller(x, R, days)), theta0, options );
    
    clear functions
    
    param = exp(theta);
    alpha = param(1);
    beta  = param(2);
    sigma = param(3);
   
    
   
    fprintf('\n\nThe estimated coefficients correspond to the annualized spot rate (%%)');
    
    n = length( R );
   
    
lmin = min(eig(hessian));
if(lmin<0)
 fprintf('\n The Hessian is not positive definite. Estimation errors are not reliable. The numerical method will be used instead\n\n');
 method = 'num';
end


   %  Fisher information matrix 

%%  Fisher information matrix 
switch method
    case 'Hessian'
        FI = hessian/n;
fprintf('\n Fisher information computed with the numerical Hessian from fminunc (Appendix B.5.1)\n\n');
    case 'num'
        J = NumJacobian(@(x) LogLikFeller(x, R, days),theta,0.001);
        fprintf('\n Fisher information computed with the numerical gradient (Appendix B.5.1)\n\n');
        FI = cov(J);
end

rmpath('../ChapterA','../ChapterB');
  
  D= diag(param) ;   %Jacobian matrix of the transformation
  
 cov_est = D *pinv(FI)* D;
 
  %% precision
    criticalValue = norminv( 0.5 *(1+significanceLevel) );
    error = criticalValue * sqrt( diag( cov_est / n ) );
    
    alpha0=0.55;
    phi = exp(-alpha0*h);
    phiest = exp(-param(1)*h);
    errorphi = h*phiest*error(1);
    fprintf( '\n alpha = %.4f /+ %.4f \n', param(1), error(1));
    fprintf( '\n  beta = %.4f /+ %.4f \n', param(2), error(2));
    fprintf( '\n sigma = %.4f /+ %.4f \n', param(3), error(3));
    fprintf( '\n   phi = %.4f, phiest = %.4f /+ %.4f \n', phi, phiest, errorphi);   
    
    
end

function LL = LogLikFeller( theta, R, days )
%  This function computes the -Log-likelihoods for the CIR model
%
% dr = alpha(beta-r)dt + sigma sqrt(r) dW   
%
% Input
%       theta: log of parameters of the annualized spot rate; 
%           R: annual returns in percent of the bonds (n x 1);
%        days: number of days in a year. 
%
% Output
%        LL: -log-likelihoods.

persistent n
if isempty( n )
    n = length( R );
end

h = 1/days;

LL = ones(n-1,1);

%%
alpha   = exp(theta(1));
beta    = exp(theta(2));
sigma   = exp(theta(3));
phi     = exp( - alpha *h);
nu      = 4 * alpha * beta / sigma^2;
omega   = beta * ( 1 - phi )/ nu ; 
z       = R / omega;
D       = z(1:end-1) * phi; % non-centrality parameters;

%% log-likelihood function
%LL = log( omega )  -  log( max(  PDFNChi2( z(2:end), nu, D ), eps ) ) ;
%seems unstable for estiamtion. Preferable to use the function from Matlab 
LL = log( omega )  -  log( max(  ncx2pdf( z(2:end), nu, D ), eps ) ) ;


if any(isinf( LL )) | any(isnan( LL )) | any(imag( LL ))
    LL = 1.0e+20*ones(n-1,1);
    return
end
end