function [paramNum, errorNum, paramExp, errorExp] = EstOU( R, method, days , significanceLevel)
%  Estimation of the parameters of the OU model 
%
%           dr = alpha(beta-r)dt + sigma  dW,
%
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input
%      data: observations;
%      method: 'Hessian' (default), 'exp', 'num';
%      days: number of pdays per year (default: 360);
%      significanceLevel: (95% default).
%
% Output
%       paramNum: Numerical estimation of the parameters alpha, beta, sigma, phi;
%       errorNum: Estimation errors for a 95% confidence level of paramNum;
%       paramExp: Explicit estimation of the parameters alpha, beta, sigma, phi;
%       errorExp: Estimation errors for a 95% confidence level of paramExp.
%
% Example:
%  [paramNum, errorNum, paramExp,errorExp] = EstOU(data);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin < 2
    method = 'Hessian';
    days = 360;
    significanceLevel = 0.95;

elseif nargin < 3
    days = 360;
    significanceLevel = 0.95;
elseif nargin < 4
    significanceLevel = 0.95;
end 

    h = 1/days;

       
    theta0 = [log(1); mean(R); log(1)]; 

    %% Options for fminunc
    options = optimset( 'largescale',   'off', ...
                        'MaxFunEvals',  50000, ...
                        'MaxIter',      5000, ...
                        'Display',      'iter', ...
                        'TolFun'      , 1e-6, ...
                        'TolCon'      , 1e-6, ...
                        'TolX'        , 1e-6, ...                        
                        'diagnostics',  'on');
    
    %% Maximization of the log-likelihood
    
    [theta, fval, exitflag, output, grad, hessian] = fminunc(   @(x) sum(LogLikOU(x, R, days)), theta0, options );
    clear functions
     
    alpha = exp(theta(1));
    beta  = theta(2);
    sigma = exp(theta(3));
    phi = exp(-alpha * h );
    paramNum = [alpha, beta, sigma, phi];
    
    
    n = length( R );
    
    
 lmin = min(eig(hessian));
if(lmin<0)
 fprintf('\n The Hessian is not positive definite. Estimation errors are not reliable. The numerical method will be used instead\n\n');
 method = 'num';
end

addpath('../ChapterB');
   %  Fisher information matrix 

%%  Fisher information matrix 
switch method
    case 'Hessian'
        FI = hessian/n;
fprintf('\n Fisher information computed with the numerical Hessian from fminunc (Appendix B.5.1)\n\n');
    case 'num'
        J = NumJacobian(@(x) LogLikOU(x, R, days),theta,0.01);
        fprintf('\n Fisher information computed with the numerical gradient (Appendix B.5.1)\n\n');
        FI = cov(J);
end

rmpath('../ChapterB');
  
D = diag([alpha 1 sigma]);
    
 covNum = D *pinv(FI)* D;
 
 
  %% precision
  
    fprintf('\n\n Estimation with the numerical method\n\n');
    
    criticalValue = norminv( 0.5 *(1+significanceLevel) );
    errorNum0 = criticalValue * sqrt( diag( covNum / n ) );
    errorNum4 = phi * h * errorNum0(1);
    errorNum = [errorNum0;errorNum4];
    fprintf( '\n alpha = %.4f /+ %.4f \n', alpha, errorNum(1));
    fprintf( '\n  beta = %.4f /+ %.4f \n', beta, errorNum(2));
    fprintf( '\n sigma = %.4f /+ %.4f \n', sigma, errorNum(3));
    fprintf( '\n   phi = %.4f /+ %.4f \n', phi, errorNum(4));
   
  
    
    %% exp MLE estimation
    
     z1 = R(2:end); z2 = R(1:(end-1));
     m1 = mean(z1); m2 = mean(z2);
     s1 = std(z1); s2  =std(z2);
     
     h = 1/days;
     rho = corr(z1,z2);
     
     phiest = rho*s1/s2
     
     gamma = std( z1-phiest*z2);
     g = 1-phiest^2;
     
     alphaexp = -log(phiest)/h;
     betaexp = (m1-phiest*m2)/(1-phiest);
     sigmaexp = gamma*sqrt(2*alphaexp/g);
      zeta = (sigmaexp^2/gamma)*( 2*alphaexp*phiest^2*h-g  )/(phiest*g^2*h ) ;
      
     
     
     error_exp(4) = criticalValue*sqrt(g/n);
     error_exp(1) =  error_exp(4)/(h*phiest);
     error_exp(2) = criticalValue*gamma/(1-phiest)/sqrt(n);
     error_exp(3) = criticalValue*sqrt((zeta^2*g + 0.5*sigmaexp^2)/n);
     error_exp(5) = criticalValue*sqrt(0.5*gamma^2/n);
     
    
     
     Vexp = diag([g gamma^2/(1-phiest)^2 0.5*gamma^2]);
     
     Jexp = [ -1/(phiest*h) 0 0; 0 1 0; zeta 0 sigmaexp/gamma;  1 0 0; 0 0 1;];
     
     covExp = Jexp*Vexp*Jexp';
     
     paramExp = [alphaexp;betaexp;sigmaexp;phiest;gamma];
     errorExp = criticalValue*sqrt(diag(covExp)/n);
     
    fprintf('\n\n Estimation with the explicit method\n\n');
    
    
    
    
    fprintf( '\n alpha = %.4f /+ %.4f \n', alphaexp, error_exp(1));
    fprintf( '\n  beta = %.4f /+ %.4f \n', betaexp, error_exp(2));
    fprintf( '\n sigma = %.4f /+ %.4f \n', sigmaexp, error_exp(3));
    fprintf( '\n phi   = %.4f /+ %.4f \n', phiest, error_exp(4));
    fprintf( '\n gamma = %.4f /+ %.4f \n', gamma, error_exp(5));
    
   
  for i=1:3
      for j=1:3
          [i j]
          covNum(i,j)
          covExp(i,j)
      end
  end

end
function LL  = LogLikOU( theta, R, days )
%  This function computes the -log-likelihoods for the Orstein-Uhlenbeck model
%
%           dr = alpha(beta-r)dt + sigma  dW.
%
%
% Input
%   theta: parameters of the annualized spot rate ;
%   R: annual returns;
%   days: number of days in a year.
%
% Output
%    -log-likelihoods (LL).


likelihoodFactor = 0.5 * log( 2 * pi );
    
 n = length(R)-1;
 
 LL = ones(n,1);

h = 1/days;
alpha   = exp(theta( 1 ));
beta    = theta( 2 );
sigma   = exp(theta( 3 ));
phi     = exp( - alpha *h);
gamma   = sigma * sqrt( ( 1 - phi^2 ) / ( 2 * alpha ) );


r = R;

% residuals
eps = ( r(2:end) - beta - phi * ( r(1:(end-1)) - beta ) ) / gamma;

% log-likelihood function to be minimized
LL = likelihoodFactor + 0.5 * eps.^2 + log( gamma )  ;

if any(isinf( LL )) | any(isnan( LL )) | any(imag( LL ))
    LL = 1.0E+20*LL;
    return
end
end