function [param, error, rimp] = EstVasicek( data, method, days , significanceLevel)
%  Estimation of the parameters of the Vasicek model 
%
%           dr = alpha(beta-r)dt + sigma  dW,
%
%  with market price of risk q(r) = q1+q2 r. The time scale is in years and
%  the units are percentages.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input
%      data: [R,tau] (n x 2), with R: annual bonds yields in percentage,
%             and tau: maturities in years;
%      method: 'Hessian' (default), 'exact', 'num';
%      days: number of days per year (default: 360);
%      significanceLevel: (95% default).
%
% Output
%       theta: parameters (alpha, beta, sigma, q1,q2) of the model;
%       error: estimation errors for the given confidence level;
%       rimp: implied spot rate.
%
% Example:
%  [theta, error, rimp] = EstVasicek(DataVasicek);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%
if nargin < 2
    method = 'Hessian';
    days = 360;
    significanceLevel = 0.95;

elseif nargin < 3
    days = 360;
    significanceLevel = 0.95;
    
elseif nargin < 4
    significanceLevel = 0.95;
end 


%%
R   = data(:,1);
tau = data(:,2);
h = 1/days;
scalingFact = 100;

%% starting values   
% Rough estimation in the case of a Feller process
phi0 = autocorr(R,1) ;
alpha0 = -log(phi0(2))/h;
beta0  = mean(R);
sigma0 = std(R)*sqrt(2*alpha0/(1-phi0(2)^2));

param0 =  [alpha0; beta0; sigma0; 0 ; 0];
theta0 = [log(alpha0); log(beta0); log(sigma0); 0 ; 0];
%theta0 = [log(2); log(mean(R));log(1); 0; 0];
%param0 = [0.5;2.55;0.365;0.3;0]; % true parameters


% pricing coefficients (computed by changing the scale from percentage
% to fraction)


[A, B] = getVasicekParam( param0, tau, scalingFact );

% get short rate in percent
r = ( tau.* R + scalingFact * A )./ B;    

figure(2)
plot(r)
title('Annual implied rates (%) for the starting parameters');

button = questdlg(  'Do you want to continue?',...
                    'Estimation','Yes','No','Yes');
                
if strcmp(button,'No')
    
    fprintf('\n\n');
    disp('If the rates are too big, decrease q1.');
    fprintf('\n');
    disp('If the rates are too small, increase q1.');
    fprintf('\n\n');
    param   = NaN;
    error   = NaN;
    rimp    = NaN;
    close

else %strcmp(button,'Yes')
    
    close;
    
    %% Options for fminunc
    options = optimset( 'largescale',   'off', ...
                        'MaxFunEvals',  50000, ...
                        'MaxIter',      5000, ...
                        'Display',      'iter', ...
                        'TolFun'      , 1e-6, ...
                        'TolCon'      , 1e-6, ...
                        'TolX'        , 1e-6, ...                        
                        'diagnostics',  'on');
    
    %% Maximization of the log-likelihood
    
    [theta, fval, exitflag, output, grad, hessian] = fminunc(   @(x) sum(LogLikVasicek(x, R, tau, days)), theta0, options );
    clear functions

    
     
       
     alpha = exp(theta(1));
     beta  = exp(theta(2));
     sigma = exp(theta(3));
     q1    = theta(4);
     q2    = theta(5);
    
    param = [alpha; beta; sigma; q1; q2];
    [A, B] = getVasicekParam( param, tau, scalingFact );
    
    
    rimp = ( tau .* R + scalingFact * A ) ./ B; % implied spot rate
    
    
   
    
    
   paramTrue = [0.5; 2.55; 0.365; 0.3; 0];% true parameters


       
    [A, B] = getVasicekParam( paramTrue, tau, scalingFact );
    
    % estimated spot rates
    r0 = ( tau .* R + scalingFact * A ) ./ B;
    
    figure
    plot(rimp,'b-')
    hold on
    plot(r0,'r:')
    title('Implied and annual spot rates (%)');
    legend({'Implied rates', 'True rates'}, 'location', 'best' );
    
    fprintf('\n\n The estimated coefficients correspond to the annualized spot rate (%%)');
    
    n = length( R );
    
    
    lmin = min(eig(hessian));
if(lmin<0)
 fprintf('\n The Hessian is not positive definite. Estimation errors are not reliable. The numerical method will be used instead\n\n');
 method = 'num';
end

addpath('../ChapterB');
   %  Fisher information matrix 

%%  Fisher information matrix 
switch method
    case 'Hessian'
        FI = hessian/n;
fprintf('\n Fisher information computed with the numerical Hessian from fminunc (Appendix B.5.1)\n\n');
    case 'num'
        J = NumJacobian(@(x) LogLikVasicek(x, R, tau, days),theta,0.01);
        fprintf('\n Fisher information computed with the numerical gradient (Appendix B.5.1)\n\n');
        FI = cov(J);
end

rmpath('../ChapterB');
  
  D = diag([alpha beta sigma 1 1]);  
 cov_est = D *pinv(FI)* D;
 
  %% precision
    criticalValue = norminv( 0.5 *(1+significanceLevel) );
    error = criticalValue * sqrt( diag( cov_est / n ) );
    
    phi = exp(-paramTrue(1)*h);
    phiest = exp(-param(1)*h);
    errorphi = h*phiest*error(1);
    fprintf( '\n alpha = %.4f /+ %.4f \n', param(1), error(1));
    fprintf( '\n  beta = %.4f /+ %.4f \n', param(2), error(2));
    fprintf( '\n sigma = %.4f /+ %.4f \n', param(3), error(3));
    fprintf( '\n    q1 = %.4f /+ %.4f \n', param(4), error(4));
    fprintf( '\n    q2 = %.4f /+ %.4f \n', param(5), error(5));
    fprintf( '\n   phi = %.4f phiest = %.4f /+ %.4f \n', phi, phiest, errorphi);
end
end
function LL  = LogLikVasicek( theta, R, tau, days )
%  This function computes the -log-likelihoods for the Vasicek model
%
%           dr = alpha(beta-r)dt + sigma  dW,
%
% with market price of risk q(r) = q1+q2 r.
%
% Input
%   theta: parameters of the annualized spot rate in percent;
%   R: annual returns in percent of the bonds (n x 1);
%   tau: maturities (n x 1) in years;
%   scalingFact: measurement scale ( 100 x h, h is 1 if annual, 360 or 365
%                if daily).  
%
% Output
%    -log-likelihoods (LL).

scalingFact = 100;

    likelihoodFactor = 0.5 * log( 2 * pi );
    
 n = length(R)-1;
 
 LL = ones(n,1);

 %% annual parameters 
alpha   = exp(theta( 1 ));
beta    = exp( theta( 2 ) );
sigma   = exp(theta( 3 ));
  q1    = theta(4);
  q2    = theta(5);
  
  param = [alpha;beta;sigma;q1;q2];

[ A, B, a, b ] = getVasicekParam( param, tau, scalingFact );

if (a<=0) || (b<=0)
    LL = 1.0e+20*LL;
    return
end

r = ( tau .* R + scalingFact * A ) ./ B;
if any( r <= 0 )
    LL = 1.0e+20*LL;
    return
end


h = 1/days;

phi     = exp( - alpha * h);
gamma   = sigma * sqrt( ( 1 - phi^2 ) / ( 2 * alpha ) );

% residuals
eps = ( r(2:end) - beta - phi * ( r(1:end-1) - beta ) ) / gamma;

% log-likelihood function to be minimized
LL = likelihoodFactor + 0.5 * eps.^2 + log( gamma ) + log(  B(2:end) ./ tau(2:end)  );

if any(isinf( LL )) | any(isnan( LL )) | any(imag( LL ))
    LL = 1.0e+20*ones(n,1);
    return
end
end