function LL = LogLikCIR( theta, R, tau, days )
%  This function computes the -Log-likelihoods for the CIR model
%
%            dr = alpha(beta-r)dt + sigma sqrt(r) dW   
% with market price of risk q(r) = q1/sqrt(r) +q2 sqrt(r)
%
% Input
%       theta: parameters of the annualized spot rate ;
%           R: annual returns in percent of the bonds (n x 1);
%         tau: maturities (n x 1) in days;
%        days: number of days in a year.
%
% Output
%        LL: -log-likelihoods.

persistent n
if isempty( n )
    n = length( R );
end

scalingFact = 100;
h = 1/days;

LL = ones(n-1,1);

    alpha = exp(theta(1));
    beta  = exp(theta(2));
    sigma = exp(theta(3));
    q1    = theta(4);
    q2    = theta(5);
    
   param = [alpha; beta; sigma; q1; q2];


[ A, B, a, b ] = getCIRParam(param, tau, scalingFact );
if (a<=0) || (b<=0)
    LL = 1.0e+20*LL;
    return
end

r = ( tau .* R + scalingFact * A ) ./ B;
if any( r <= 0 )
    LL = 1.0e+20*LL;
    return
end

% parameters for the returns
phi     = exp( - alpha *h);
nu      = 4 * alpha * beta / sigma^2;
omega = beta * ( 1 - phi ) / nu ; 

r       = r / omega;
D       = r(1:end-1) * phi;

% log-likelihood function
%LL = log( omega ) +  log( abs( B(2:end) ./ tau(2:end) ) ) -  log( max(  PDFNChi2( r(2:end), nu, D ), eps ) ) ;
% Better take the Matlab function ncx2pdf which is more stable numerically.
LL = log( omega ) +  log( abs( B(2:end) ./ tau(2:end) ) ) -  log( max(  ncx2pdf( r(2:end), nu, D ), eps ) ) ;

if any(isinf( LL )) | any(isnan( LL )) | any(imag( LL ))
    LL = 1.0e+20*ones(n-1,1);
    return
end
endd

