function [param, error, V] = EstMerton( data, method, h, significanceLevel )
% This function estimates the parameters of the Merton Jump-Diffusion
% model using a numerical method to maximize the likelihood. Fisher information
% can be estimated with the Hessian matrix (fminunc output) or with the 
% numerical gradients. See Appendix B.5.1 for more details.
% 
% Input    
%           data: prices (column vector);
%           method: 'Hessian' (default), 'num';
%           h: time scale; for daily data, h=1/252 (default);
%           significanceLevel (95% default).
%
% Output:   param: estimation of the parameter (annual time scale):
%            mu, sigma, lambda, gamma, delta;
%           error:  significanceLevel% CI error for the parameters;
%           V: estimation of covariance matrix.
% Example: 
%  [param,error, V] = EstMerton(ApplePrices);


if nargin < 2
    method = 'Hessian';
    significanceLevel = 0.95;
    h = 1/252;
elseif nargin < 3
    significanceLevel = 0.95;
    h = 1/252;
elseif nargin < 4
    significanceLevel = 0.95;
end

%% compute and plot log-returns
returns = diff( log( data ), 1 );
n       = length( returns );
 
%% PRE-OPTIMIZATION
 %theta = (mu, log(sigma),log(lambda),gamma,log(delta))
 
 
 %do not start with a too small value; we are working on the annual time scale for the likelihood
initialGuess    = zeros(5,1); % Initial values of the parameters
intitialGuess(3) = log(1/h);  % 5 per day

options         = optimset('largescale','on','MaxFunEvals',5000,'MaxIter',2000,'Display','iter');

%% OPTIMIZATION
[theta, f, exitflag, output, grad, hessian] = fminunc( @(x) sum(LogLikMerton( x, returns,h)), initialGuess, options );

% remove persistent variables
clear functions

%% OUTPUT

param = T(theta);

 addpath(genpath('../'))
 %addpath('../Chapter03','../Chapter04''../ChapterB')
%%  Fisher information matrix 
fprintf('\n\n');

lmin = min(eig(hessian));

if(lmin<0)
 fprintf('\n The Hessian is not positive definite. Estimation errors are not reliable. The numerical method will be used instead\n\n');
 method = 'num';
end

switch method
    case 'Hessian'
        FI = hessian/n;
fprintf('\n Fisher information computed with the numerical Hessian from fminunc (Appendix B.5.1)\n\n');


    case 'num'
        J = NumJacobian(@(x) LogLikMerton(x,returns,h),theta,0.001);
        fprintf('Fisher information computed with the numerical gradient (Appendix B.5.1)\n\n');
        FI = cov(J);
end



%% Corrections

D = Jacob(theta); %Jacobian matrix of the transformation for positiveness


V = D*pinv(FI)*D;


%% Precision

criticalValue = norminv( 0.5 *(1+significanceLevel) );

error  = criticalValue * sqrt( diag(V) / n );



names = {'mu    ','sigma ','lambda','gamma ','delta '};
%% Print output
fprintf(' %d%% Confidence Intervals for the parameters\n\n',100*significanceLevel);

for j=1:5
    fprintf('%6s   %4.4f      +/-  %4.4f \n',names{j},param(j),error(j));

end


x = sort(returns);
paramTrue = [0.08; 0.22; 100; 0.05; 0.005];

out = EstPdfGK( x,250);
t = out.x;
g = out.fn;
[f]   = MertonPdf(t',param,h);
[fTrue]   = MertonPdf(t',paramTrue,h);
%addpath('../Chapter03');
figure
plot(t,f,'-')
hold on
plot(t,g,'r-.');
% hold on
% plot(t,fTrue,'k:')
%legend('MertonEst','EstGK','MertonTrue','location','best')
legend('MertonEst','EstGK','location','best')

pseudos = MertonCdf(x,param,h);
[Edf, pts] = EstCdf( x );

figure
plot(x,pseudos,'-')
hold on
plot(pts,Edf,'r-.')
legend('Merton','Empirical','location','best')
%rmpath(genpath('../'))

end

%%  transformation of parameters
function param = T(theta)

param    = exp(theta);
param(1) = theta(1);
param(4) = theta(4);
end

%%  Jacobian matrix
function D = Jacob(theta)

param = T(theta);

D = diag(param);
D(1,1) = 1;
D(4,4) = 1;
end


%%
function LL = LogLikMerton( theta, returns, h)
% This function computes minus the log-likelihoods 
%
% Input
%        theta = (mu, log(sigma),log(lambda),gamma,log(delta))
%        returns
%        n (length of returns )
%
% Output
%        LL: -log-likelihood
%

n = length(returns);

param = T(theta);

% if(param(3) < 10)
%  LL = 1.0e+20*ones(n,1);
%  return
% end

LL = -log(MertonPdf(returns, param, h));

if any(isinf( LL )) | any(isnan( LL )) | any(imag( LL ))
    LL = 1.0e+20*ones(n,1);
    return
end
end


