function [param, error] = EstMertonCum( data, method, h, significanceLevel)
% This function estimates the parameters of the Merton Jump-Diffusion
% model using the cunulant-mathching method. The Jacobian can be estimated 
% with the output of fsolve  or with the numerical Jacobian. 
% 
% Input    
%           data: prices (column vector);
%           method: 'Jacobian' (default), 'num';
%           h: time scale; for daily data, h=1/252 (default);
%           significanceLevel (95% default).
%
% Output:   param: estimation of the parameter (annual time scale):
%            mu, sigma, lambda, gamma, delta;
%           error:  significanceLevel% CI error for the parameters;
%           V: estimation of covariance matrix.
% Example: 
%  [param,error, V] = EstMertonCum(ApplePrices);

%%

if nargin < 2
    method = 'Jacobian';
    significanceLevel = 0.95;
    h = 1/252;
elseif nargin < 3
    significanceLevel = 0.95;
    h = 1/252;
elseif nargin < 4
    significanceLevel = 0.95;
end


%% compute and plot log-returns
returns = diff( log( data ), 1 );
n       = length( returns );



%%
[kappa0, V00] = EstCumulants( returns );

kappa   = kappa0(1:5)/h;
V0       = V00(1:5,1:5)/h^2;

%%
initialGuess  = [ log(mean(exp(returns)))/h; log(std(returns)/sqrt(h)); log(1000 ); 0; log( .01 ) ];



options = optimset( 'largescale',   'off',...
                    'MaxFunEvals',  50000,...
                    'MaxIter',      10000,...
                    'Display',      'off');

[para,Fval,exitflag,output,J] = fsolve( @(x) kappaMerton(x,kappa,1), initialGuess, options );

%%
param(1) = para(1);
param(2) = exp(para(2));
param(3) = exp(para(3));
param(4) = para(4);
param(5) = exp(para(5));

switch method
    case 'Jacobian'
        
fprintf('\n Jacobian computed from fsolvve (Appendix B.5.1)\n\n');


    case 'num'
        J = NumJacobian(@(x) kappaMerton(x,kappa,1),para,0.001);
        fprintf('Jacobian computed with the numerical gradient\n\n');
       
end

%% 
D          = diag( [1 param(2) param(3) 1 param(5)] );
M          = inv(J);
V   = D * M * V0 * M' * D;

criticalValue = norminv( 0.5 *(1+significanceLevel) );

error = criticalValue * sqrt( diag( V ) / n );



names = {'mu    ','sigma ','lambda','gamma ','delta '};
%% Print output
fprintf(' %d%% Confidence Intervals for the parameters\n\n',100*significanceLevel);

for j=1:5
    fprintf('%6s   %4.4f      +/-  %4.4f \n',names{j},param(j),error(j));

end

%% Graphs
x = sort(returns);
paramTrue = [0.08; 0.22; 100; 0.05; 0.005];

out = EstPdfGK( x,250);
t = out.x;
g = out.fn;
[f]   = MertonPdf(t',param,h);
%[fTrue]   = MertonPdf(t',paramTrue,h);
%addpath('../Chapter03');
figure
plot(t,f,'-')
hold on
plot(t,g,'r-.')
% hold on
% plot(t,fTrue,'k:')
% legend('MertonEst','EstGK','MertonTrue','location','best')
legend('MertonEst','EstGK','location','best')

pseudos = MertonCdf(x,param,h);
[Edf, pts] = EstCdf( x );

figure
plot(x,pseudos,'-')
hold on
plot(pts,Edf,'r-.')
legend('Merton','Empirical','location','best')
%rmpath(genpath('../'))

end

function F = kappaMerton( param, kappa, h )

mu      = param( 1 );
sigma   = exp( param( 2 ) );
lambda  = exp( param( 3 ) );
gamma   = param( 4 );
delta   = exp( param( 5 ) );

F = kappa - CumJumpDiffMerton( mu, sigma, lambda, gamma, delta, h );

end

function cum = CumJumpDiffMerton( mu, sigma, lambda, gamma, delta, h )
%  Cumulants of the jump-diffusion model of Merton
%
% [kappa] = CumJumpDiffMerton(0.08,0.12,1000,-0.02,0.01,1/250);

cum     = zeros(5,1);

kappa   = exp( gamma + delta^2 / 2) - 1;

alpha   = h * ( mu - lambda * kappa - sigma^2 / 2 );

cum(1) = alpha + lambda * h * gamma;
cum(2) = h* ( sigma^2 +  lambda * ( delta^2 + gamma^2 ) );
cum(3) = lambda * h * ( gamma^3 + 3 * gamma * delta^2 );
cum(4) = lambda * h * ( gamma^4 + 6 * gamma^2 * delta^2 + 3 * delta^4 );
cum(5) = lambda * h * ( gamma^5 + 10 * gamma^3 * delta^2 + 15 * gamma * delta^4 );
end