function [param, error, V] = EstVG( data, method, h, significanceLevel )
% This function estimates the parameters of the Variance Gamma
% model using a numerical method to maximize the likelihood. Fisher information
% can be estimated with the Hessian matrix (fminunc output), with the exact
% gradients, or with the numerical gradients. See Appendix B.5.1 for more
% details.
% 
% Input    
%           data: prices (column vector);
%           method: 'Hessian' (default), 'num';
%           h: time scale; for daily data, h=1/252 (default);
%           significanceLevel (95% default).
%
% Output:   Param: estimation of the parameter (annual time scale):
%            mu, sigma, alpha;
%           error:  significanceLevel% CI error for the parameters;
%           V: estimation of covariance matrix.
% Example: 
%  [param,error, V] = EstVG(DataVG);


if nargin < 2
    method = 'Hessian';
    significanceLevel = 0.95;
    h = 1/252;
elseif nargin < 3
    significanceLevel = 0.95;
    h = 1/252;
elseif nargin < 4
    significanceLevel = 0.95;
end

%% compute and plot log-returns
returns = diff( log( data ), 1 );
n       = length( returns );
sreturns = sort(returns);
%% PRE-OPTIMIZATION
 %theta = (log(alpha),log(beta),theta)
 
sigma0 = std(returns);
alpha0 = 30;
mu0     = mean(returns);

param0 = [mu0;sigma0;alpha0];


 %do not start with a too small value; we are working on the annual time scale for the likelihood
initialGuess    = [mu0; log(param0(2:3))]; % Initial values of the parameters
 





options         = optimset('largescale','on','MaxFunEvals',50000,'MaxIter',20000,'Display','iter');

%% OPTIMIZATION
[theta, f, exitflag, output, grad, hessian] = fminunc( @(x) sum(LogLikVG( x, returns,1)), initialGuess, options );

% remove persistent variables
clear functions

%% OUTPUT
param =zeros(3,1);
param(2:3) = exp(theta(2:3));
param(1) = theta(1);

D0 = diag([1/h; 1/sqrt(h); 1/h]);


param = D0*param;

 addpath(genpath('../'))
 %addpath('../Chapter03','../Chapter04''../ChapterB')
%%  Fisher information matrix 
fprintf('\n\n');


lmin = min(eig(hessian));

if(lmin<0)
 fprintf('\n The Hessian is not positive definite. Estimation errors are not reliable. The numerical method will be used instead\n\n');
 method = 'num';
end

switch method
    case 'Hessian'
        FI = hessian/n;
fprintf('\n Fisher information computed with the numerical Hessian from fminunc (Appendix B.5.1)\n\n');


    case 'num'
        J = NumJacobian(@(x) LogLikVG(x,sreturns,1),theta,0.001);
        fprintf('Fisher information computed with the numerical gradient (Appendix B.5.1)\n\n');
        FI = cov(J);
end



%% Corrections

D = Jacob(theta); %Jacobian matrix of the transformation for positiveness


V = D*pinv(FI)*D;


%% Precision

criticalValue = norminv( 0.5 *(1+significanceLevel) );

error  = criticalValue * sqrt( diag(V) / n );

names = {'mu   ', 'sigma','alpha'};
%% Print output
fprintf(' %d%% Confidence Intervals for the parameters\n\n',100*significanceLevel);

for j=1:3
    fprintf('%6s   %4.4f      +/-  %4.4f \n',names{j},param(j),error(j));

end


%%
paramTrue = [0.15; 0.5; 1000];

out = EstPdfGK( sreturns,250);
t = out.x;
g = out.fn;
[f]   = VGPdf(t',param,h);
[fTrue]   = VGPdf(t',paramTrue,h);
%addpath('../Chapter03');
figure
plot(t,f,'-')
hold on
plot(t,g,'r-.')
% hold on
% plot(t,fTrue,'k:')
%legend('VGEst','EstGK','VGTrue','location','best')
legend('VGEst','EstGK','location','best')



%rmpath(genpath('../'))

end

% %%  transformation of parameters
% function param = T(theta);
% 
% param    = exp(theta);
% param(3) = theta(3);
% 
% end

%%  Jacobian matrix
function D = Jacob(theta);

D = diag(exp(theta));
%D(3,3) = 1;

end


%%
function LL = LogLikVG( theta, returns,h)
% This function computes minus the log-likelihoods 
%
% Input
%        theta = (mu, log(sigma),log(lambda));
%        returns;
%        m = mean(returns)/h;
%        n (length of returns ).
%
% Output
%        LL: -log-likelihood.
%


param = zeros(3,1);

param(2:3) = exp(theta(2:3));
param(1) = theta(1);



f = max(VGPdf(returns, param, 1), 1E-20);

LL = -log(f);

end


