function [param, error, V] = EstVGMean( data, method, h, significanceLevel )
% This function estimates the parameters of the Variance Gamma
% model using a numerical method to maximize the likelihood, coupled with 
% a moment-estimation of the mean. Fisher information
% can be estimated with the Hessian matrix (fminunc output), with the exact
% gradients, or with the numerical gradients. See Appendix B.5.1 and B.9 
% for more details.
% 
% Input    
%           data: prices (column vector);
%           method: 'Hessian' (default), 'num';
%           h: time scale; for daily data, h=1/252 (default);
%           significanceLevel (95% default).
%
% Output:   Param: estimation of the parameter (annual time scale):
%            mu, sigma, alpha;
%           error:  significanceLevel% CI error for the parameters;
%           V: estimation of covariance matrix.
% Example: 
%  [param,error, V] = EstVGMean(DataVG);


if nargin < 2
    method = 'Hessian';
    significanceLevel = 0.95;
    h = 1/252;
elseif nargin < 3
    significanceLevel = 0.95;
    h = 1/252;
elseif nargin < 4
    significanceLevel = 0.95;
end

%% compute and plot log-returns
returns = diff( log( data ), 1 );
n       = length( returns );
sreturns = sort(returns);
%% PRE-OPTIMIZATION
 %theta = (log(alpha),log(beta),theta)
 
sigma0 =std(returns);
alpha0 = 30;

param0 = [sigma0;alpha0];


 %e are working on the daily  time scale for the likelihood
initialGuess    = log(param0); % Initial values of the parameters
 
m = mean(returns);


options         = optimset('largescale','on','MaxFunEvals',50000,'MaxIter',20000,'Display','iter');

%% OPTIMIZATION
[theta, f, exitflag, output, grad, hessian] = fminunc( @(x) sum(LogLikVGMean( x, sreturns,m,1)), initialGuess, options );

% remove persistent variables
clear functions

%% OUTPUT

param = zeros(3,1);
param(2:3) = exp(theta);
param(1) = m;

D0 = diag([1/h; 1/sqrt(h); 1/h]);


param = D0*param;



 addpath(genpath('../'))
 %addpath('../Chapter03','../Chapter04''../ChapterB')
%%  Fisher information matrix 
fprintf('\n\n');

 J = NumJacobian(@(x) LogLikVGMean(x,sreturns,m,1),theta,0.001)
lmin = min(eig(hessian));

if(lmin<0)
 fprintf('\n The Hessian is not positive definite. Estimation errors are not reliable. The numerical method will be used instead\n\n');
 method = 'num';
end

switch method
    case 'Hessian'
        FI = hessian/n;
fprintf('\n Fisher information computed with the numerical Hessian from fminunc (Appendix B.5.1)\n\n');


    case 'num'
        fprintf('Fisher information computed with the numerical gradient (Appendix B.5.1)\n\n');
        FI = cov(J);
end



%% Corrections

D = Jacob(theta); %Jacobian matrix of the transformation for positiveness


V22 = D*pinv(FI)*D;

V11 = var(returns);

V0 = mean( repmat(returns,1,2) .* J );

V12 = V0 * V22;

V = D0*[ V11 V12; V12' V22]*D0;


%% Precision

criticalValue = norminv( 0.5 *(1+significanceLevel) );

error  = criticalValue * sqrt( diag(V) / n );

names = {'mu   ', 'sigma','alpha'};
%% Print output
fprintf(' %d%% Confidence Intervals for the parameters\n\n',100*significanceLevel);

for j=1:3
    fprintf('%6s   %4.4f      +/-  %4.4f \n',names{j},param(j),error(j));

end


%%

paramTrue = [0.15; 0.5; 1000];

out = EstPdfGK( sreturns,250);
t = out.x;
g = out.fn;
[f]   = VGPdf(t',param,h);
[fTrue]   = VGPdf(t',paramTrue,h);
%addpath('../Chapter03');
figure
plot(t,f,'-')
hold on
plot(t,g,'r-.')
% hold on
% plot(t,fTrue,'k:')
% legend('VGEst','EstGK','VGTrue','location','best')
legend('VGEst','EstGK','location','best')




%rmpath(genpath('../'))

end


%%  Jacobian matrix
function D = Jacob(theta);

D = diag(exp(theta));
%D(3,3) = 1;

end


%%
function LL = LogLikVGMean( theta, returns, m, h)
% This function computes minus the log-likelihoods when the mean is given.
%
% Input
%        theta = (mu, log(sigma),log(lambda));
%        returns;
%        m = mean(returns)/h;
%        n (length of returns ).
%
% Output
%        LL: -log-likelihood.
%



param(2:3) = exp(theta);
param(1) = m;



f = max(VGPdf(returns, param, h), 1E-20);

LL = -log(f);

end


