function S = SimJumpDiffKou( param, n, h, graph)
% Simulation of a path of Kou's Jump Diffusion with 
% weighted-symmetric exponential jumps.  
%
% Generates daily data 
%
% Input
%       param: mu, sigma>0, lambda>0, 1> p>0, omega = eta2/eta1 > 0, theta, 
%              delta = 1/eta2>0  (annual parameters);
%       n: number of observations;
%       h: time between observations (default h =1/252 corresponding to
%       daily onservations);
%       graph: 1 if graph is requires, 0 otherwise (default).
%
%
%  Output
%       S: prices, with S(0) = 100.
%
% S = SimJumpDiffKou([0.08,0.12,100,0.7,0.8,0,0.01],500)


if nargin < 3
    h = 1/252;
    graph = 0;
elseif nargin <4
    graph=0;
end

mu     = param(1);
sigma  = param(2);
lambda = param(3);
p      = param(4);
omega  = param(5);
theta  = param(6);
delta  = param(7);



mu1             = mu * h;
sig1            = sigma * sqrt( h );
jumpIntensity   = lambda * h;
kappa           = ( 1 - p ) * exp( theta ) / ( 1 + delta ) + p * exp( theta ) / ( 1 - delta * omega ) - 1;

nbJumps = poissrnd( jumpIntensity, n, 1 );
Z = zeros(n,1);
maxNbJumps = max(nbJumps);

X = cumsum( theta + delta * SimWSExp( n, maxNbJumps, p, omega ), 2);
for j = 1:n
    if nbJumps(j) > 0
        Z(j) = X( j, nbJumps(j) );
    end
end
Z0  = ( mu1 - jumpIntensity * kappa - sig1^2 / 2 ) + sig1 * randn( n, 1 );
S   = 100 * exp( cumsum( Z + Z0 ) );

if(graph)
figure
plot( S );
title({['Kou''s  jump diffusion with \mu = ',num2str(mu),...
    ' ,\sigma = ', num2str(sigma), ' , \lambda = ', num2str(lambda), ...
    ' and weighted-symmetric'];...
   [ '  exponential jump parameters p = ', num2str(p), ' ,\omega = ',num2str(omega), ...
    ' , \theta = ', num2str(theta), ' , \delta = ', num2str(delta)]});
end
end

function  [X] = SimWSExp( n, m, p, omega )

Z = - log( rand( n, m ) );

U = rand(n,m);

X = -Z.*( U > p ) + omega * Z .* ( U < p );

end