function S = SimJumpDiffMerton(param, n, h, graph )
% Simulation of a path of Merton's Jump Diffusion with Gaussian jumps of
% mean gamma and variance delta^2 observed at periods $kh$.
%
% Input
%       param: mu, sigma>0, lambda>0, gamma, delta>0 (annual parameters);
%       n: number of observations;
%       h: time between observations (default h =1/252 corresponding to
%       daily onservations);
%       graph: 1 if graph is requires, 0 otherwise (default).
%
%  Output
%       S: prices, with S(0) = 100.
%
% Example:
%   S = SimJumpDiffMerton([0.08,0.12,1000,-0.02,0.01],500)

if nargin < 3
    h = 1/252;
    graph = 0;
elseif nargin < 4
    graph= 0;
end

mu     = param(1);
sigma  = param(2);
lambda = param(3);
gamma  = param(4);
delta  = param(5);


mu1             = mu * h;
sig1            = sigma * sqrt( h );
jumpIntensity   = lambda * h;
kappa           = exp( gamma + delta^2 / 2 ) - 1;

a =     mu1 - jumpIntensity * kappa - 0.5*sig1^2; 

nbJumps         = poissrnd( jumpIntensity, n, 1 );
Z               =a + gamma * nbJumps +  sqrt( sig1^2 + delta^2 * nbJumps ) .* randn( n, 1 );
S               = 100 * exp( cumsum( Z ) );

if(graph)
figure
plot( S );
title({['Merton''s jump diffusion with \mu = ',num2str(mu),...
    ' ,\sigma = ', num2str(sigma), ' , \lambda = ', num2str(lambda)];...
    ['  and Gaussian jump parameters  \gamma = ', num2str(gamma), ' ,\delta = ',num2str(delta)]});
end