function S = SimJumpDiffWSG( n, mu, sigma, lambda, p, omega, theta, delta, h, graph)
% Simulation of a path of a jump diffusion model with 
% weighted-symmetric Gaussian jumps 
%
% Generates daily data 
%
% Input
%       n: number of years,
%       mu, sigma>0, lambda>0, gamma, delta>0: annual parameters;
%       h: time between observations (default h =1/252 corresponding to
%       daily onservations);
%       graph: 1 if graph is requires, 0 otherwise (default).
%
%  Output
%       S: prices, with S(0) = 100.
%
% S = SimJumpDiffWSG(2,0.08,0.12,100,0.7,0.8,-0.02,0.01)

if nargin < 9
    h = 1/252;
    graph = 0;
elseif nargin < 10
    graph = 0;
end
    

m               = n / h;
mu1             = mu * h;
sig1            = sigma * sqrt( h );
jumpIntensity   = lambda * h;
kappa           = 2 * ( 1 - p ) * exp( theta + delta^2 / 2 ) * normcdf( - delta ) + 2 * p * exp( theta + delta^2 * omega^2 / 2 ) * normcdf( delta * omega ) - 1;

nbJumps         = poissrnd( jumpIntensity, m, 1 );
Z               = zeros(m,1);
maxNbJumps      = max(nbJumps);

X = cumsum( theta + delta * SimWSGauss( m, maxNbJumps, p, omega ), 2);
for j = 1:m
    if nbJumps(j) > 0
        Z(j) = X( j, nbJumps(j) );
    end
end
Z0  = ( mu1 - jumpIntensity * kappa - sig1^2 / 2 ) + sig1 * randn( m, 1 );
S   = 100 * exp( cumsum( Z + Z0 ) );

%%
if(graph)
figure
plot( S );
title({['Merton''s weighted symmetric jump diffusion with \mu = ',num2str(mu),...
    ' ,\sigma = ', num2str(sigma), ' , \lambda = ', num2str(lambda)];...
   [ ' and jump parameters p = ', num2str(p), ' ,\omega = ',num2str(omega), ...
    ' , \theta = ', num2str(theta), ' , \delta = ', num2str(delta)]});

end
end

%%
function  [X] = SimWSGauss( n, m, p, omega )

Z = abs( randn( n , m ) );

U = rand(n,m);

X = - Z .* ( U > p ) + omega * Z .* ( U < p );

end