function [param, innovations, sigma, residuals, error, cov_est] = EstGARCHGED(data,p,q)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Maximum likelihood estimation of the parameters of the GARCH(p,q) model
%  with constant mean mu and GED innovations.
%  
% Input
%      data:  returns (n x 1);
%         p:  number of parameters for the GARCH coefficients;
%         q:  number of parameters for the  ARCH coefficients.
%
% Output
%       param: estimated parameters [mu; omega; garchcoef; archcoef; nu];
%       innovations: unstandardized residuals;
%       sigma: estimated conditional standard deviations;
%       residuals : standardized innovation estimates;
%       error: estimator error for each parameter;
%       cov_est: estimation of the limiting covariance between errors.
%
% Last modification: February 2 2015
% interior-point instead of active-set in optimset

n = length( data );

% initial guess
theta0 = [mean(data); var(data) * ( 1 - 0.85 - 0.05);  0.85 * ones( p, 1) / p; 0.05 * ones( q, 1) / q; 2];
eps = 1E-10;
                                    
%% Options for fminunc
options = optimset( 'largescale',   'off', ...
                    'Algorithm', 'interior-point', ...
                    'MaxFunEvals',  10000, ...
                    'MaxIter',      10000, ...
                    'Display', 'off');

%% Maximisation of the log-likelihood

[theta, fval, exitflag, output, qs, grad, hessian] = fmincon( @(x) LogLikGARCHGED( x, data, n, p, q), theta0, [0 0 ones( 1, q + p ) 0], 1 - eps, [], [], [-1; eps*ones( 2 + q + p, 1)], [], [], options);
clear functions

% get estimated parameters and filtered conditional variances;
[L, G, H, mu, omega,  garchcoef, archcoef, nu, h, residuals] = LogLikGARCHGED( theta, data, n, p, q );

I_Fisher    = hessian / n;
cov_est     = inv(I_Fisher);

error       = 1.96 * sqrt( diag( cov_est / n ) );

sigma = sqrt(h);
innovations = residuals .* sigma;

param.Distribution = 'GED';
param.C     = mu;
param.P     = p;
param.Q     = q;
param.K     = omega;
param.GARCH = garchcoef;
param.ARCH  = archcoef;
param.Shape = nu;

end

function  [L, G, H, mu, omega, beta, alpha, nu, h, z] = LogLikGARCHGED( theta, x, n, p, q )

persistent r logLikelihoodFactor sampleVariance
if isempty( r )
    r                   = max( p, q );
    
    sampleVariance      = cov( x );
end

% get parameters
mu      = theta(1);
omega   = theta(2);
beta   = theta( 3:( p + 2 ) );
alpha    = theta( ( p + 3 ):end-1 );
nu      = theta(end);

a= 1/nu;
 thetanu = sqrt(gamma(a)/gamma(3*a))/2^a;
 

% filter conditional variances
h         = zeros( n, 1 );
h( 1:r )  = sampleVariance;
for t = r+1:n
    
    indQ = t - (1:q)';
    indP = t - (1:p)';
    
    h( t ) = omega + alpha' * ( ( x( indQ ) - mu ).^2 ) + beta' * h( indP );
    
end



z = ( x - mu ) ./ sqrt( h );

L =  sum( 0.5*log( h ) + 0.5*(abs(z)/thetanu).^nu + log(thetanu/nu)+log(gamma(a))+(1+a)*log(2) );

G = [];
H = [];

if isinf( L ) | imag( L ) | isnan( L )
    L = 1.0e+20;
end

end