%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Maximum likelihood estimation of the parameters of the GARCH(p,q) model
%  with constant mean mu and Gaussian innovations.
%  
% Input
%      data:  returns (n x 1);
%         p:  number of parameters for the GARCH coefficients;
%         q:  number of parameters for the  ARCH coefficients;
%     method: string array precising how to estimate Fisher information: 
%             'Hessian' for the numerical Hessian from fmincon, or 'exact' 
%             (default) for the exact gradients;
%    significanceLevel: level for confidence intervals. Default is 0.95.
%
% Output
%       param: estimated parameters 
%          [mu, arcoef, macoef, regcoef, omega, garchcoef, archcoef];
%       innovations: unstandardized residuals;
%       sigma: estimated conditional standard deviations;
%       residuals : standardized innovation estimates;
%       error: estimator error for each parameter;
%       cov_est: estimation of the limiting covariance between errors;
%       W: statistics needed for the multipliers method (Ghoudi & Remillard
%       2012).
%
% Last modification: February 2 2015
% interior-point instead of active-set in optimset


function [param, innovations, sigma, residuals, error, cov_est, W] = EstGARCHGaussian(data, p, q, method, significanceLevel)

%%
if nargin < 4
    method = 'exact';
    significanceLevel = 0.95;
elseif  nargin < 5
    significanceLevel = 0.95;
end

n = length( data );
m = p+q+2;


%% Here we uses normalized data as in library fGarch

s = std(data);
datastd = data/s;

p0=.85;
q0=.05;
eps = 1E-10;
% In library fGarch, p0=.8 and q0=.1;
% Initial guess as in garchfit of Matlab

theta0 = [mean(datastd); ( 1 - p0 - q0);  p0 * ones( p, 1) / p; q0 * ones( q, 1) / q];

                                    
%% Options for fminunc
options = optimset( 'largescale',   'off', ...
                    'Algorithm', 'interior-point', ...
                    'MaxFunEvals',  10000, ...
                    'MaxIter',      10000, ...
                    'Display', 'off');

%% Maximisation of the log-likelihood

[thetastd, fval, exitflag, output, qs, gradnum, hessianstd] = fmincon( @(x) LogLikGARCHGaussian( x, datastd, n, p, q), theta0, [0 0 ones( 1, q + p ) ], 1 - eps, [], [], [-1; eps*ones( 1 + q + p, 1)], [], [], options);
%clear functions
%exitflag
param0 = thetastd;
param0(1) = s*thetastd(1);
param0(2) = s^2*thetastd(2);
% get estimated parameters and filtered conditional variances;
[L, ~, ~, sigma, residuals] = LogLikGARCHGaussian(param0, data, n, p, q );



%%  Fisher information matrix 
%fprintf('\n\n');

lmin = min(eig(hessianstd));

if(lmin<0)
% fprintf('\n The Hessian is not positive definite. Estimation errors are not reliable. The exact method will be used instead.\n\n');
 method = 'exact';
end

switch method
    case 'Hessian'
        W =[];
       I_Fisher    = hessianstd / n;   %one must change it so that I_Fisher is non-negative definite.

       D = diag([s; s^2; ones(p+q,1)]);
       param0 = D*thetastd;
       cov_est = D*pinv(I_Fisher)*D;
%fprintf('\n Fisher information computed with the numerical Hessian from fmincon (Appendix B.5.1)\n\n');


    case 'exact'
      
 %       fprintf('Fisher information computed with the exact gradients (Appendix B.5.1)\n\n');
        
 y = data-param0(1); 
h = sigma.^2;
dh = zeros(n,m);

W0 = zeros(n,m);
r = max(p,q);
if(p==0)
    phi = 0; indp=0;
else
phi = param0(3:p+2);
indp = -(1:p)';
end

if(q==0)
    psi = 0; indq=0;
else
psi = param0(p+3:m);
indq = -(1:q)';
end

for l=(r+1):n
    hi = h((l+indp));
    yi = y((l+indq));
    dh(l,:) = phi'*dh(l+indp,:);
    dh(l,2) = 1+dh(l,2);
    dh(l,3:p+2) = hi'+dh(l,3:p+2);
    dh(l,p+3:m) = dh(l,p+3:m)+(yi.^2)';
    dh(l,1) = dh(l,1)-2*psi'*yi;
end

for i=1:n
    W0(i,:) = 0.5*(residuals(i)^2-1)*dh(i,:)/h(i);
    W0(i,1) = W0(i,1)+residuals(i)/sigma(i);
end

W= W0-repmat(mean(W0),n,1);  

J = cov(W);


Jinv0 = pinv(J);
Jinv = 0.5*(Jinv0+Jinv0');
cov_est =Jinv;
end

criticalValue = norminv( 0.5 *(1+significanceLevel) );

    
error       = criticalValue * sqrt( diag( cov_est / n ) );


innovations = residuals .* sigma;

param.Distribution = 'Gaussian';
param.C     = param0(1);
param.P     = p;
param.Q     = q;
param.K     = param0(2);
param.GARCH = param0(3:p+2);
param.ARCH  = param0(p+3:m);
param.Shape = [];


end

function  [L, G, H, sigma, stdres] = LogLikGARCHGaussian( theta, x, n, p, q )

persistent r logLikelihoodFactor
if isempty( r )
    r                   = max( p, q );
 logLikelihoodFactor = 0.5 * n * log( 2 * pi );   
    
end

% get parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
mu      = theta(1);
omega   = theta(2);
garchcoef   = theta( 3:( p + 2 ) );
archcoef   = theta( ( p + 3 ):end );
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

y = x-mu;

vary = mean(y.^2);

persistence = sum(garchcoef)+sum(archcoef);

% garchcoef
% archcoef
% filter conditional variances
h         = zeros( n, 1 );
h( 1:r )  = omega + persistence*vary;
for t = r+1:n
    
    indQ = t - (1:q)';
    indP = t - (1:p)';
    
    h( t ) = omega + archcoef' * ( ( y( indQ )).^2 ) + garchcoef' * h( indP );
    
end

sigma = sqrt( h );

stdres = ( x - mu ) ./ sigma ;

L =  logLikelihoodFactor + 0.5*sum( log( h ) + stdres.^2 );

G = [];
H = [];

if isinf( L ) | imag( L ) | isnan( L )
    L = 1.0e+20;
end
end