function [CVM,KS,CVMPvalue,KSPvalue, KS95, pseudos] = GofGARCHGED(residuals, data, param, N, graph)
% Compute the Cramer-von Mises and Kolmogorov statistics for a goodness-of-fit 
% test on the innovations of a GARCH(p,q) model with GED innovations. The 
% p-values are estimated with N parametric bootstrap replications. 
% 
%
% Inputs
%        residuals: residuals of the estimated model;
%        data: original time series;
%        param: list of parameters; (param.C, param.K, param.GARCH, param.ARCH,param.Shape);
%        N: number of replications for the parametric bootstrap;
%        graph: if 1, a graph of the empirical distribution of the 
%        pseudo-observations is plotted; 0 is the default.
%
% Outputs
%       CVM  : Cramer-von Mises statistic,
%       KS   : Kolmogorov-Smirnov statistic,
%       CVMPvalue : p-value(%) of CVM;
%       KSPvalue  : p-value(%) of KS;
%       KS95: 95% quantile of KS.
%
% Reference: Remillard, B. (2011) 'Validity of the parametric bootstrap
%            for goodness-of-fit testing in dynamic models', SSRN Working 
%            Paper Series No. 1966476.

%% Estimation of parameters

if nargin <9
    graph=0;
end

nu = param.Shape;
garchcoef = param.GARCH;
archcoef  = param.ARCH;
mu        = param.C;
omega     = param.K;


U = sort(GEDCdf(residuals,nu));

pseudos = U;
%% Computation of statistics

n = length(residuals);
t = ((1:n)'-0.5)/n;
p = length(garchcoef);
q = length(archcoef);
r = max(p,q);

[hh,pp,ss] = kstest(norminv(U));  
KS = sqrt(n)*ss;

sn0 = (U-t)'*(U-t);
CVM = sn0 +1/(12*n);

%% Parametric bootstrap

SnB =zeros(N,1);

KnB =zeros(N,1);

  x0 = data(1:r);
  h0 = ones(r,1)*cov(data);
parfor k=1:N
  %%  Simulation
 
  
  xnew = SimGARCHGED(param, x0, h0, n);
  
    
  %% Estimation
    
  [paramn, innovationsn, sigman, resn] = EstGARCHGED(xnew,p,q);


  %% Computation of statistics

Un = sort(GEDCdf(resn,paramn.Shape));  

[hh,pp,ss] = kstest(norminv(Un));
KnB(k) = sqrt(n)*ss;

sn0 = (Un-t)'*(Un-t);
SnB(k) = sn0 +1/(12*n);

end

CVMPvalue = 100*mean(SnB>CVM);
KSPvalue = 100*mean(KnB>KS);
KS95 = quantile(KnB,0.95);


fprintf('   Craver-von Mises statistic: %4.4f, P-Value (%%): %3.2f\n',CVM,CVMPvalue);
fprintf(' Kolmogorov-Smirnov statistic: %4.4f, P-Value (%%): %3.2f\n',KS,KSPvalue);
%% Graph

if(graph)
   D = [0; (1:n)'/n];
   t = [0; pseudos];
   ub = t+KS95/sqrt(n);
   lb = t-KS95/sqrt(n);
   figure
   plot(t,D,'k');
   hold on
   plot(t,ub,'k:')
   hold on
   plot(t,lb,'k:')
end

end
%%
function U = GEDCdf(x,nu)

 a= 1/nu;
 theta = sqrt(gamma(a)/gamma(3*a))/2^a;
 
 s = sign(x);
 
 z = 0.5* (abs(x)/theta).^nu;
 
 U = 0.5*(1+s.*gamcdf(z,a,1));
end

