function [KS, CVM, W] = GofGARCHGaussian(x, method, test)
% Test of normality for the innovations of ARMA-GARCH models based on
% Khmaladze transform. An earlier implementation was proposed in Bai(2003). 
% This new implementation, using midpoint method for integration, was 
% proposed by K. Ghoudi. The user can also choose the Gauss-Kronrod 
% quadrature. See, e.g., Ghoudi & % Remillard (2012).
%
% Input
%          x: Standardized residuals of the model;
%          method: 'midpoint' integration (default) or 'quadgk' quadrature;
%          test: 1 for the printed output of the results and a graph of the 
%                Brownian path, 0 otherwise (default).
%
%           
% Output
%         KS: Kolmogorov-Smirnov test statistic;
%        CVM: Cramer-von Mises test statistic;
%          W: Approximated Brownian motion at points t(j) = j/n.
%

%%
if nargin<2
    test = 0;
    method = 'midpoint';
elseif nargin<3
    test = 0;
end

%%
 n = length(x);
 y = sort(x);
 v = normcdf(y);
 v0 = [0;v];
 
 v1 = [v;1];

 vv = [0;v;1];
 W = zeros(n,1);
 
 gd1 = ones(n,1); 
 gd2 = -y; 
 gd3 = 1-y.^2;
  gd = [gd1 gd2 gd3];
 gd0 = [0 0 0;gd];
 D0  = cumsum(gd0);
  Dk  = (repmat(D0(end,:),n,1)-D0(1:n,:))/n;
 
 
SUM = zeros(n,1);

%% Quadrature
switch method
    case 'midpoint'
        
dv = diff(v0);

mdv = repmat(dv,1,3);
 
midv = 0.5*(v0+v1);
midv = midv(1:n);
[y1 y2 y3] = BaiCg(midv);

yy0 = [y1 y2 y3];
yy = mdv .* yy0;

    case 'quadgk'    % Gauss-Kronrod quadrature 
        
   yy  = zeros(n,3);

    for j = 1:n
     yy(j,1) = quadgk( @BaiCg1, v0( j ), v0( j+1 ));
     yy(j,2) = quadgk( @BaiCg2, v0( j ), v0( j+1 ));
     yy(j,3) = quadgk( @BaiCg3, v0( j ), v0( j+1 ));
    end

end

%% Brownian motion
SUM = 1 / n - sum(yy .* Dk , 2);
 W  = sqrt( n ) * cumsum( SUM );
 

  W0 = W-1/sqrt(n);
  KS = max(max(abs(W)),max(abs(W0)));
 CVM = sum ((W.^2) .* diff(v1));
  
%% printed output
 if(test)
    QKS =  [1.960  2.241 2.807];

    QCVM = [1.200  1.657 2.800];
    
    fprintf('\n\nK-S test statistic: %1.4f\n', KS);
    fprintf('\nQuantiles of order 90, 95 and 99 percents for the K-S test: %1.3f %1.3f %1.3f\n', QKS(1),QKS(2),QKS(3));
    fprintf('\n\nCVM test statistic: %1.4f\n', CVM);
    fprintf('\nQuantiles of order 90, 95 and 99 percents for the CVM test: %1.3f %1.3f %1.3f\n', QCVM(1), QCVM(2), QCVM(3));
fprintf('\n');
if (CVM > QCVM(3))
    fprintf('P-Value of CVM smaller than 1%%\n');
elseif (CVM > QCVM(2))
    fprintf('P-Value of CVM smaller than 5%%\n');
elseif  (CVM > QCVM(1))
    fprintf('P-Value of CVM smaller than 10%%\n');
else
    fprintf('P-Value of CVM larger than 10%%\n');
end
  

if (KS > QKS(3))
    fprintf('P-Value of KS  smaller than 1%%\n');
elseif (KS > QKS(2))
    fprintf('P-Value of KS  smaller than 1%%\n');
elseif (KS > QKS(1))
    fprintf('P-Value of KS  smaller than 10%%\n');
else
    fprintf('P-Value of KS  larger than 10%%\n');
end

t = (1:n)'/n;
ub = ones(n+1,1)*QKS(2);
lb = -ub;
plot([0;t],[0;W]);
hold on
plot([0;t],ub,':');
hold on
plot([0;t],lb,':');

title('Process trajectory and uniform 95% confidence band for Brownian motion');
       
 end
        
end

%%
function [y1 y2 y3] = BaiCg( s )

a = norminv(s); 
x = normpdf(a);
[n,d] = size(s);

v1 = ones(n,d);
v2 = -a ; v3 = 1 - a.^2;

% precomputation
oneMinusS           = 1 - s;
aTimesX             = a .* x;
onesPlusASquared    = 1 + a.^2;

% computing the matrix C
C11 = oneMinusS;
C12 = - x;
C13 = - aTimesX; 
C21 = C12;
C22 = oneMinusS + aTimesX;
C23 = onesPlusASquared .* x;
C31 = C13;
C32 = C23;
C33 = 2  * oneMinusS + aTimesX .* onesPlusASquared;

%% C^{-1} x dot g
A11 = C11.*C22-C21.*C12;
A12 = C11.*C23-C21.*C13;
A21 = C11.*C32-C31.*C12;
A22 = C11.*C33-C31.*C13;

D = A11.*A22-A12.*A21;

B11 = A22./D; B12  = -A12./D; B21 = -A21./D; B22 = A11./D;

z1 = C11.*v2-C21.*v1;
z2 = C11.*v3-C31.*v1;
y2 = B11.*z1 + B12.*z2;
y3 = B21.*z1 + B22.*z2;

y1 = (v1 - C12.*y2 - C13.*y3)./C11;
         
end

%%
function y1 = BaiCg1(u)

[y1 y2 y3] = BaiCg(u);
end

%%
function y2 = BaiCg2(u)

[y1 y2 y3] = BaiCg(u);
end

%%
 function y3 = BaiCg3(u)
        
 [y1 y2 y3] = BaiCg(u);
 end