function [Returns, Vol, varlimit, Prices] = SimGARCHGED(param, x0, h0, n, graph )
% Simulation of a GARCH(p,q) with GED innovations: 
%
%          x_i = mu + sqrt(h_i) epsilon_i,
%
%          h_i = omega + sum_{k=1}^q archcoef(k)(x_{i-k}-mu)^2 
%                 + sum_{k=1}^p garchcoef(k) h_{i-k}, i=max(p,q)+1,...,n,
% with starting points x_0 and h_1.
% param: list of parameters; (param.C, param.K, param.GARCH, param.ARCH,param.Shape);
%
% Caution: The condition alpha+beta <1 is required for stationarity. In
% that case, the asymptotic variance (varlimit) is :
%       varlimit = omega/(1-alpha-beta).
%
% [Returns,Vol,varlimit,Prices] = SimGarchGED(-0.004,0.005, 0.6, 0.3,5.4,0,0.1,250,1)
%
% If graph=1, returns and volatilies will be plotted.


if nargin < 9
    graph=0;
end

Prices = [];

nu = param.Shape;
garchcoef = param.GARCH;
archcoef  = param.ARCH;
mu        = param.C;
omega     = param.K;


eps = GEDRnd(nu,n,1);

% memory allocation
v = zeros(n,1);
x = zeros(n,1);
h = zeros(n,1);
varlimit = Inf;

persistence = sum(archcoef) + sum(garchcoef);

if persistence < 1
    varlimit = omega / ( 1 - persistence );
end

% generate time series
p = length(garchcoef);
q = length(archcoef);
r = max(p,q);
h(1:r) = h0;
x(1:r) = x0;


tp = (1:p)';
tq = (1:q)';
for t = (r+1):n
    h( t ) = omega + garchcoef'* h( t- tp ) + archcoef'* ( x( t- tq ) - mu ).^2;
    x( t ) = mu + sqrt( h( t ) ) * eps( t );
    
end

t       = (1:1:n)';
Vol     = sqrt( h );
Returns = x;


if graph
    Prices  = 100*[1; exp(cumsum(x))];

    subplot(3,1,1); plot([0;t], Prices );
    title({' Prices S starting from 100';['Parameters: \mu = ' num2str(mu) ' , \omega = ' num2str(omega) ' , \alpha= ' num2str(alpha) ' , \beta = ' num2str(beta)]});
    
    subplot(3,1,2); plot( t, x );
    title([' Returns x starting from x_0 = ' num2str(x(1))]);
    
    subplot(3,1,3); plot( t, Vol );
    title([' Conditional volatilities v^{1/2} starting from h_1 = ' num2str(h(1))] );
    
end
end

%%
function X = GEDRnd(nu,n,m)

if nargin < 3
    m=1;
end  
  V = rand(n,m)-0.5;  
  eps = sign(V);
  
 a= 1/nu;
 theta0 = sqrt(gamma(a)/gamma(3*a));
 
 
 
 z = gamrnd(a,1,n,m);
 
 
 X = theta0*eps.*(z.^a);
end