function [Val, spread, Pt, PD, Q ] = BondMarkovChain( t, r )
% Compute the values of defaultable bonds using credit ratings from S&P and
% the continuous time Markov chain model.
%
% Input
%       t: Maturity in years;
%       r: annual rate.
% Output
%       Val:  zero-counpon bond,
%       spread: spread yield,
%       Pt: transition matrix t time t,
%       PD: Probability of default at time t.

%% S&P tansition matrix for one year.
P= [ ...
    90.81  , 8.33  , 0.68  , 0.06  , 0.12  , 0     , 0     , 0;
    0.70  , 90.65 , 7.79  , 0.64  , 0.06  , 0.14  , 0.02  , 0;
    0.09  , 2.27  , 91.05 , 5.52  , 0.74  , 0.26  , 0.01  , 0.06;
    0.02  , 0.33  , 5.95  , 86.93 , 5.30  , 1.17  , 0.12  , 0.18;
    0.03  , 0.14  , 0.67  , 7.73  , 80.53 , 8.84  , 1.00  , 1.06;
    0.01  , 0.11  , 0.24  , 0.43  , 6.48  , 83.46 , 4.07  , 5.20;
    0.22  , 0     , 0.22  , 1.30  , 2.38  , 11.24 , 64.86 , 19.78;
    0     , 0     , 0     , 0     , 0     , 0     , 0     ,100]/100;

P= [ 88   , 8     , 3  , 1     , 0      , 0     , 0    , 0;
    1     , 88    , 6     , 4  , 1      , 0     , 0    , 0;
    0.5   , 2     , 90    , 6     , 1      , 0.5     , 0     , 0;
    0     , 0     , 6     , 87    , 6      , 1     , 0    , 0;
    0     , 0     ,0      , 8     , 81     , 10     , 1     , 0;
    0     , 0     ,0      , 0.5    ,  3.5      , 82    , 8    ,   6;
    0     , 0     ,0      , 0.5     , 2.5   , 10    , 66    , 21;
    0,0,0,0,0,0,0,100]/100;

% face value
F = 1000;
 
% recovery rates
Z = [53.80, (53.80+51.13)/2,51.13,(51.13+38.52)/2,38.52,(38.52+17.09)/2,17.09]/100;

%% Infinitesimal generator of the continuous time Markov chain
Q = GeneratorMarkovChain( P );

[V, D0] = eig( Q );
D2      = diag( D0 );
invV    = inv( V );

Dt = exp( t * D2 );
Pt = max( 0, V * diag( Dt ) * invV );

PD = Pt(:,8)';

Val     = zeros(1,7);

for i = 1:7
    sum1 = 0;
    for k = 1:7
        for j = 1:7
            sum1  =  sum1 +  Z( k ) * Q( j, 8 ) * V( i, k ) * invV( k, j ) * ( 1 - Dt( k ) ) / D2( k );
        end
    end
    Val( i ) = min( 1, 1 - PD(i) - sum1 );
end

spread  = 100 * ( - log( Val ) / t ); % annual spread in (%)
B       = F * exp( - r * t );
Val     = B * Val;
end
%%
function Q = GeneratorMarkovChain( P )
% Compute the infinitesimal generator of a continuous time
% Markov chain from a transition of a discrete time Markov chain,
% with P = expm(Q).
%
% Input: P: Transition matrix
%
% Output:
%       Q: infinitesimal generator.

[V, D] = eig( P );

Q = V * diag( log( diag( D ) ) ) * inv( V );

Q = max( Q - diag( diag( Q ) ), 0 );

Q = Q - diag( sum( Q' ) );
end