function DemoCopula()
% Examples of bivariate copulae
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Clayton  family                                 
%   phi(t) = (t^{-\theta}-1)/theta         
%   theta \ge 0.                          
%  ind:  theta = 0, Frechet: theta=infty  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Frank family                                      
%   phi(t) = -log( (1- theta^t)/(1-theta) ) 
%   0 < theta <= 1                          
%   ind: theta = 1, Frechet: theta = 0      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Gumbel family                                     
%   phi(t) = (-log(t))^(1/theta)            
%   0 < theta <= 1                          
%   ind: theta = 1, Frechet: theta = 0      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Ali-Mikhail-Haq family                                 
%   phi(t) = log( (1-theta)/t + theta)/(1-theta)  
%   0 <= theta <= 1                               
%   ind: theta = 0,  ? : theta=1                  

% initialize support
nbPoints    = 20;
u0          = [1:1:(nbPoints-1)]'/nbPoints;
[u,v]       = meshgrid(u0,u0);
[n, m]      = size(u);

%% Independence copula

z0 = u .* v;

figure
surf( u, v, z0 );
title('Independence copula'); 

%% Gaussian copula

x       = norminv(u); 
y       = norminv(v);
rho     = 0.7;
temp    = inv( [1 rho; rho 1] ) - eye(2);

z= exp( - 0.5 * ( temp(1,1) * x.^2 + temp(2,2) * y.^2 + 2 * temp(1,2) * x .* y ) ) / sqrt( 1 - rho^2 );

figure
surf( u, v, z );
title(['Gaussian copula with rho = ',num2str(rho)]); 

%% Cauchy copula

x   = tan( pi * ( u - 0.5 ) ); 
y   = tan( pi * ( v - 0.5 ) );
y1  = y + sqrt( 1 + y .* y );
y2  = x ./ ( sqrt( 1 + y .* y )+ sqrt( 1 + y .* y + x .* x ) );
z   = ( 1 / pi ) * ( atan( y1 ) + atan( y2 ) );

figure
surf( u, v, z );
title('Cauchy copula with rho = 0'); 

%% Farlie-Gumbel-Morgenstern copula 

alpha   = 0.7;
z1      = ( u .* v ) .* ( 1 + alpha * ( 1 - u ) .* ( 1 - v ) );

figure
surf( u, v, z1 );
title(['Farlie-Gumbel-Morgenstern copula with alpha = ',num2str(alpha)]); 

%% Gumbel copula 

alpha   = 0.3;
z2      = phiinv_gumbel( phi_gumbel( u, alpha ) + phi_gumbel( v, alpha ), alpha );

figure
surf( u, v, z2);
title(['Gumbel copula with alpha = ',num2str(alpha)]); 

%% Ali-Mikhail-Haq copula 

theta   = 0.3;
z3      = phiinv_amh( phi_amh( u, theta ) + phi_amh( v, theta ), theta );

figure
surf( u, v, z3);
title(['Ali-Mikhail-Haq copula with theta = ',num2str(theta)]); 

%% Clayton copula 

theta   = 2.0;
z4      = phiinv_clayton( phi_clayton( u, theta ) + phi_clayton( v, theta ), theta );

figure
surf( u, v, z4 );
title(['Clayton copula  with theta = ',num2str(theta)]); 

%% Frank copula 

theta   = 0.3;
z5      = phiinv_frank( phi_frank( u, theta ) + phi_frank( v, theta ), theta );

figure
surf( u, v, z5 );
title(['Frank copula  avec theta = ',num2str(theta)]); 

%% Frchet-Hoeffding upper bound
z8 = min(u,v);

figure
surf( u, v, z8 );
title('Frchet-Hoeffding upper bound'); 

%% Frchet-Hoeffding lower bound

z9 = ( u + v - 1 ) .* ( u + v > 1 );

figure
surf( u, v, z9 );
title('Frchet-Hoeffding lower bound'); 

%% Tent map copula

alpha   = 0.5;
z6      = alpha *u .* v + ( 1 - alpha ) * ( min( u, 0.5 * v ) + ( u + 0.5 * v - 1 ) .* ( u + 0.5 * v > 1 ) );

figure
surf( u, v, z6 );
title(['Tent map copula with alpha = ',num2str(alpha)]); 

%% Tent map copula

alpha   = 1.0;
z7      = alpha *u .* v + ( 1 - alpha ) * ( min( u, 0.5 * v ) + ( u + 0.5 * v - 1 ) .* ( u + 0.5 * v > 1 ) );


figure
surf( u, v, z7 );
title(['Tent map copula with alpha = ',num2str(alpha)]); 

%% Auxiliary functions

%%
function phi = phi_clayton( t, theta )

phi = ( t .^ ( - theta ) - 1 ) / theta;

%%
function phiinv = phiinv_clayton( s, theta )

phiinv = ( 1 + s .* theta ).^ ( - 1 / theta );

%%
function phi = phi_gumbel( t, alpha )

phi = ( - log( t ) ) .^ ( 1 / alpha );

%%
function phiinv = phiinv_gumbel( s, alpha )

phiinv = exp( - s .^ alpha );

%%
function phi = phi_frank( t, theta )

phi = - log( ( 1 - theta .^ t ) / ( 1 - theta ) );

%%
function phiinv = phiinv_frank( s, theta )

phiinv = log( 1 - ( 1 - theta ) * exp( - s ) ) / log( theta );

%%
function phi = phi_amh( t, theta )

phi = log( ( 1 - theta ) ./ t + theta ) / ( 1 - theta );

%%
function phiinv = phiinv_amh( s, theta )

phiinv= ( 1 - theta ) ./ ( exp( s .* ( 1 - theta ) ) - theta );