function out = EstDep( X )
% Estimation of dependence measures for pairs of data.
%  
% Input 
%     X       : data (n x 2 matrix).
%
% Output
%      pseudos: normalized ranks U and pseudo=observations V for Kendall
%                 function;
%      stats  : Kendall tau, Spearman rho, van der Waerden rho, Pearson
%               rho;
%      std    : estimated standard deviations;
%      error  : 95% error for the estimations;
%      pvalue : P-values (%) for each statistic under the null hypothesis of
%              independence.


%X = [1.5,5.3;-2.4,2.6;4.2,-3.4;-6.1,-5.6;-8.9,-9.7]
n = size( X, 1 );

[U, V] = Pseudos( X );

U1 = U(:,1); U2 = U(:,2);

%% Estimate Kendall's tau
tau = 4 * mean(V) - 1;
  
Z = 4 * ( 2 * V - U1 - U2 );
sigma_tau = sqrt( var( Z ) );
error_tau = 1.96 * sigma_tau / sqrt( n );

%% Estimate Spearman's rho
rho_S = corr(U1,U2);

W1 = zeros(n,1);
W2 = zeros(n,1);

for i=1:n
    W1(i) = mean( ( ( U1 >= U1( i ) ) - U1 ) .* ( U2 - 0.5 ) );
    W2(i) = mean( ( ( U2 >= U2( i ) ) - U2 ) .* ( U1 - 0.5 ) ); 
end

Z = 12 * ( ( U1 - 0.5 ) .* ( U2 - 0.5 ) + W1 + W2 );

sigma_rho_S = sqrt( var( Z ) );
error_rho_S = 1.96 * sigma_rho_S / sqrt( n );

%% Estimate van der Waerden's rho
Z1 = norminv( U1 );
Z2 = norminv( U2 );

rho_W = corr(Z1, Z2);

for i=1:n
   W1(i) = mean( Z2 .* ( ( U1 >= U1(i) ) - U1 ) ./ normpdf( Z1 ) );
   W2(i) = mean( Z1 .* ( ( U2 >= U2(i) ) - U2 ) ./ normpdf( Z2 ) );
end

Z = Z1 .* Z2 + W1 + W2;

sigma_rho_W = sqrt( var( Z ) );
error_rho_W = 1.96 * sigma_rho_W / sqrt( n );

%%  Estimation of Pearson rho

rho_P =  corr(X(:,1),X(:,2));
sigma_rho_P = 1-rho_P^2;
error_rho_P = 1.96 * sigma_rho_P / sqrt( n );




%% Computing p-values
pValueFct = @(x) 2 * ( 1 - normcdf( abs( x ) ) );

% Kendall's tau
Z           = 1.5 * sqrt( n ) * tau;
pvalue_tau  = pValueFct( Z );

% Spearman's rho
Z               = sqrt( n ) * rho_S;
pvalue_rho_S    =  pValueFct( Z );

% van der Waerden's rho
Z               = sqrt( n ) * rho_W;
pvalue_rho_W    = pValueFct( Z );

Z               = sqrt( n ) * rho_P;
pvalue_rho_P    = pValueFct( Z );

%% output
out.pseudo  = [U V];
out.stat    = [tau rho_S rho_W rho_P];
out.std     = [ sigma_tau  sigma_rho_S sigma_rho_W sigma_rho_P];
out.error   = [error_tau  error_rho_S  error_rho_W error_rho_P];
out.pvalue  = 100*[pvalue_tau  pvalue_rho_S  pvalue_rho_W pvalue_rho_P];
out.names   = {'Kendall tau', 'Spearman rho', 'van der Waerden rho','Pearson rho'};