function [param,CVM,pvalue,CVMSim,Ehat] = GofRosenCopula(x, model, method, N)
% Goodness-of-fit test for the multivariate copula families Clayton, Frank, 
% Gumbel, Gaussian and Student, based on the Cramer-von Mises statistic SnB 
% or SnC calculated from the Rosenblatt transform. 
% For the Archimedean families, one must have  d=2.
%
% Ref: Genest, Remillard & Beaudoin 2009
%
%   Input
%        x: (n x d matrix);
%        model: 'Clayton', 'Frank', 'Gumbel', 'Gaussian',  or 't' (Student);
%        method: 'B' (SnB)  or 'C' (SnC);
%        N: number of iterations for the computation of the P-value.
%
%   Ouput
%      param.rhohat: estimation of rho (Gaussian or T copulas)
%      param.nuhat: estimation of nu (T copula);
%      param.theta: estimation of theta (Clayton, Frank or Gumbel copulas);
%      CVM: Cramer-von Mises statistic SnB or SnC;
%      pvalue: (%) computed by parametric bootstrap;
%      CVMSim: simulated values of the statistic under H0;
%      Ehat: (n x d matrix) of pseudo-observations.
%       
%
%  By Bruno Remillard (July 29, 2015)
%

[n,d] = size(x);
U = Pseudos(x);

param.nu    = [];
param.rho   = [];
param.theta = [];
param1      = param;



switch model
    
    case 't'
    % Estimation
    [rhohat,nuhat] = copulafit(model,U);
    param.rho = rhohat;
    param.nu  = nuhat;

     % Pseudo-observations
     Ehat = RosenblattStudent(U,rhohat,nuhat);
    %Ehat = Rosenblatt(U,model,param);
     % Statistic
     CVM = CVMStat(Ehat,method);
     
     
     
     %% Parametric bootstrap
     CVMSim = zeros(N,1);
     parfor k=1:N
         y = copularnd(model,rhohat,nuhat,n);
         U = Pseudos(y);
        [rho,nu] = copulafit(model,U);
%         param1.nu = nu;
%         param1.rho = rho;
        
        %E= Rosenblatt(U,model,param1);
        E= RosenblattStudent(U,rho,nu);
        CVMSim(k) = CVMStat(E,method);
     end

      
    case 'Clayton'
       % Estimation
     thetahat = copulafit(model,U);
     param.theta = thetahat;
     
     % Pseudo-observations
     %Ehat = Rosenblatt(U,model,param);
     Ehat = RosenblattClayton(U,thetahat);
     % Statistic
     CVM = CVMStat(Ehat,method);
     
     alphahat = 1/thetahat;
     %% Parametric bootstrap
     CVMSim = zeros(N,1);
     parfor k=1:N
         
        y=SimArchi(model,param.theta,n,d);
         
         U = Pseudos(y);
        theta = copulafit(model,U);
        %param1.theta = theta;
        E = RosenblattClayton(U,theta);
        %E = Rosenblatt(U,model,param1);
        CVMSim(k) = CVMStat(E,method);
     end  
     
     case 'Gaussian'
       % Estimation
     rhohat = copulafit(model,U);
     param.rho = rhohat;
     % Pseudo-observations
     Ehat = RosenblattGaussian(U,rhohat);

     % Statistic
     CVM = CVMStat(Ehat,method);
     
     %% Parametric bootstrap
     CVMSim = zeros(N,1);
     parfor k=1:N
         
         y = copularnd(model,rhohat,n);
         
         U = Pseudos(y);
        rho = copulafit(model,U);
%         param1.rho = rho;
%         E = Rosenblatt(U,model,param1);
          E = RosenblattGaussian(U,rho);
        CVMSim(k) = CVMStat(E,method);
     end  
        
     case 'Gumbel'
       % Estimation
     alphahat = copulafit(model,U);
     param.theta = 1/alphahat;  % parametrization of Matlab different from ours
     
     % Pseudo-observations
     %Ehat = Rosenblatt(U,model,param);
     Ehat = RosenblattGumbel(U,param.theta);
     % Statistic
     CVM = CVMStat(Ehat,method);
     
     
     %% Parametric bootstrap
     CVMSim = zeros(N,1);
      
     parfor k=1:N
         
        y=SimArchi(model,param.theta,n,d);
         
         U = Pseudos(y);
        alpha = copulafit(model,U);
        theta = 1/alpha;
        %param1.theta = theta;
        E = RosenblattGumbel(U,theta);
        %E = Rosenblatt(U,model,param1);
        CVMSim(k) = CVMStat(E,method);
     end  
     
        case 'Frank'
       % Estimation
     alphahat = copulafit(model,U);
     param.theta = exp(-alphahat);  % parametrization of Matlab different from ours
     
     % Pseudo-observations
     %Ehat = Rosenblatt(U,model,param);
     Ehat = RosenblattFrank(U,param.theta);
     % Statistic
     CVM = CVMStat(Ehat,method);
     
     
     %% Parametric bootstrap
     CVMSim = zeros(N,1);
      
     parfor k=1:N
         
         y=SimArchi(model,param.theta,n,d);
         
         U = Pseudos(y);
        alpha = copulafit(model,U);
        theta = exp(-alpha);
        %param1.theta = theta;
        E = RosenblattFrank(U,theta);
        %E = Rosenblatt(U,model,param1);
        CVMSim(k) = CVMStat(E,method);
     end  
        
end   


pvalue = 100*mean(CVMSim>CVM);

fprintf('\n\n Computations done!! \n\n');
    
end



%%

% function  R = Rosenblatt(u,model,param)
% 
% switch model
%     
% case 't'
%     
%    R = RosenblattStudent(u,param.rho,param.nu);
%    
% case 'Gaussian'
%    R = RosenblattGaussian(u,param.rho);  
%    
% case 'Clayton'
%     
%     R = RosenblattClayton(u,param.theta);
% end
% 
% end
%%
function   R = RosenblattGaussian(u,rho)
%  Rosenblatt transform for the Gaussian copula
%  Input
%        U: n x d matrix of pseudos-observations (normalized ranks);
%        rho:  d x d correlation matrix.

%
%  By Bruno Remillard (Feb 10, 2010)


[n,d] = size(u);

R = zeros(n,d);

R(:,1)=u(:,1);

for k=2:d
   R11 = rho(1:k-1,1:k-1);
   R21 = rho(k,1:k-1);
   R11inv = inv(R11);
   B = R21*R11inv;
   tu = norminv(u);
   x = tu(:,1:k-1);
   
  
   y = tu(:,k);
   Omega = 1-B*R11*B';
   mu = x*B';
   
   w = (y-mu)/sqrt(Omega);
   R(:,k) = normcdf(w);
   
    
end

end

%%
function   R = RosenblattStudent(u,rho,nu)
%  Rosenblatt transform for the Student copula
%  Input
%        U: n x d matrix of pseudos-observations (normalized ranks);
%        rho:  d x d correlation matrix;
%        nu: degrees of freedom.
%
%  By Bruno Remillard (Feb 5, 2010)



[n,d] = size(u);

R = zeros(n,d);

R(:,1)=u(:,1);

for k=2:d
   R11 = rho(1:k-1,1:k-1);
   R21 = rho(k,1:k-1);
   R11inv = inv(R11);
   B = R21*R11inv;
   tu = tinv(u,nu);
   x = tu(:,1:k-1);
   
  
   y = tu(:,k);
   Omega = 1-B*R11*B';
   mu = x*B';
   z = diag( nu+x*R11inv*x')/(nu+k-1);
   w = (y-mu)./sqrt(Omega*z);
   R(:,k) = tcdf(w,nu+k-1);
   
    
end

end


%%
function R =  RosenblattClayton(U,theta)
%  Rosenblatt transform for the Clayton copula
%  Input
%        theta: parameter of the Clayton;
%        U : n x d matrix of pseudos-observations (normalized ranks)
%
%  By Bruno Remillard (Feb 11, 2010)



[n,d] =  size(U);

R = zeros(n,d);

R(:,1)=U(:,1);
x = U.^(-theta)-1;

u = 1+cumsum(x,2);

for j=2:d
    
    a = -1/theta-(j-1);
    z1 = (u(:,j)).^a;
    z2 = (u(:,j-1)).^a;
    R(:,j) = z1./z2;
    
end

end

function R =  RosenblattGumbel(U,theta)
%  Rosenblatt transform for the Gumbel copula
%  Input
%        theta: parameter of the Gumbel;
%        U : n x d matrix of pseudos-observations (normalized ranks)
%
%  By Bruno Remillard (Jul 27, 2015)



[n,d] =  size(U);

R = zeros(n,d);

R(:,1)=U(:,1);
x = (-log(U)).^(1/theta);

v = (cumsum(x,2)).^theta; % -log(C)

for j=2:d
    l=j-1;
    a = l/theta;
    t1= v(:,j);
    t2= v(:,l);
    coef =  CoefPolyGumbel(l,theta);
    
    p1 = zeros(n,1);
    p2 = p1;
    for k=1:l
        p1 = coef(k)* t1.^k;
        p2 = coef(k)* t2.^k;
    end
        
    R(:,j) = exp(t2-t1) .* (t2 ./ t1).^a .* (p1 ./ p2);
    
end

end

%%
function [a]  = CoefPolyGumbel(k,theta)

a = zeros(k,1);

for j=1:k
    a(j) = polyGumbel(k,theta,j);
end

end
%%
function coeff = polyGumbel(k,theta,j)
% Computes polynomials associated with the Gumbel copula of parameter theta 
% in [0,1].
%
if k==1
    coeff = theta;
elseif    k==2;
    if j==1
        coeff = theta*(1-theta);
    elseif j==2
        coeff = theta^2;
    end
elseif k>2
    if j==1
        coeff = (k-1-theta)*polyGumbel(k-1,theta,1);
    elseif j==k
        coeff = theta*polyGumbel(k-1,theta,k-1);
    else
        coeff = (k-1-j*theta)*polyGumbel(k-1,theta,j)+theta*polyGumbel(k-1,theta,j-1);
    end
end
        
end
%% 
function R =  RosenblattFrank(U,theta)
%  Rosenblatt transform for the Gumbel copula
%  Input
%        theta: parameter of the Frank;
%        U : n x d matrix of pseudos-observations (normalized ranks)
%
%  By Bruno Remillard (Jul 27, 2015)


alpha = -log(theta); % copulafit matlab parameter

[n,d] =  size(U);

R = zeros(n,d);

R(:,1)=U(:,1);
x = -log(  (1-exp(-alpha*U))/(1-theta));

v = 1 ./ (1-(1-theta)*exp(-cumsum(x,2)))-1; %  exp(-alpha C)

for j=2:d
    l=j-1;
   
    t1= v(:,j);
    t2= v(:,l);
    coef =  CoefPolyFrank(l);
    
    p1 = zeros(n,1);
    p2 = p1;
    for k=1:l
        p1 = coef(k)* t1.^k;
        p2 = coef(k)* t2.^k;
    end
        
    R(:,j) =  p1 ./ p2;
    
end

end

%%
function [a]  = CoefPolyFrank(k)

a = zeros(k,1);

for j=1:k
    a(j) = polyFrank(k,j);
end
end

%%
function coeff = polyFrank(k,j)
% Computes polynomials associated with the Frank copula. 
% 
if k <=2
    coeff = 1;
elseif k>2
    if j==1
        coeff = polyFrank(k-1,1);
    elseif j==k
        coeff = (k-1)*polyFrank(k-1,k-1);
    else
        coeff = j*polyFrank(k-1,j)+ (j-1)*polyFrank(k-1,j-1);
    end
end
        
end

%%
function  stat = CVMStat(E,method)

switch method
    case 'B'
        stat = SnB(E);
    case 'C'
        stat = SnC(E);
end

end

%%
function  Sn = SnB(E)
%  Cramer-von Mises statistic SnB for GOF based on the Rosenblatt transform
%  Ref: Genest, Remillard & Beaudoin 2009
%
%   Input
%        E: n x d matrix of pseudo-observations.
%
%   Output
%        Sn: Cramer-von Mises statistic SnB.

[n,d] = size(E);

Dn = zeros(n,1);
S1 = n/3^d;

G0 = 1-E.*E;
E0 = 1-E;

S2 = sum(prod(G0,2))/2^(d-1);


for i=1:n
    G0 = repmat(E0(i,:),n,1);
    
    Dn(i)= mean( prod( min(G0,E0),2));
end

Sn = sum(Dn)-S2+S1;


end

%%
function  Sn = SnC(E)
%  Cramer-von Mises statistic SnC for GOF based on the Rosenblatt transform
%  Ref: Genest, Remillard & Beaudoin 2009
%
%   Input
%        E: n x d matrix of pseudo-observations.
%
%   Output
%        Sn: Cramer-von Mises statistic SnC.


[n,d] = size(E);

Dn = zeros(n,1);
Cn = prod(E,2);
for i=1:n
    G0 = repmat(E(i,:),n,1);
    Dn(i)=mean( prod( double((E<=G0)),2));
end

dif = Dn-Cn;

Sn = sum(dif.*dif);

end



