function out = IndGRTest(x,N,Sig,graph)
% This function computes the Cramer-von Mises statistics  and the Fisher 
% statistic combining the p-values of the CVM statistics for testing
% independence between d variables, as in Genest & Remillard (2004). All 
% statistics are non-parametric. The p-values are computed by simulation. 
% The Fisher statistic is the optimal one to combine p-values.
%
% The test can also be applied to Rosenblatt transforms of copula-based
% univariate time series, as in Simard & Remillard (2014).
%
% Input
%    x: observations ( n x d matrix) assumed to be i.i.d. (2 <= d <= 6) ;
%    N: Number of samples to generate for estimating the p-value;
%    Sig: Order of quantile, default is 0.95;
%    graph: if 1, the dependogram is computed; the default is 0.
%
% Output 
%   List containing
%
%   cvmStats: Cramer-von Mises statistics;
%   FisherStat: Statistic combining all p-values of Cramer-von Mises
%               statistics, distributed as a chi square;
%   cvmPvalues: Estimated p-values (percentage of rejection of the null 
%               hypothesis of independence) computed with N samples of i.i.d. 
%               uniforms variables;
%   FisherPvalue: p-value (%) of the Fisher statistic;
%   Q: quantiles of the simulated statistics;
%   cvm_sets: list of the subsets of indices.
%
% References
%    [1] Genest, C. & Remillard, B. (2004). Tests of independence or 
%        randomness based on the empirical copula process. Test, 13, 
%        335--369.
%    [2] Simard, C. and Remillard, B. Forecasting time series with
%        multivariate copulas. SSRN.
%
%% Simulation

if nargin < 3
    Sig = 0.95;
    graph = 0;
elseif nargin <4;
    graph = 0;
end

[n,d] = size(x);

if(d<2)
    fprintf('d must be greater than  1 \n');
    return ;
elseif (d>6)
     fprintf('d must be smaller than 7 \n');
    return ;
end

%% Subsets
if d==2
  cvm_sets = {'12'};
elseif d== 3
  cvm_sets = {'12','13','23','123'} ;
elseif d==4
  cvm_sets = {'12','13','14','23','24','34','123','124','134','234','1234'} ;
elseif d==5
  cvm_sets = {'12','13','14','15','23','24','25','34','35','45','123',...
              '124','125','134','135','145','234','235','245','345',...
              '1234','1235','1245','1345','2345','12345'} ;
elseif d==6
   cvm_sets = {'12','13','14','15','16','23','24','25','26','34','35','36',...
               '45','46','56','123','124','125','126','134','135','136','145',...
               '146','156','234','235','236','245','246','256','345','346',...
               '356','456','1234','1235','1236','1245','1246','1256','1345','1346',...
               '1356','1456','2345','2346','2356','2456','3456','12345','12346',...
               '12356','12456','13456','23456','123456'} ; 
end
 

[cvmStats,m] = IndGRCVM(x);




S = zeros(N,m);

parfor k=1:N
    U = rand(n,d);
    S(k,:) = IndGRCVM(U);
end

Q = quantile(S,Sig);
 
   cvmPvalues = mean( ( S>repmat(cvmStats,N,1) ) ); 


FisherStat = -2*sum(log(cvmPvalues)); % Fisher stat

 cvmPvalues = 100* cvmPvalues; % percentage of rejection

FisherPval = 100*(1-chi2cdf(FisherStat,2*m));


%%

out.cvmStats     = cvmStats;
out.FisherStat   = FisherStat;
out.cvmPvalues   = cvmPvalues, 
out.FisherPvalue = FisherPval;
out.cvm_sets     = cvm_sets;
out.quantile     = Q;


%%
if(graph)
ligne = 5*ones(m,1);
figure


lineHandles = stem(cvmPvalues','filled','b-o');
 set(gca,'XLim',[0.95 (m+.05)]);
 set(gca,'XTick',[1:m]);
 set(gca,'XTickLabel',cvm_sets);
 set(lineHandles(1),'MarkerSize',4)
 
  if verLessThan('matlab','8.4')
 % <=R2014a 
   addpath('../Chapter03');
   rotateXLabels( gca(), 90 )% From Matlab Central, by Ben Tordoff.
else
 % >R2014a
  set(gca, 'XTickLabelRotation', 90);
end

   hold on
   
   plot(ligne,'--r')
          
    xlabel('Sets')
    ylabel('P-value (%) of CVM ')
    title('Dependogram of CVM tests')
    
%% printed output
fprintf('\n\n Cramr-von Mises test of independence\n\n');


fprintf('  Subset   Statistic   Pvalue\n\n');
for j=1:m
fprintf('%6s       %4.4f        %3.1f%%\n',cvm_sets{j},cvmStats(j),cvmPvalues(j));
end
fprintf('\n\n');
   
fprintf('Global Fisher test combining %d p-values of the CVM statistics\n\n',m);

fprintf('Fisher Stat: %4.2f    P-value: %3.1f%% \n\n',FisherStat,FisherPval);
    
end
end

% Cramer-von Mises statistics for testing independence between  
% series X(i,1), X(i,2), ...X(i,d), as in Genest & Remillard (2004).
function [cvm,m] = IndGRCVM(x)


[n,d] = size(x);

if d==2
  ind ={ [1 2]} ;
elseif d== 3
  ind = {[1 2],[1 3],[2 3],[1 2 3]} ;
elseif d==4
  ind = {[1 2],[1 3],[1 4],[2 3], [2 4], [3 4], [1 2 3],[1 2 4],[1 3 4],...
        [2 3 4], [1 2 3 4]} ;
elseif d==5
  ind = {[1 2],[1 3],[1 4],[1 5],[2 3],[2 4],[2 5],[3 4],[3 5],[4 5],...
         [1 2 3],[1 2 4],[1 2 5],[1 3 4],[1 3 5],[1 4 5],[2 3 4], [2 3 5], [2 4 5], [3 4 5],...
              [1 2 3 4],[1 2 3 5],[1 2 4 5],[1 3 4 5],[2 3 4 5],[1 2 3 4 5]} ;
elseif d==6
  ind = {[1 2],[1 3],[1 4],[1 5],[1 6],[2 3],[2 4],[2 5],[2 6],[3 4],[3 5],[3 6],...
         [4 5],[4 6],[5 6],[1 2 3],[1 2 4],[1 2 5],[1 2 6],[1 3 4],[1 3 5],[1 3 6],[1 4 5],...
         [1 4 6],[1 5 6],[2 3 4],[2 3 5],[2 3 6],[2 4 5],[2 4 6],[2 5 6],[3 4 5],[3 4 6], [3 5 6], [4 5 6],...
         [1 2 3 4],[1 2 3 5],[1 2 3 6],[1 2 4 5],[1 2 4 6],[1 2 5 6],[1 3 4 5],[1 3 4 6],...
         [1 3 5 6],[1 4 5 6],[2 3 4 5],[2 3 4 6],[2 3 5 6],[2 4 5 6],[3 4 5 6],...
         [1 2 3 4 5],[1 2 3 4 6],[1 2 3 5 6],[1 2 4 5 6],[1 3 4 5 6],...
         [2 3 4 5 6], [1 2 3 4 5 6]} ; 
end

m = 2^d -d-1; %number of statistics



for i=1:n
    y= repmat(x(i,:),n,1);
    v = (x<=y);
    F = repmat(mean(v),n,1);
    vf = sqrt(6)*(v-F);  %to get a mean 1 approx for the cvm stats
    for j=1:m
        ll = ind{j};
        
      D(i,j) = mean ( prod( vf(:,ll) , 2 ) );
    end
end

cvm = sum(D.^2);
end