function out = IndKendallTest( X, N, graph )
% Tests of independence based on Kendall's function
%
% Input
%     X: data (n x d matrix),
%     N: number of iterations for the estimation of p-values,
%     graph: 0 (default) or 1 according of one wants the graph of Kn.
%
% Output
%     stats: Kolmogorov-Smirnov and Cramer-von Mises test statistics,
%     pvalue: P-values (in percent) for the test of independence.
%     quantiles: Empirical quantiles of order 95% and 99% for both statistics.
%
%     For the Kolmogorov-Smirnov test: the 95% and 99% quantiles are 1.14
%     and 2.135 respectively (valid if the sample size is larger than 200).
%
%     For the Cramer-von-Mises test: the 95% and 99% quantiles are 0.21
%     and 0.32 respectively (valid if the sample size is larger than 200).

if nargin <3
    graph = 0;
end

[n, d]  = size( X );
[U, V]  = Pseudos(X);

   V  = sort( V );

   V0 = mean(V==0);
   V1 = V(V>0);
   LV = 1-gamcdf(-log(V1),d,1);
   Kn = EstKendall( X, V1 );
   H  = [V0; Kn-LV];

KS    = sqrt( n ) * max( abs( H ) );
CVM   = sum( H.^2 );

%% Simulated values under H0

simKS = zeros(N,1);
simCVM = simKS;

parfor i = 1: N
   Y = rand(n,d); 
   [Uk, Vk]  = Pseudos(Y);

   Vk  = sort( Vk );

   V0 = mean(Vk==0);
   V1 = Vk(Vk>0);
   LV = 1-gamcdf(-log(V1),d,1);
   Kn = EstKendall( Y, V1 );
   H  = [V0; Kn-LV];

simKS(i)    = sqrt( n ) * max( abs( H ) );
simCVM(i)   = sum( H.^2 );

end

pvalueKS    = 100*mean( simKS > KS );
pvalueCVM   = 100*mean( simCVM > CVM );

out.stat     = [CVM KS];
out.pvalue   = [pvalueCVM  pvalueKS];
out.names    = {'Cramer-von Mises', 'Kolmogorov-Smirnov'};
out.quantiles = [quantile(simCVM,[.95;.99]) quantile(simKS,[.95;.99])];

%% printed output
if graph
    t = (0:0.01:1)';
    [Kn,t] = EstKendall(X,t);
    out.Kn = [Kn,t] ;
    Kind = [0;1-gamcdf(-log(t(2:end)),d,1)];
    error = out.quantiles(1,2)/sqrt(n);
    figure
    plot(t,Kn,'k-')
    hold on
    plot(t,Kind,'b:')
    hold on
    plot(t,Kind-error,'m--')
    hold on
    plot(t,Kind+error,'m--')
    
    legend( {'K_n','K for C_\perp','95% confidence band about K'}, 'location', 'best' );
end




