function [U] = SimArchi(model, theta, n,d)
% Simulate multivariate copulas from the Archimedean families Clayton, Frank or Gumbel 
% using the Marshall-Olkin method.
% d=2.
%
% Ref: Remillard (2013)
%
%   Input
%        model: 'Clayton', 'Frank', 'Gumbel';
%        n: number of vectors;
%        d: dimension;
%        theta: parameter ( theta >0 for Clayton while theta in (0,1) for
%        Frank or Gumbel families).

%
%   Ouput
%      U: n x d matrix 
%       
%
%  By Bruno Remillard (July 29, 2015)
%


  E = -log(rand(n,d));

switch model
    
        
    case 'Clayton'
      alpha = 1/theta;      
      S = repmat(gamrnd(alpha,1,n,1),1,d); 
      V = E ./ S ;
      U = (1 + V).^(-alpha);
 
  
         
     case 'Gumbel'
              
     S = repmat(SimStable(theta,n),1,d); 
     V = E ./ S ;
     U = exp( -(V).^(theta));
    
 
     
      case 'Frank'
      
     alpha = -log(theta);
     S = repmat(SimLogSeries(theta,n),1,d); 
     V = E ./ S;
     U = -(1/alpha) * log( 1 -(1-theta)*exp(-V)  );
       
   
        
end   

U=U;
    
end

%%
function S = SimStable(theta,n)
% Generates n stable variates with index theta in (0,1). See formula on
% page 306.

X  = rand(n,1); 
W = -log(rand(n,1));

S = sin(theta*pi*X) .* (sin(pi*X)).^(-1/theta) .* ( sin((1-theta)*pi*X)./W).^((1-theta)/theta);
end

%%
function S = SimLogSeries(theta,n)
% Generates n Logaritmic Series variates with parameter theta in (0,1). See formula on
% page 307.

alpha = -log(theta);  %Matlab parameter 
W1 = rand(n,1);
W2 = rand(n,1);

S =  floor (1+ log(W2) ./ log(1 - exp(-W1*alpha)));
end

