function out = EstKalman(y,Z0,P0,p,z)
%   Kalman filter  for the model
%
%   Z_i = alpha + phi Z_{i-1} +w_i, w_i \sim N(0,Q),
%   Y_i = Z_i +\epsilon_i, \epsilon_i \sim N(0,R),
%
%
%  INPUT
%     y: observations (n x 1);
%     Z0: Initial mean;
%     P0: Initial variance;
%     p: proportion of observations used for the estimation. If
%     missing, the whole sample is taken.
%     z: signa (if available).
%
%  OUTPUT
%     param: estimated parameters(alpha, phi, Q, R);
%     error: 95% error of estimation;
%     OutofSampleRMSE: if signal is provided, compute the RMSE for the 
%                      errors of prediction for the rest of the sample;
%     InSampleRMSE: if signal is provided, compute the RMSE for the 
%                   errors of prediction for the whole sample (here the
%                   estimations we used the whole sample);
%    prediction: predicted conditional mean;
%    bounds: lower and upper bound for the C95% CI for the predicted values;
%    V: conditional variance of observations;
%    e:prediction of observations;
%    LogLik: log-likelihhod.
%
%
    

n = length( y );

%% Sub-sample for the estimation

if nargin < 4
    p = 1;
    z = [];
elseif nargin <5 
    z=[];
end

 n0 = round( n * p );
% First n0 observations
y0 = y(1:n0);

%% Estimation

options =  optimset(    'largescale',       'off', ...
                        'MaxFunEvals',      10000, ...
                        'MaxIter',          2000, ...
                        'Display',          'iter', ...
                        'diagnostics',      'on');

initialGuess = [ 0; 0; log(.001); log(.001)];        %[ alpha; phi; log(Q) ; log(R)]
                   
                              

[theta, LL, exitflag] = fminunc( @( x ) sum( KalmanFilterEx( x, y0, Z0, P0 ) ), initialGuess, options);

exitflag
alpha      = theta( 1 );
phi        = theta( 2 );
Q          = exp(theta( 3 ) );
R          = exp(theta( 4 ) );

param   = [alpha; phi ; Q; R];


%% Computation of the filter
[L, Zhat, Phat, V, e] = KalmanFilterEx( theta, y, Z0, P0 );

 addpath('../ChapterB')
J = NumJacobian( @(x) KalmanFilterEx( x, y0, Z0, P0 ), theta, 0.001 );
rmpath('../ChapterB')
 
% Estimation of Fisher's information
FI  = cov(J);  

D           = diag( [1; 1; Q; R] );
Covariance  = D * pinv( FI ) * D;
error       = 1.96 * sqrt( diag( Covariance ) / n0 );

t = (1:1:n)';

binf = Zhat - 1.96 * sqrt( Phat );
bsup = Zhat + 1.96 * sqrt( Phat );


%% output
indout          = (n0:n)';
OutofSampleRMSE =[];
if nargin <5
    InSampleRMSE = [];
else
InSampleRMSE    = sqrt( mean( ( Zhat - z ) .^2 ) );
if p<1
OutofSampleRMSE = sqrt( mean( ( Zhat( indout ) - z( indout ) ) .^2 ) );
end
end

out.param.names     = {'alpha', 'phi', 'Q', 'R'};
out.param.values    = param;
out.param.error     = error;
out.prediction      = Zhat;
out.bounds          = [binf bsup];
out.V               = V;
out.e               = e;
out.LogLIK          = -LL;
out.OutofSampleRMSE = OutofSampleRMSE;
out.InSampleRMSE    = InSampleRMSE;


  %% printed output  

figure
plot(t,y,'k')
hold on
plot(t,Zhat,'k--')
hold on
plot(t,binf,'k:')
hold on
plot(t,bsup,'k:')
title({['Kalman filter starting from Z_{0|0} = ',num2str(Z0), ', P_{0|0} = ',num2str(P0), ', and using ', num2str(100*p),' % of the sample for estimation.'];...
  ['Estimated values: \alpha = ' num2str(alpha),', \phi =', num2str(phi), ',Q = ',num2str(Q),', R= ',num2str(R) ,', RMSE = ',num2str(OutofSampleRMSE)]});
legend('Observations','Prediction','Lower bound','Upper bound','location','Best');

if nargin == 5
figure
plot(t(indout),z(indout),'k')
hold on
plot(t(indout),Zhat(indout),'k:')
hold on
plot(t(indout),binf(indout),'k-.')
hold on
plot(t(indout),bsup(indout),'k-.')
title({['Kalman filter starting from Z_{0|0} = ',num2str(Z0), ', P_{0|0} = ',num2str(P0), ', and using ', num2str(100*p),' % of the sample for estimation.'];...
  ['Estimated values: \alpha = ' num2str(alpha),', \phi =', num2str(phi), ',Q = ',num2str(Q),', R= ',num2str(R) ,', RMSE = ',num2str(OutofSampleRMSE)]});

legend('Signal','Prediction','Lower bound','Upper bound','location','Best');
end

figure
plot(t(indout),Zhat(indout),'k')
hold on
plot(t(indout),binf(indout),'k-.')
hold on
plot(t(indout),bsup(indout),'k-.')
title({['Kalman filter starting from Z_{0|0} = ',num2str(Z0), ', P_{0|0} = ',num2str(P0), ', and using ', num2str(100*p),' % of the sample for estimation.'];...
    ['Estimated values: \alpha = ' num2str(alpha),', \phi =', num2str(phi), ',Q = ',num2str(Q),', R= ',num2str(R) ,', RMSE = ',num2str(OutofSampleRMSE)]});
legend('Prediction','Lower bound','Upper bound','location','Best');
end
%%
function [L, Z, P, V, e] = KalmanFilterEx( param, y, Z0, P0)
% Log-densities for the Kalman filter model 
%   Z_i = alpha + phi Z_{i-1} + w_i, w_i \sim N(0,Q),
%   Y_i = Z_i +\epsilon_i, \epsilon_i \sim N(0,R),
%
% and computation of the Kalman filter equations.

%% Transformation of parameters
Q      = exp(param( 3 ) );
R      = exp(param( 4 ) );
phi    = param( 2 );
alpha  = param( 1 );

n = length( y );

%% Filter initialization

P = zeros(n,1);
Z = P;
e = P;
V = P;
L = P;

P( 1 ) = R * ( phi^2 * P0 + Q ) / (  phi^2 *  P0 + Q + R );
e( 1 ) = y( 1 ) - phi * Z0 - alpha;
Z( 1 ) =  phi * Z0 + alpha + ( P( 1 ) / R ) * e( 1 );
V( 1 ) =  phi^2 * P0 + Q + R;

L( 1 ) = 0.5 * ( log( V( 1 ) ) + e( 1 ) * e( 1 ) / V( 1 ) +log(2*pi) ) ;
for ii = 2:n
    
    P( ii ) = R * (  phi^2 * P( ii-1 ) + Q  ) / ( phi^2 *  P( ii-1 ) + Q + R );
    e( ii ) = y( ii ) -  phi * Z( ii-1 ) - alpha;
    Z( ii ) = alpha + phi *  Z( ii-1 ) + e( ii ) * P( ii ) / R;
    V( ii ) =  phi^2 * P( ii-1 ) + Q + R;
    L( ii ) = 0.5 * ( log( V( ii ) ) + e( ii ) * e( ii ) / V( ii ) +log(2*pi) );
    
end
end
