function out = EstKalmanBS(data,h,Z0,P0,p)
%   Kalman filter  for the model
%
%   Z_i = alpha + Z_{i-1} +w_i, w_i \sim N(0,Q),
%   Y_i = Z_i +\epsilon_i, \epsilon_i \sim N(0,R),
%
%   Here alpha = h(mu-Q/2), Q = sigma^2/2, Y = log(closing price), Z =
%   log(asset value).
%
%   INPUT
%         data: adjusted prices;
%         h: time  between observations;
%         Z0: Initial mean;
%         P0: Initial variance;
%         p: proportion of observations used for the estimation. If
%         missing, the whole sample is taken.
%
%   OUTPUT
%         param: estimated parameters(mu,sigma,sigma_obs);
%         error: 95% error of estimation;
%         Zhat: predicted conditional mean;
%         bounds: lower and upper bound for the C95% CI for the predicted 
%                 values;
%         V: conditional variance of observations;
%         e:prediction of observations;
%         LogLik: log-likelihhod.
%
    

y = log(data); %  log of closing price


n = length( y );

%% Sub-sample for the estimation

if nargin < 5
    p = 1;
end

 n0 = round( n * p );
% First n0 observations
y0 = y(1:n0);

%% Estimation

options =  optimset(    'largescale',       'off', ...
                        'MaxFunEvals',      10000, ...
                        'MaxIter',          2000, ...
                        'Display',          'iter', ...
                        'diagnostics',      'on');

initialGuess = [ 0; ...          % mu    
                 log(1); ...   % log(sigma)  
                 log(0.01)];      % log(sqrt(R))
                              

[theta, LL] = fminunc( @( x ) sum( KalmanFilterEx( x, y0, h, Z0, P0 ) ), initialGuess, options);

mu      = theta( 1 );
sigma   = exp( theta( 2 ) );
illiq   = exp( theta( 3 ) );

param   = [mu; sigma; illiq];


%% Computation of the filter
[L, Zhat, Phat, V, e] = KalmanFilterEx( theta, y, h, Z0, P0 );



 addpath('../ChapterB')
J = NumJacobian( @(x) KalmanFilterEx( x, y0,h, Z0, P0 ), theta, 0.001 );
rmpath('../ChapterB')
 
% Estimation of Fisher's information
FI  = cov(J);  

D           = diag( [1; sigma; illiq] );
Covariance  = D * inv( FI ) * D;
error       = 1.96 * sqrt( diag( Covariance ) / n0 );

%% output
out.param.names   = {'mu', 'sigma', 'illiq'};
out.param.values = param;
out.param.error = error;
out.Zhat = Zhat;
out.Phat = Phat;
out.V    = V;
out.e    = e;
out.LogLIK = -LL;



indout          = (n0:n)';

t = (1:1:n)';

binf = Zhat - 1.96 * sqrt( Phat );
bsup = Zhat + 1.96 * sqrt( Phat );

%% output
out.param.names     = {'mu', 'sigma', 'illiq'};
out.param.values    = param;
out.param.error     = error;
out.prediction      = Zhat;
out.bounds          = [binf bsup];
out.V               = V;
out.e               = e;
out.LogLIK          = -LL;

%% printed output
figure
plot(t,exp(y),'g')
hold on
plot(t,exp(Zhat),'--')
hold on
plot(t, exp( binf),':')
hold on
plot(t, exp(bsup),':')
title({['Kalman filter for the Black-Scholes model.'];...
    ['Starting points: Z_{0|0} = ',num2str(Z0), ', P_{0|0} = ',num2str(P0), ', and using ', num2str(100*p),' % of the sample for estimation.'];...
    ['Estimated values: \mu = ' num2str(mu),', \sigma =',num2str(sigma),', \sigma_{obs}= ',num2str(illiq) ]});
legend('Closing price','Predicted price','Lower bound','Upper bound',0);


figure;
plot(t,exp(Zhat),'--')
hold on
plot(t,exp(binf),':')
hold on
plot(t, exp(bsup),':')

title({['Kalman filter for the Black-Scholes model.'];...
    ['Starting points: Z_{0|0} = ',num2str(Z0), ', P_{0|0} = ',num2str(P0), ', and using ', num2str(100*p),' % of the sample for estimation.'];...
    ['Estimated values: \mu = ' num2str(mu),', \sigma =',num2str(sigma),', \sigma_{obs}= ',num2str(illiq) ]});
legend('Predicted price','Lower bound','Upper bound',0);

%%%%%%%%%% Log-prices
figure
plot(t,y,'k')
hold on
plot(t,Zhat,'k--')
hold on
plot(t,binf,'k:')
hold on
plot(t,bsup,'k:')
title({['Kalman filter for the Black-Scholes model.'];...
    ['Starting points: Z_{0|0} = ',num2str(Z0), ', P_{0|0} = ',num2str(P0), ', and using ', num2str(100*p),' % of the sample for estimation.'];...
    ['Estimated values: \mu = ' num2str(mu),', \sigma =',num2str(sigma),', \sigma_{obs}= ',num2str(illiq) ]});
legend('Closing log-price','Predicted log-price','Lower bound','Upper bound',0);

figure
plot(t,Zhat,'--')
hold on
plot(t,binf,':')
hold on
plot(t,bsup,':')
title({['Kalman filter for the Black-Scholes model.'];...
    ['Starting points: Z_{0|0} = ',num2str(Z0), ', P_{0|0} = ',num2str(P0), ', and using ', num2str(100*p),' % of the sample for estimation.'];...
    ['Estimated values: \mu = ' num2str(mu),', \sigma =',num2str(sigma),', \sigma_{obs}= ',num2str(illiq) ]});
legend('Predicted log-price','Lower bound','Upper bound',0);

end
%%
function [L, Z, P, V, e] = KalmanFilterEx( param, y, h, Z0, P0)
% Log-densities for the Kalman filter model 
%   Z_i = alpha + Z_{i-1} + w_i, w_i \sim N(0,Q),
%   Y_i = Z_i +\epsilon_i, \epsilon_i \sim N(0,R),
%
% and computation of the Kalman filter equations.

%% Transformation of parameters
Q   = h*exp(2* param( 2 ) );
R   = h*exp(2* param( 3 ) );
alpha  = param( 1 )*h - 0.5*Q;

n = length( y );

%% Filter initialization

P = zeros(n,1);
Z = P;
e = P;
V = P;
L = P;

P( 1 ) = R * ( P0 + Q ) / ( P0 + Q + R );
e( 1 ) = y( 1 ) - Z0 - alpha;
Z( 1 ) = Z0 + alpha + ( P( 1 ) / R ) * e( 1 );
V( 1 ) = P0 + Q + R;

L( 1 ) = 0.5 * ( log( V( 1 ) ) + e( 1 ) * e( 1 ) / V( 1 ) +log(2*pi) ) ;
for ii = 2:n
    
    P( ii ) = R * ( P( ii-1 ) + Q  ) / ( P( ii-1 ) + Q + R );
    e( ii ) = y( ii ) - Z( ii-1 ) - alpha;
    Z( ii ) = alpha + Z( ii-1 ) + e( ii ) * P( ii ) / R;
    V( ii ) = P( ii-1 ) + Q + R;
    L( ii ) = 0.5 * ( log( V( ii ) ) + e( ii ) * e( ii ) / V( ii ) +log(2*pi) );
    
end
end
