function out = EstKalmanSV( y , p , z)
%   Kalman filter  for the stochastic volatility model of Ruiz (1994):
%
%   Z_i = omega + phi Z_{i-1} +w_i, w_i \sim N(0,sigma^2),
%   Y_i = \epsilon_i x exp(Z_i/2), \epsilon_i \sim N(0,1).
%
%
%   INPUT
%         y: series;
%         p: proportion of the observations for the estimation of parameters. 
%            If missing, the whole sample is taken;
%         z: signal (if available).
%
%   
%  OUTPUT
%     param: estimation of omega, phi and sigma;
%     error: 95% error of estimation;
%     OutofSampleRMSE: if signal is provided, compute the RMSE for the 
%                      errors of prediction for the rest of the sample;
%     InSampleRMSE: if signal is provided, compute the RMSE for the 
%                   errors of prediction for the whole sample (here the
%                   estimations we used the whole sample);
%    Zhat: predicted conditional mean (best linear prediction);
%    Phat: predicted conditional variance;
%    V: conditional variance of observations;
%    e:prediction of observations;
%    LogLik: log-likelihhod.
%    
%% Load data


n = length( y );

%% Sub-sample for the estimation

if nargin < 2
    p = 1;
    z = [];
elseif nargin <3 
    z=[];
end

n0 = floor( n * p );


% First n0 observations
y0 = y(1:n0);

%% Estimation

options =  optimset(    'largescale',       'off', ...
                        'MaxFunEvals',      100000, ...
                        'MaxIter',          1000, ...
                        'Display',          'iter', ...
                        'diagnostics',      'on');

initialGuess = [    -0.3; 
                    atanh(0.95); 
                    log(.5)];

[theta, LL, exitflag] = fminunc( @(x) sum( KalmanFilterSV( x, y0 ) ), initialGuess, options);

omega   = theta( 1 );
phi     = tanh( theta( 2 ) );
sigma   = exp( theta( 3 ) );

param = [omega; phi; sigma];

%% Computation of the filter for the whole sample

[L, Zhat, Phat, V, e] = KalmanFilterSV( theta, y );

 addpath('../ChapterB')
 
J = NumJacobian( @( x ) KalmanFilterSV( x, y0 ), theta, 0.001 );

H = mean( NumHessian( @(x) KalmanFilterSV( x, y0 ), theta, 0.001), 3 );

 rmpath('../ChapterB')
 
 
FI          = cov( J );  
D           = diag( [ 1; 1 - phi^2; sigma ] );
H1          = inv( H );
Covariance  = D * H1 * FI * H1 * D;
error       = 1.96 * sqrt( diag( Covariance ) / n0 );
 
indout          = (n0:n)';
OutofSampleRMSE =[];
if nargin <3
    InSampleRMSE = [];
else
InSampleRMSE    = sqrt( mean( ( Zhat - z ) .^2 ) );
if p<1
OutofSampleRMSE = sqrt( mean( ( Zhat( indout ) - z( indout ) ) .^2 ) );
end
end

t = (1:1:n)';

binf = Zhat - 1.96 * sqrt( Phat );
bsup = Zhat + 1.96 * sqrt( Phat );

%% output
out.param.names     = {'omega', 'phi', 'sigma'};
out.param.values    = param;
out.param.error     = error;
out.prediction      = Zhat;
out.bounds          = [binf bsup]; 
out.LogLIK          = -LL;
out.OutofSampleRMSE = OutofSampleRMSE;
out.InSampleRMSE    = InSampleRMSE;

%% printed utput
figure
plot(t,log(y.^2),'g')
hold on
plot(t,Zhat,'--')
plot(t, binf,':')
plot(t, bsup,':')
hold off

title({['Kalman filter using the first ', num2str(n0),...
    ' observations for estimation'];...
     ['Estimated values: \omega = ' num2str(omega),', \phi =', num2str(phi), ' ,\sigma = ',num2str(sigma), ', RMSE = ',num2str(OutofSampleRMSE)]});
legend( {'Observations','Predictions','Lower bound','Upper bound'}, 'location', 'Best' );

if nargin == 3
figure
plot(t(indout),z(indout),'k')
hold on
plot(t(indout),Zhat(indout),'k:')
hold on
plot(t(indout),binf(indout),'k-.')
hold on
plot(t(indout),bsup(indout),'k-.')
title({['Kalman filter using the first ', num2str(n0),' observations for estimation'];...
     ['Estimated values: \omega = ' num2str(omega),', \phi =', num2str(phi), ' ,\sigma = ',num2str(sigma), ', RMSE = ',num2str(OutofSampleRMSE)]});
 legend('Signal','Prediction','Lower bound','Upper bound','location','Best');
end


figure;
plot(t,Zhat,'--')
plot(t, binf,':')
plot(t, bsup,':')
hold off

title({['Kalman filter using the first ', num2str(n0),...
    ' observations for estimation'];...
     ['Estimated values: \omega = ' num2str(omega),', \phi =', num2str(phi), ' ,\sigma = ',num2str(sigma), ', RMSE = ',num2str(OutofSampleRMSE)]});
legend( {'Prediction','Lower bound','Upper bound'}, 'location', 'Best' );

end

%%

function [LL, Z, P, V, e]= KalmanFilterSV( theta, y )
% Log-densities of the Kalman filter for the stochastic volatility model 
%   Z_i = omega + phi Z_{i-1} +w_i, w_i \sim N(0,sigma^2),
%   log(Y_i^2) = Z_i + log(\epsilon_i^2), \epsilon_i \sim N(0,1),
%
% and computation of the Kalman equations.
%                                                                         

%% Transformation of parameters

omega   = theta( 1 );
phi     = tanh( theta( 2 ) );
sigma   = exp( theta( 3 ) );

Q   = sigma^2;
R   = pi^2 / 2;
n   = length( y );
Z0  = omega / ( 1 - phi );
P0  = Q / ( 1 - phi^2 );

y   = log(y.^2) + 1.2704;

%% Filter initialization

LL = zeros(n,1);
P = LL;
Z = LL;
e = LL;
V = LL;

Pi      = phi^2 * P0 + Q;
V(1)    = Pi + R;
K       = Pi / V( 1 );
P( 1 )  = Pi * ( 1 - K ); 
Zi      = omega + phi * Z0;
e(1)    = y(1) - Zi;
Z(1)    = Zi + K * e( 1 );

LL(1) = 0.5 * ( log( V( 1 ) ) + e( 1 ) * e( 1 ) / V( 1 ) );

for ii = 2:n
    
    Pi      = phi^2 * P( ii-1 ) + Q;
    V( ii ) = Pi + R;
    K       = Pi / V( ii );
    P( ii ) = Pi * ( 1 - K );
    Zi      = omega + phi * Z( ii-1 );
    e( ii ) = y( ii ) - Zi;
    Z( ii ) = Zi + e( ii ) * K;
    
    LL( ii ) = 0.5 * ( log( V( ii ) ) + e( ii ) * e( ii ) / V( ii ) );
end
end