function out = EstSPF(y,N,p,z)
% Smooth particle filter for the model 
%        Z_i = alpha + phi Z_{i-1}+w_i, w_i \sim N(0,Q)
%        Y_i = Z_i + eps_i, eps_i \sim N(0,R)
%
%  INPUT
%         y: series;
%         N: number of particles;
%         p: proportion of the observations for the estimation of parameters. 
%            If missing, the whole sample is taken;
%         z: signal (if available).

%  OUTPUT
%     param: estimated parameters(alpha, phi, Q, R);
%     error: 95% error of estimation;
%     prediction: predicted conditional mean;
%     bounds: lower and upper bound for the C95% CI for the predicted values;
%     LogLik: log-likelihhod.
%     OutofSampleRMSE: if signal is provided, compute the RMSE for the 
%                      errors of prediction for the rest of the sample;
%     InSampleRMSE: if signal is provided, compute the RMSE for the 
%                   errors of prediction for the whole sample (here the
%                   estimations we used the whole sample);
%


    
if nargin < 3
    p = 1;
    z = [];
elseif nargin <4
    z=[];
end


n = length(y);
n0 = floor(n*p);
noise = randn(N,n+1);
unif = rand(1,n+1);

y0 = y(1:n0);


initialGuess = [0;0;log(.1 );log(.1)];

 

options         = optimset('largescale','on','MaxFunEvals',10000,'MaxIter',2000,'Display','iter');



%% OPTIMIZATION
%[theta, f, exitflag, output] = fminsearch( @(x) SmoothParticleFilter(x,N,y0,noise,unif), initialGuess, options );
[theta, LL, exitflag, output, grad, hessian] = fminunc( @(x) sum(SmoothParticleFilter(x,N,y0,noise,unif)), initialGuess, options );



% remove persistent variables
clear functions

[L,Zhat,bounds] = SmoothParticleFilter(theta,N,y,noise,unif);

alpha =  theta(1); 
phi   =  theta(2); 
Q     =  exp(theta(3));
R     =  exp(theta(4));


param = [alpha; phi; Q; R];
 addpath('../ChapterB')

J = NumJacobian(@(x) SmoothParticleFilter(x,N,y,noise,unif),theta,0.001);

 rmpath('../ChapterB')

FI = cov(J);

D = diag([1;1;Q;R]);
Covariance  =  D*pinv(FI)*D;

error = 1.96*sqrt( diag(Covariance)/n0);

t = (1:1:n)';

binf = bounds(:,1);
bsup = bounds(:,2);



%% output
indout          = (n0:n)';
OutofSampleRMSE =[];
if nargin <4
    InSampleRMSE = [];
else
InSampleRMSE    = sqrt( mean( ( Zhat - z ) .^2 ) );
if p<1
OutofSampleRMSE = sqrt( mean( ( Zhat( indout ) - z( indout ) ) .^2 ) );
end
end

out.param.names     = {'alpha', 'phi', 'Q', 'R'};
out.param.values    = param;
out.param.error     = error;
out.prediction      = Zhat;
out.bounds          = bounds; 
out.LogLIK          = -LL;
out.OutofSampleRMSE = OutofSampleRMSE;
out.InSampleRMSE    = InSampleRMSE;


%% printed output
figure
plot(t,y,'k')
hold on
plot(t,Zhat,'k--')
hold on
plot(t,binf,'k:')
hold on
plot(t,bsup,'k:')
title({['Particle filter  with N = ', num2str(N), ' particles and using ', num2str(100*p),' % of the sample for estimation.'];...
    ['Estimated values: \alpha = ' num2str(alpha),', \phi =', num2str(phi), ',Q = ',num2str(Q),', R= ',num2str(R) ,', RMSE = ',num2str(OutofSampleRMSE)]});
 legend('Observations','Prediction','Lower bound','Upper bound','location','Best');

if nargin == 4
figure
plot(t(indout),z(indout),'k')
hold on
plot(t(indout),Zhat(indout),'k:')
hold on
plot(t(indout),binf(indout),'k-.')
hold on
plot(t(indout),bsup(indout),'k-.')
title({['Particle filter  with N = ', num2str(N), ' particles and using ', num2str(100*p),' % of the sample for estimation.'];...
     ['Estimated values: \alpha = ' num2str(alpha),', \phi =', num2str(phi), ',Q = ',num2str(Q),', R= ',num2str(R) ,', RMSE = ',num2str(OutofSampleRMSE)]});
 legend('Signal','Prediction','Lower bound','Upper bound','location','Best');
end

figure
plot(t(indout),Zhat(indout),'k')
hold on
plot(t(indout),binf(indout),'k-.')
hold on
plot(t(indout),bsup(indout),'k-.')
title({['Particle filter  with N = ', num2str(N), ' particles and using ', num2str(100*p),' % of the sample for estimation.'];...
    ['Estimated values: \alpha = ' num2str(alpha),', \phi =', num2str(phi), ',Q = ',num2str(Q),', R= ',num2str(R) ,', RMSE = ',num2str(OutofSampleRMSE)]});
legend('Prediction','Lower bound','Upper bound','location','Best');

end
%%
  function [LL,Zhat,bounds] = SmoothParticleFilter(theta,N,y,noise,unif)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%   Inputs:  theta: parameters [log(Q);log(R);mu],
%            N: number of particles,
%            Y: obervations,
%            noise: innovations (N x n),
%            unif: uniform variates for smooth sampling (N x n).
%
%  Outputs: LL: vector of log-conditional densities,
%           Zhat: vector of predictions of the  mean,
%           Phat: vector of predictions of the  variance.
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



alpha =  theta(1); 
phi   =  theta(2); 
Q     =  exp(theta(3));
R     =  exp(theta(4));
  
n = length(y);

X0 = alpha*ones(N,1); % initial particles 


Zhat   = zeros(n,1);
bounds = zeros(n,2);
LL     = zeros(n,1);


for i=1:n
  
    %evolution step
  
  X10 = sort( alpha + phi*X0+ sqrt(Q)*noise(:,i) );  
  
  
 
   lik = exp(-0.5*((y(i)-X10).^2)/R)/sqrt(R);
   lik = max(lik,1E-25);
   
  
   
  prob = lik/sum(lik); % computation of the likelihood weights

 
 
   X0 =PittResampling(N,unif(i),X10,prob);
   
 
  LL(i) =  - log(mean(lik));
  Zhat(i) = mean(X0);
  
  bounds(i,:) = quantile(X0,[.025 .975]) ;
end

  end
  
%%

function boot = PittResampling( M, unif, y, prob )
% Implementation of Algorithm 2 in Pitt (2002)
% See also Gentil & Remillard (2008).

R           = length( y );
[y, index]  = sort( y );
prob        = prob( index );
r           = zeros(M,1);
v           = r;

w           = 0.5 * ( prob(1:R-1) + prob(2:R) );
w( 1 )      = w( 1 ) + 0.5 * prob( 1 );
w( R-1 )    = w( R-1 ) + 0.5 * prob( R );

u           = OrderedUnif( M, unif );

s   = 0;
j   = 1;
for ii = 1:R-1
    s = s + w( ii );
    while (j <= M) && (u( j ) <= s)
        r( j )  = ii;
        v( j )  = 1 + ( u( j ) - s ) / w( ii );
        j       = j + 1;
    end
end
boot = ( y( r(1:M) + 1 ) - y( r(1:M) ) ) .* v( 1:M ) + y( r(1:M) );


end

%%
function u = OrderedUnif( M, unif )
% Implementation of Algorithm 1 of Pitt % (2002) 
% Generates n ordered uniform variates based on one single uniform.

u = (0:M-1)' / M + unif / M;
end
      

 


