function out = EstSPFSV(y, N, p, z )
%  Smooth particle filter for the model of Ruiz (1994):
%
%   Z_i = omega + phi Z_{i-1} +w_i, w_i \sim N(0,sigma^2),
%   Y_i = \epsilon_i x exp(Z_i/2), \epsilon_i \sim N(0,1).
%
%  INPUT
%         y: series;
%         N: number of particles;
%         p: proportion of the observations for the estimation of parameters. 
%            If missing, the whole sample is taken;
%         z: signal (if available).

%  OUTPUT
%      param: estimation of omega, phi and sigma;
%      error: estimation errors (95%) for the parameters;
%      prediction: prediction of the conditional mean.  
%      bounds: bounds: lower and upper bound for the C95% CI for the 
%                      predicted values;
%      OutofSampleRMSE: if signal is provided, compute the RMSE for the 
%                      errors of prediction for the rest of the sample;
%      InSampleRMSE: if signal is provided, compute the RMSE for the 
%                   errors of prediction for the whole sample (here the
%                   estimations we used the whole sample);
%      LogLik: log-likelihhod.
%
%
    

    


%%
n       = length( y );


%% Sub-sample for the estimation

if nargin < 3
    p = 1;
    z = [];
elseif nargin <4
    z=[];
end
    n0 = floor( n * p );


% First n0 observations
y0 = y(1:n0);




noise   = randn( N, n );
unif    = rand( n, 1 );

options =  optimset(    'largescale',       'on', ...
                        'MaxFunEvals',      1000, ...
                        'MaxIter',          200, ...
                        'Display',          'iter');
                    
initialGuess = [    -0.3; ...
                    atanh(0.95); ...
                    log(.5)];

y0 = y(1:n0);

%% OPTIMIZATION
[theta, LL, exitflag, output] = fminunc( @(x) sum( SmoothParticleFilterSV( x, N, y0, noise, unif ) ), initialGuess, options );

[L, Zhat, bounds] = SmoothParticleFilterSV( theta, N, y, noise, unif );

omega   = theta( 1 ); 
phi     = tanh( theta( 2 ) );
sigma   = exp( theta( 3 ) );

param   = [ omega; phi; sigma ];

 addpath('../ChapterB')
 
J = NumJacobian( @(x) SmoothParticleFilterSV(x,N,y0,noise,unif), theta, 0.001 );

 rmpath('../ChapterB')
% Estimation of Fisher's information
FI = cov(J);

D           = diag( [ 1; 1 - phi^2; sigma ] );
Covariance  = D * pinv( FI ) * D;
error       = 1.96 * sqrt( diag( Covariance ) / n0 );


t    = (1:1:n)';

indout          = (n0:n)';
OutofSampleRMSE =[];
if nargin <4
    InSampleRMSE = [];
else
InSampleRMSE    = sqrt( mean( ( Zhat - z ) .^2 ) );
if p<1
OutofSampleRMSE = sqrt( mean( ( Zhat( indout ) - z( indout ) ) .^2 ) );
end
end



%% output
out.param.names     = {'omega', 'phi', 'sigma'};
out.param.values    = param;
out.param.error     = error;
out.prediction      = Zhat;
out.bounds          = bounds; 
out.LogLIK          = -LL;
out.OutofSampleRMSE = OutofSampleRMSE;
out.InSampleRMSE    = InSampleRMSE;

binf = bounds(:,1)
bsup = bounds(:,2)

%% printed output
figure
plot(t,y,'k')
hold on
plot(t,Zhat,'k--')
hold on
plot(t,binf,'k:')
hold on
plot(t,bsup,'k:')
title({['Particle filter  with N = ', num2str(N), ' particles and using ', num2str(100*p),' % of the sample for estimation.'];...
    ['Estimated values: \omega = ' num2str(omega),', \phi =', num2str(phi), ' ,\sigma = ',num2str(sigma), ', RMSE = ',num2str(OutofSampleRMSE)]});
 legend('Observations','Prediction','Lower bound','Upper bound','location','Best');

if nargin == 4
figure
plot(t(indout),z(indout),'k')
hold on
plot(t(indout),Zhat(indout),'k:')
hold on
plot(t(indout),binf(indout),'k-.')
hold on
plot(t(indout),bsup(indout),'k-.')
title({['Particle filter  with N = ', num2str(N), ' particles and using ', num2str(100*p),' % of the sample for estimation.'];...
    ['Estimated values: \omega = ' num2str(omega),', \phi =', num2str(phi), ' ,\sigma = ',num2str(sigma), ', RMSE = ',num2str(OutofSampleRMSE)]});
 legend('Signal','Prediction','Lower bound','Upper bound','location','Best');
end

figure
plot(t(indout),Zhat(indout),'k')
hold on
plot(t(indout),binf(indout),'k-.')
hold on
plot(t(indout),bsup(indout),'k-.')
title({['Particle filter  with N = ', num2str(N), ' particles and using ', num2str(100*p),' % of the sample for estimation.'];...
   ['Estimated values: \omega = ' num2str(omega),', \phi =', num2str(phi), ' ,\sigma = ',num2str(sigma), ', RMSE = ',num2str(OutofSampleRMSE)]});
legend('Prediction','Lower bound','Upper bound','location','Best');

end
%%
function [LL, Zhat, bounds] = SmoothParticleFilterSV( theta, N, y, noise, unif )
%   Input
%          theta: parameters [omega, atanh(phi), log(sigma)];
%          N: number of particles;
%          y: observations (n x 1);
%          noise: innovations (N x n);
%          unif: uniform variates for smooth sampling (N x n).
%
%  Output  
%          LL: vector of log-conditional densities;
%          Zhat: vector of predictions.
%

omega   = theta( 1 );
phi     = tanh( theta( 2 ) );  % -1 < phi < 1
sigma   = exp( theta( 3 ) );  % sigma >0

n       = length( y );

% initial particles
X0 = omega / ( 1 - phi ) * ones( N, 1 );

Zhat    = zeros(n,1);
LL      = Zhat;
bounds = zeros(n,2); 
for ii = 1:n
    
    % evolution step
    X10 = sort( omega + phi * X0 + sigma * noise(:,ii) );
    
    % transition step
    h   = exp( -0.5 * X10 );
    lik = h .* exp( -0.5 * ( y( ii ) * h ) .^2 );
    lik = max(lik,1E-25);
    
    % computation of the likelihood weights    
    prob = lik / sum( lik ); 
    
    X0 = PittResampling( N, unif( ii ), X10, prob );
    
    LL( ii )     = - log( mean( lik ) );
    Zhat( ii )   = mean( X0 );
    bounds( ii,: ) = quantile(X0,[.025 .975]);
    
end
end

%%
function boot = PittResampling( M, unif, y, prob )
% Implementation of Algorithm 2 in Pitt (2002)
% See also Gentil & Remillard (2008).

R           = length( y );
[y, index]  = sort( y );
prob        = prob( index );
r           = zeros(M,1);
v           = r;

w           = 0.5 * ( prob(1:R-1) + prob(2:R) );
w( 1 )      = w( 1 ) + 0.5 * prob( 1 );
w( R-1 )    = w( R-1 ) + 0.5 * prob( R );

u           = OrderedUnif( M, unif );

s   = 0;
j   = 1;
for ii = 1:R-1
    s = s + w( ii );
    while (j <= M) && (u( j ) <= s)
        r( j )  = ii;
        v( j )  = 1 + ( u( j ) - s ) / w( ii );
        j       = j + 1;
    end
end
boot = ( y( r(1:M) + 1 ) - y( r(1:M) ) ) .* v( 1:M ) + y( r(1:M) );


end

%%
function u = OrderedUnif( M, unif )
% Implementation of Algorithm 1 of Pitt % (2002) 
% Generates n ordered uniform variates based on one single uniform.

u = (0:M-1)' / M + unif / M;
end