function [F A B] = FuturesSchwartz(s,delta,tau,r,kappa,alpha,sigma1,sigma2,rho,lambda)
% Computes the value of a futures on a commodity according to Schwartz (1997) 
% model 2.
%
%   X = log(S);
%   dX = (mu - sigma1^2/2 - delta)dt +sigma1 dW1,
%   ddelta = kappa(alpha-delta)dt+sigma2 dW2,
%
%
% INPUT
%     s: spot price of the commodity (can be a vector of length n);
%     delta: convenience yield (can be a vector of length n);
%     tau: time to maturity (can be a vector of length n);
%     r: risk-free rate (can be a vector of length n);
%     sigma1: volatility of spot price;
%     sigma2: volatility of convenience yield;
%     rho: correlation between Brownian motions;
%     alpha: long-term average of convenience yield;
%     kappa: mean-return speed;
%     lambda: market price of convenience yield risk.
%
% OUTPUT
%     F: futures price.
%
% Exemple: F = FuturesSchwartz(100,0.25,[0:1:252]/252,.05, 1.876, .106, .393, .527, .766, .198) 
% 
a = alpha - lambda/kappa; % long-term average under risk neutral measure

B = (1-exp(-kappa*tau))/kappa;
v = sigma2^2/kappa;
cov =  rho*sigma1*sigma2;

A = tau * ( r - a + 0.5 * v/kappa - cov/kappa) + 0.25 * v * ( 1-exp(-2*kappa*tau))/kappa^2 + (a*kappa +cov -v)*B/kappa;

%K = r*tau + + (0.5 * v - a * kappa - cov)*(tau - B)/kappa - 0.25*v *B.^2;

F = s .* (exp( A  - delta .* B));
 
