function [s delta t] = SimSchwartz(s0,delta0, n, h, mu,kappa,alpha,sigma1,sigma2,rho,graph)
% Computes the value of a futures on a commodity according to Schwartz (1997) 
% model 2.
%
%   X = log(S);
%   dX = (mu - sigma1^2/2 - delta)dt +sigma1 dW1,
%   ddelta = kappa(alpha-delta)dt+sigma2 dW2,
%
%
% INPUT
%     s: initial value of the spot price of the commodity;
%     delta: initial value of the convenience yield;
%     n: number of ''observations'';
%     h: time between ''observations'';
%     mu: average value of spot price;
%     sigma1: volatility of spot price;
%     sigma2: volatility of convenience yield;
%     rho: correlation between Brownian motions;
%     alpha: long-term average of convenience yield;
%     kappa: mean-return speed;
%     graph: 1 if graph needed, 0 otherwise (default).
%
% OUTPUT
%     F: futures price.
%
% Exemple: [s delta t]  = SimSchwartz(16,0.025, 250, 1/252, .142, 1.876, .106, .393, .527, .766, 1) 
%

if nargin < 11
    graph = 0;
end

Q = zeros(2,2);

B   = ( 1-exp( -kappa * h ) )/kappa;
mu2 = alpha * kappa * B ;
mu1 = h * (mu - 0.5 * sigma1^2 -alpha ) + alpha * B ;
phi = exp( -kappa * h ) ;

s = zeros(n+1,1);
delta = s;
ls = s;

v = sigma2^2/kappa;
cov =  rho*sigma1*sigma2;


Q(1,2) =  cov * B - 0.5 * ( sigma2 * B )^2 ;

Q(2,1) = Q(1,2);

Q(2,2) = 0.5 * v * ( 1 - phi^2 );

Q(1,1) = sigma1^2 * h + v * (h - 2 * B + 0.5 * (1-phi^2) / kappa  )/kappa - 2 * cov * ( h - B ) / kappa;

Z = randn(n,2);

a = chol(Q);

w = Z * a;

delta(1) = delta0;
ls(1)     = log(s0);

for i=1:n
    delta(i+1) = mu2  + phi * delta(i)         + w(i,2);
    ls(i+1)    = mu1  + ls(i) - B * delta(i)   + w(i,1);
end

t = (0:1:n)'*h;
s = exp( ls );

%%
if (graph)
figure
plot(t,s);
title('Spot price')

figure
plot(t,delta);
title('Convenience yield')
end