function [s delta t] = SimSchwartzApprox(s0,delta0, n, h, mu,kappa,alpha,sigma1,sigma2,rho,graph)
% Computes the value of a futures on a commodity according to Schwartz (1997) 
% model 2.
%
%   X = log(S);
%   dX = (mu - sigma1^2/2 - delta)dt +sigma1 dW1,
%   ddelta = kappa(alpha-delta)dt+sigma2 dW2,
%
%
% INPUT
%     s: initial value of the spot price of the commodity;
%     delta: initial value of the convenience yield;
%     n: number of ''observations'';
%     h: time between ''observations'';
%     mu: average value of spot price;
%     sigma1: volatility of spot price;
%     sigma2: volatility of convenience yield;
%     rho: correlation between Brownian motions;
%     alpha: long-term average of convenience yield;
%     kappa: mean-return speed;
%     graph: 1 if graph needed, 0 otherwise (default).
%
% OUTPUT
%     F: futures price.
%
% Exemple: [s delta t]  = SimSchwartzApprox(16,0.025, 250, 1/252, .142, 1.876, .106, .393, .527, .766, 1) 
%

if nargin < 11
    graph = 0;
end

Q = zeros(2,2);


mu2 = alpha * kappa * h ;
mu1 = h * (mu - 0.5 * sigma1^2) ;
phi = 1 - kappa * h  ;

s = zeros(n+1,1);
delta = s;
ls = s;

cov =  rho*sigma1*sigma2;


Z = sqrt(h) * randn(n,2);

a = [ sigma1  rho*sigma2; 0  sigma2*sqrt(1-rho^2)];

w = Z * a;

delta(1) = delta0;
ls(1)    = log(s0);

for i=1:n
    delta(i+1) = mu2  + phi * delta(i)         + w(i,2);
    ls(i+1)    = mu1  + ls(i) - h * delta(i)   + w(i,1);
end

t = (0:1:n)'*h;
s = exp( ls );


%%
if (graph)
figure
plot(t,s);
title('Spot price')

figure
plot(t,delta);
title('Convenience yield')
end