function [mu, sigma, error] = EstCauchy(data)
% This functions estimates by maximum likelihood the location and scaling 
% parameters mu and sigma of a Cauchy distribution.
%
% Input
%        data: dataset (n x 1) 
%
% Output
%      mu = theta(1); location parameter;
%     sigma =  exp(theta(2)); 
%
% Example:
% [mu, sigma, error] = EstCauchy(data);

n = length(data);
initialGuess = [0; log(10)];  % Initial values of mu and log(sigma)

[theta,f] = fminsearch( @(x) sum(LogLikCauchy(x, data)), initialGuess);

mu     = theta(1);
sigma  = exp(theta(2));

J = NumJacobian(@(x)LogLikCauchy(x,data),theta,0.001);

FI = cov(J);  %Estimation of Fisher's information

D = diag([1;sigma]);
Covariance = D*pinv(FI)*D;


fprintf('\n 95 percent confidence intervals for mu and sigma:\n\n');

error = 1.96*sqrt( diag(Covariance)/n);

fprintf('    mu = %4.4f  +- %4.4f    \n',mu, error(1));
fprintf(' sigma = %4.4f  +- %4.4f    \n',sigma, error(2));


fprintf('\n\n Observations in DataCauchy.mat were simulated with mu = 10 and sigma = 5.\n');

end


function LL = LogLikCauchy(theta, data)
% This function computes minus the log-likelihood densities
% for a Cauchy distribution with parameters mu and sigma.

mu         = theta(1);
sigma      = exp(theta(2));

LL = log(sigma) +  log( 1 + ( ( data - mu ) / sigma ) .^ 2 );
end


 